@@ -7,20 +7,16 @@
 
 
 #include "db_crl.h"
+#include "db_x509.h"
+#include "main.h"
 #include "exception.h"
-#include "widgets/MainWindow.h"
-#include "widgets/CrlDetail.h"
-#include "widgets/NewCrl.h"
+#include "database_model.h"
 #include <QMessageBox>
 #include <QContextMenuEvent>
-#include "widgets/XcaDialog.h"
 #include "widgets/ItemCombo.h"
-#include "ui_NewCrl.h"
 
-db_crl::db_crl(MainWindow *mw)
-	:db_x509name(mw)
+db_crl::db_crl() : db_x509name("crls")
 {
-	class_name = "crls";
 	sqlHashTable = "crls";
 	pkitype << revocation;
 	pkitype_depends << x509;
@@ -55,13 +51,16 @@ void db_crl::load()
 
 void db_crl::revokeCerts(pki_crl *crl)
 {
+	db_x509 *certs = Database.model<db_x509>();
 	x509revList revlist;
 
-	if (!mainwin->certs)
+	if (!certs || !crl)
 		return;
+
 	pki_x509 *signer = crl->getIssuer();
 	if (!signer)
 		return;
+
 	revlist = crl->getRevList();
 	signer->mergeRevList(revlist);
 	foreach(x509rev revok, revlist) {
@@ -73,7 +72,7 @@ void db_crl::revokeCerts(pki_crl *crl)
 
 void db_crl::removeSigner(pki_base *signer)
 {
-	FOR_ALL_pki(crl, pki_crl) {
+	foreach(pki_crl *crl, Store.getAll<pki_crl>()) {
 		if (crl->getIssuer() == signer) {
 			crl->setIssuer(NULL);
 		}
@@ -82,11 +81,11 @@ void db_crl::removeSigner(pki_base *signer)
 
 void db_crl::inToCont(pki_base *pki)
 {
-	pki_crl *crl = static_cast<pki_crl *>(pki);
+	pki_crl *crl = dynamic_cast<pki_crl *>(pki);
 	unsigned hash = crl->getSubject().hashNum();
 	QList<pki_x509 *> items;
 
-	items = sqlSELECTpki<pki_x509>(
+	items = Store.sqlSELECTpki<pki_x509>(
 		"SELECT x509super.item FROM x509super "
 		"JOIN certs ON certs.item = x509super.item "
 		"WHERE x509super.subj_hash=? AND certs.ca=1",
@@ -100,7 +99,7 @@ void db_crl::inToCont(pki_base *pki)
 
 pki_base *db_crl::insert(pki_base *item)
 {
-	pki_crl *crl = static_cast<pki_crl *>(item);
+	pki_crl *crl = dynamic_cast<pki_crl *>(item);
 	pki_crl *oldcrl = dynamic_cast<pki_crl *>(getByReference(crl));
 	if (oldcrl) {
 		XCA_INFO(tr("The revocation list already exists in the database as:\n'%1'\nand so it was not imported").arg(oldcrl->getIntName()));
@@ -109,55 +108,27 @@ pki_base *db_crl::insert(pki_base *item)
 	}
 	Transaction;
 	if (TransBegin()) {
-		insertPKI(crl);
-		revokeCerts(crl);
-		TransCommit();
+		crl = dynamic_cast<pki_crl *>(insertPKI(crl));
+		if (crl) {
+			revokeCerts(crl);
+			TransCommit();
+		}
 	}
 	return crl;
 }
 
-void db_crl::showPki(pki_base *pki)
-{
-	pki_crl *crl = dynamic_cast<pki_crl *>(pki);
-	if (!crl)
-		return;
-	CrlDetail *dlg = new CrlDetail(mainwin);
-	if (!dlg)
-		return;
-
-	dlg->setCrl(crl);
-	connect(dlg->issuerIntName, SIGNAL(doubleClicked(QString)),
-		mainwin->certs, SLOT(showItem(QString)));
-	connect(mainwin->certs, SIGNAL(pkiChanged(pki_base*)),
-		dlg, SLOT(itemChanged(pki_base*)));
-	if (dlg->exec()) {
-		QString newname = dlg->descr->text();
-		QString newcomment = dlg->comment->toPlainText();
-		if (newname != pki->getIntName() ||
-		    newcomment != pki->getComment())
-		{
-			updateItem(pki, newname, newcomment);
-		}
-	}
-	delete dlg;
-}
-
 void db_crl::store(QModelIndex index)
 {
-	QList<exportType> types;
+	pki_crl *crl = fromIndex<pki_crl>(index);
 
-	if (!index.isValid())
+	if (!index.isValid() || !crl)
 		return;
 
-	pki_crl *crl = static_cast<pki_crl*>(index.internalPointer());
-	if (!crl)
-		return;
-
-	types <<
+	QList<exportType> types; types <<
 		exportType(exportType::PEM, "pem", "PEM") <<
 		exportType(exportType::DER, "der", "DER") <<
 		exportType(exportType::vcalendar, "ics", "vCalendar");
-	ExportDialog *dlg = new ExportDialog(mainwin,
+	ExportDialog *dlg = new ExportDialog(NULL,
 			tr("Revocation list export"),
 			tr("CRL ( *.pem *.der *.crl )"), crl,
 			QPixmap(":revImg"), types);
@@ -176,89 +147,16 @@ void db_crl::store(QModelIndex index)
 		}
 	}
 	catch (errorEx &err) {
-		mainwin->Error(err);
+		XCA_ERROR(err);
 	}
 	pki_base::pem_comment = false;
 	delete dlg;
 }
 
-#if 0
-void db_crl::updateRevocations(pki_x509 *cert)
+pki_crl *db_crl::newCrl(const crljob &task)
 {
-	x509name issname = cert->getSubject();
-	x509revList revlist;
-	pki_crl *latest = NULL;
-
-	FOR_ALL_pki(crl, pki_crl) {
-		if (!(issname == crl->getSubject()))
-			continue;
-		pki_key *key = cert->getPubKey();
-		if (!key)
-			continue;
-		if (!crl->verify(key)) {
-			delete key;
-			continue;
-		}
-		delete key;
-		pki_x509 *old = crl->getIssuer();
-		if (!old) {
-			crl->setIssuer(cert);
-		} else if (old != cert) {
-			if (old->getNotAfter() < cert->getNotAfter())
-				crl->setIssuer(cert);
-		}
-		if (!latest || (latest->getCrlNumber() < crl->getCrlNumber()))
-			latest = crl;
-	}
-	if (latest) {
-		revlist = latest->getRevList();
-		cert->mergeRevList(revlist);
-		cert->setCrlNumber(latest->getCrlNumber());
-	}
-}
-#endif
-
-void db_crl::newItem()
-{
-	QList<pki_x509 *> cas = mainwin->certs->getAllIssuers();
-	pki_x509 *ca = NULL;
-
-	switch (cas.size()) {
-	case 0:
-		XCA_INFO(tr("There are no CA certificates for CRL generation"));
-		return;
-	case 1:
-		ca = cas[0];
-		break;
-	default: {
-		itemComboCert *c = new itemComboCert(NULL);
-		XcaDialog *d = new XcaDialog(mainwin, revocation, c,
-			tr("Select CA certificate"), QString());
-		c->insertPkiItems(cas);
-		if (!d->exec()) {
-			delete d;
-			return;
-		}
-		ca = c->currentPkiItem();
-		delete d;
-		}
-	}
-	newItem(ca);
-}
-
-void db_crl::newItem(pki_x509 *cert)
-{
-	if (!cert)
-		return;
-
 	pki_crl *crl = NULL;
-	NewCrl *widget = new NewCrl(NULL, cert);
-	XcaDialog *dlg = new XcaDialog(mainwin, revocation, widget,
-					tr("Create CRL"), QString());
-	if (!dlg->exec()) {
-		delete dlg;
-		return;
-	}
+	pki_x509 *cert = task.issuer;
 	QSqlDatabase db = QSqlDatabase::database();
 	try {
 		x509v3ext e;
@@ -271,38 +169,35 @@ void db_crl::newItem(pki_x509 *cert)
 		crl->createCrl(cert->getIntName(), cert);
 		crl->pkiSource = generated;
 
-		bool withReason = widget->revocationReasons->isChecked();
 		foreach(x509rev rev, cert->getRevList())
-			crl->addRev(rev, withReason);
+			crl->addRev(rev, task.withReason);
 
-		if (widget->authKeyId->isChecked()) {
+		if (task.authKeyId) {
 			crl->addV3ext(e.create(NID_authority_key_identifier,
 				"keyid,issuer", &ext_ctx));
 		}
-		if (widget->subAltName->isChecked()) {
+		if (task.subAltName) {
 			if (cert->hasExtension(NID_subject_alt_name)) {
 				crl->addV3ext(e.create(NID_issuer_alt_name,
 					"issuer:copy", &ext_ctx));
 			}
 		}
-		if (widget->setCrlNumber->isChecked()) {
-			a1int num;
-			num.setDec(widget->crlNumber->text());
-			crl->setCrlNumber(num);
-			cert->setCrlNumber(num);
+		if (task.setCrlNumber) {
+			crl->setCrlNumber(task.crlNumber);
+			cert->setCrlNumber(task.crlNumber);
 		}
 		crl->setIssuer(cert);
-		crl->setLastUpdate(widget->lastUpdate->getDate());
-		crl->setNextUpdate(widget->nextUpdate->getDate());
-		crl->sign(cert->getRefKey(), widget->hashAlgo->currentHash());
+		crl->setLastUpdate(task.lastUpdate);
+		crl->setNextUpdate(task.nextUpdate);
+		crl->sign(cert->getRefKey(), task.hashAlgo);
 
 		Transaction;
 		if (!TransBegin())
 			throw errorEx(tr("Failed to initiate DB transaction"));
-		cert->setCrlExpire(widget->nextUpdate->getDate());
+		cert->setCrlExpire(task.nextUpdate);
 		SQL_PREPARE(q, "UPDATE authority set crlNo=?, crlExpire=? WHERE item=?");
 		q.bindValue(0, (uint)cert->getCrlNumber().getLong());
-		q.bindValue(1, widget->nextUpdate->getDate().toPlain());
+		q.bindValue(1, task.nextUpdate.toPlain());
 		q.bindValue(2, cert->getSqlItemId());
 		AffectedItems(cert->getSqlItemId());
 		q.exec();
@@ -317,19 +212,17 @@ void db_crl::newItem(pki_x509 *cert)
 		err = q.lastError();
 		if (err.isValid())
 			throw errorEx(tr("Database error: %1").arg(err.text()));
-		insertPKI(crl);
+		crl = dynamic_cast<pki_crl *>(insertPKI(crl));
 		err = db.lastError();
 		if (err.isValid())
 			throw errorEx(tr("Database error: %1").arg(err.text()));
 		TransCommit();
-		createSuccess((crl));
+		createSuccess(crl);
 	}
 	catch (errorEx &err) {
-		MainWindow::Error(err);
-		if (crl)
-			delete crl;
+		XCA_ERROR(err);
+		delete crl;
 		crl = NULL;
 	}
-	delete dlg;
-	return;
+	return crl;
 }
