@@ -8,11 +8,8 @@
 #ifndef __DB_CRL_H
 #define __DB_CRL_H
 
-#include "db_x509.h"
+#include "db_x509super.h"
 #include "pki_crl.h"
-#include "widgets/ExportDialog.h"
-#include <QObject>
-#include <QContextMenuEvent>
 
 class db_crl: public db_x509name
 {
@@ -20,7 +17,7 @@ class db_crl: public db_x509name
 	protected:
 		dbheaderList getHeaders();
 	public:
-		db_crl(MainWindow *mw);
+		db_crl();
 		pki_base *newPKI(enum pki_type type);
 		void revokeCerts(pki_crl *crl);
 		void inToCont(pki_base *pki);
@@ -28,12 +25,7 @@ class db_crl: public db_x509name
 		void removeSigner(pki_base *signer);
 		void store(QModelIndex index);
 		void load();
-		void showPki(pki_base *pki);
 		void updateCertView();
-
-	public slots:
-		void newItem();
-		void newItem(pki_x509 *cert);
+		pki_crl *newCrl(const crljob &crljob);
 };
-
 #endif
