@@ -10,8 +10,6 @@
 
 #include "db_base.h"
 #include "pki_key.h"
-#include <QStringList>
-#include <QObject>
 
 class MainWindow;
 class QModelIndex;
@@ -25,7 +23,7 @@ class db_key: public db_base
 		virtual dbheaderList getHeaders();
 		exportType::etype clipboardFormat(QModelIndexList indexes) const;
 	public:
-		db_key(MainWindow *mw);
+		db_key();
 		QList<pki_key*> getUnusedKeys();
 		QList<pki_key*> getAllKeys();
 		pki_base *newPKI(enum pki_type type = none);
@@ -34,13 +32,11 @@ class db_key: public db_base
 		pki_base* insert(pki_base *item);
 		void setOwnPass(QModelIndex idx, enum pki_key::passType);
 		void loadContainer();
+		pki_key *newKey(const keyjob &task, const QString &name);
 
 	public slots:
-		void newItem();
-		void newItem(QString name);
 		void load();
 		void store(QModelIndex index);
-		void showPki(pki_base *pki);
 
 	signals:
 		void delKey(pki_key *delkey);
