@@ -8,21 +8,17 @@
 
 #include "db_temp.h"
 #include "func.h"
-#include <widgets/NewX509.h>
-#include <widgets/XcaDialog.h>
-#include <widgets/MainWindow.h>
+#include "widgets/XcaWarning.h"
+#include "widgets/NewX509.h"
 #include <QFileDialog>
 #include <QDir>
 #include <QContextMenuEvent>
 #include <QAction>
 #include <QInputDialog>
-#include <QMessageBox>
 #include <QFileInfo>
 
-db_temp::db_temp(MainWindow *mw)
-	:db_x509name(mw)
+db_temp::db_temp() : db_x509name("templates")
 {
-	class_name = "templates";
 	sqlHashTable = "templates";
 	pkitype << tmpl;
 
@@ -68,56 +64,35 @@ pki_base *db_temp::newPKI(enum pki_type type)
 	return new pki_temp("");
 }
 
-QList<pki_temp *> db_temp::getAllAndPredefs()
+QList<pki_temp *> db_temp::getPredefs() const
 {
-	return predefs + getAll<pki_temp>();
+	return predefs;
 }
 
-bool db_temp::runTempDlg(pki_temp *temp)
+bool db_temp::alterTemp(pki_temp *temp)
 {
-	NewX509 *dlg = new NewX509(mainwin);
-	emit connNewX509(dlg);
+	XSqlQuery q;
+	QSqlError e;
 
-	dlg->setTemp(temp);
-	if (!dlg->exec()) {
-		delete dlg;
+	Transaction;
+	if (!TransBegin())
+		return false;
+	SQL_PREPARE(q, "UPDATE templates SET version=?, template=? WHERE item=?");
+	q.bindValue(0, TMPL_VERSION);
+	q.bindValue(1, temp->toB64Data());
+	q.bindValue(2, temp->getSqlItemId());
+	q.exec();
+	e = q.lastError();
+	XCA_SQLERROR(e);
+	if (e.isValid()) {
+		TransRollback();
 		return false;
 	}
-	dlg->toTemplate(temp);
-	delete dlg;
+	updateItem(temp, temp->getIntName(), temp->getComment());
+	TransCommit();
 	return true;
 }
 
-void db_temp::newItem()
-{
-	pki_temp *temp = NULL;
-	QString type;
-
-	itemComboTemp *ic = new itemComboTemp(NULL);
-	ic->insertPkiItems(predefs);
-	XcaDialog *dlg = new XcaDialog(mainwin, tmpl, ic,
-				tr("Preset Template values"), QString());
-	if (dlg->exec()) {
-		temp = new pki_temp(ic->currentPkiItem());
-		if (temp) {
-			temp->pkiSource = generated;
-			if (runTempDlg(temp)) {
-				insertPKI(temp);
-				createSuccess(temp);
-			} else {
-				delete temp;
-			}
-		}
-	}
-	delete dlg;
-}
-void db_temp::showPki(pki_base *pki)
-{
-	pki_temp *t = dynamic_cast<pki_temp *>(pki);
-	if (t)
-		alterTemp(t);
-}
-
 void db_temp::load()
 {
 	load_temp l;
@@ -126,14 +101,14 @@ void db_temp::load()
 
 void db_temp::store(QModelIndex index)
 {
-	if (!index.isValid())
-		return;
+	pki_temp *temp = fromIndex<pki_temp>(index);
 
-	pki_temp *temp = static_cast<pki_temp*>(index.internalPointer());
+	if (!index.isValid() || !temp)
+		return;
 
 	QString fn = Settings["workingdir"] +
 		temp->getUnderlinedName() + ".xca";
-	QString s = QFileDialog::getSaveFileName(mainwin,
+	QString s = QFileDialog::getSaveFileName(NULL,
 		tr("Save template as"),	fn,
 		tr("XCA templates ( *.xca );; All files ( * )"));
 	if (s.isEmpty())
@@ -146,33 +121,6 @@ void db_temp::store(QModelIndex index)
 		temp->writeTemp(file);
 	}
 	catch (errorEx &err) {
-		MainWindow::Error(err);
+		XCA_ERROR(err);
 	}
 }
-
-bool db_temp::alterTemp(pki_temp *temp)
-{
-	XSqlQuery q;
-	QSqlError e;
-
-	if (!runTempDlg(temp))
-		return false;
-
-	Transaction;
-	if (!TransBegin())
-		return false;
-	SQL_PREPARE(q, "UPDATE templates SET version=?, template=? WHERE item=?");
-	q.bindValue(0, TMPL_VERSION);
-	q.bindValue(1, temp->toB64Data());
-	q.bindValue(2, temp->getSqlItemId());
-	q.exec();
-	e = q.lastError();
-	mainwin->dbSqlError(e);
-	if (e.isValid()) {
-		TransRollback();
-		return false;
-	}
-	updateItem(temp, temp->getIntName(), temp->getComment());
-	TransCommit();
-	return true;
-}
