@@ -9,11 +9,9 @@
 #ifndef __DB_TEMP_H
 #define __DB_TEMP_H
 
-#include "db_base.h"
-#include "pki_temp.h"
 #include "db_x509super.h"
-#include <QObject>
-#include <QPixmap>
+
+class pki_temp;
 
 class db_temp: public db_x509name
 {
@@ -22,16 +20,13 @@ class db_temp: public db_x509name
 	QList<pki_temp*> predefs;
 
     public:
-	db_temp(MainWindow *mw);
+	db_temp();
 	~db_temp();
 	pki_base *newPKI(enum pki_type type = none);
-	bool runTempDlg(pki_temp *temp);
-	bool alterTemp(pki_temp *temp);
 	void fillContextMenu(QMenu *menu, const QModelIndex &index);
-	QList<pki_temp*> getAllAndPredefs();
-	void newItem();
-	void showPki(pki_base *pki);
+	QList<pki_temp*> getPredefs() const;
 	void load();
 	void store(QModelIndex index);
+	bool alterTemp(pki_temp *temp);
 };
 #endif
