@@ -6,16 +6,26 @@
  */
 
 #include "db_x509.h"
+#include "pki_x509.h"
+#include "pki_crl.h"
+#include "pki_temp.h"
 #include "pki_pkcs12.h"
 #include "pki_pkcs7.h"
 #include "pki_evp.h"
 #include "pki_scard.h"
 #include "pass_info.h"
+#include "database_model.h"
+#include "entropy.h"
+
+#include "widgets/XcaWarning.h"
 #include "widgets/CertExtend.h"
 #include "widgets/ExportDialog.h"
 #include "widgets/MainWindow.h"
 #include "widgets/PwDialog.h"
 #include "widgets/RevocationList.h"
+#include "widgets/NewX509.h"
+#include "widgets/Help.h"
+
 #include "ui_CaProperties.h"
 #include <QMessageBox>
 #include <QContextMenuEvent>
@@ -23,12 +33,8 @@
 
 #include <openssl/rand.h>
 
-bool db_x509::treeview = true;
-
-db_x509::db_x509(MainWindow *mw)
-	:db_x509super(mw)
+db_x509::db_x509() : db_x509super("certificates")
 {
-	class_name = "certificates";
 	sqlHashTable = "certs";
 	pkitype << x509;
 	pkitype_depends << x509_req;
@@ -42,9 +48,9 @@ void db_x509::loadContainer()
 
 	XSqlQuery q("SELECT item, issuer FROM certs WHERE issuer is NOT NULL");
 	while (q.next()) {
-		pki_base *root = rootItem;
-		pki_x509 *cert = lookupPki<pki_x509>(q.value(0));
-		pki_x509 *issuer = lookupPki<pki_x509>(q.value(1));
+		pki_base *root = treeItem;
+		pki_x509 *cert = Store.lookupPki<pki_x509>(q.value(0));
+		pki_x509 *issuer = Store.lookupPki<pki_x509>(q.value(1));
 		if (cert && issuer) {
 			cert->setSigner(issuer);
 			if (cert != issuer)
@@ -54,8 +60,7 @@ void db_x509::loadContainer()
 			qDebug() << "MOVE" << cert->getIntName()
 				<< "from" << cert->getParent()->getIntName()
 				<< "to" << root->getIntName();
-			cert->getParent()->takeChild(cert);
-			insertChild(root, cert);
+			insertChild(cert, root);
 		}
 	}
 	emit columnsContentChanged();
@@ -88,11 +93,13 @@ pki_base *db_x509::newPKI(enum pki_type type)
 QList<pki_x509 *> db_x509::getAllIssuers()
 {
 	/* Select X509 CA certificates with available private key */
-	return sqlSELECTpki<pki_x509>("SELECT x509super.item FROM x509super "
+	return Store.sqlSELECTpki<pki_x509>(
+		"SELECT x509super.item FROM x509super "
 		"JOIN private_keys ON x509super.pkey = private_keys.item "
 		"JOIN certs ON certs.item = x509super.item "
 		"WHERE certs.ca=1") +
-		sqlSELECTpki<pki_x509>("SELECT x509super.item FROM x509super "
+		Store.sqlSELECTpki<pki_x509>(
+		"SELECT x509super.item FROM x509super "
 		"JOIN tokens ON x509super.pkey = tokens.item "
 		"JOIN certs ON certs.item = x509super.item "
 		"WHERE certs.ca=1");
@@ -100,17 +107,18 @@ QList<pki_x509 *> db_x509::getAllIssuers()
 
 void db_x509::remFromCont(const QModelIndex &idx)
 {
+	db_crl *crls = Database.model<db_crl>();
 	db_x509super::remFromCont(idx);
-	pki_base *pki = static_cast<pki_base*>(idx.internalPointer());
+	pki_base *pki = fromIndex(idx);
 	pki_x509 *child;
 	pki_base *new_parent;
 	QList<pki_x509 *> childs;
 
 	while (pki->childCount()) {
-		child = dynamic_cast<pki_x509*>(pki->childItems.takeFirst());
+		child = dynamic_cast<pki_x509*>(pki->takeFirst());
 		child->delSigner(dynamic_cast<pki_x509*>(pki));
 		new_parent = child->findIssuer();
-		insertChild(new_parent, child);
+		insertChild(child);
 		if (new_parent)
 			childs << child;
 	}
@@ -122,45 +130,7 @@ void db_x509::remFromCont(const QModelIndex &idx)
 		AffectedItems(child->getSqlItemId());
 		q.exec();
 	}
-	mainwin->crls->removeSigner(pki);
-}
-
-void db_x509::changeView()
-{
-	pki_base *temproot;
-	int rows = rowCount(QModelIndex());
-
-	if (!rows)
-		return;
-
-	temproot = new pki_base();
-	beginRemoveRows(QModelIndex(), 0, rows -1);
-	pki_base *pki = rootItem;
-	pki_base *parent;
-	while (pki->childCount()) {
-		pki = pki->takeFirst();
-		while (pki != rootItem && !pki->childCount()) {
-			parent = pki->getParent();
-			temproot->append(pki);
-			pki = parent;
-		}
-	}
-	endRemoveRows();
-
-	treeview = !treeview;
-	if (treeview)
-		mainwin->BNviewState->setText(tr("Plain View"));
-	else
-		mainwin->BNviewState->setText(tr("Tree View"));
-
-	while ((temproot->childCount())) {
-		pki_base *parent = rootItem;
-		pki = temproot->takeFirst();
-		if (treeview)
-			parent = static_cast<pki_x509*>(pki)->getSigner();
-		insertChild(parent, pki);
-	}
-	delete temproot;
+	crls->removeSigner(pki);
 }
 
 static bool recursiveSigning(pki_x509 *cert, pki_x509 *client)
@@ -181,13 +151,11 @@ static bool recursiveSigning(pki_x509 *cert, pki_x509 *client)
 
 void db_x509::inToCont(pki_base *pki)
 {
-	pki_x509 *cert = static_cast<pki_x509*>(pki);
+	pki_x509 *cert = dynamic_cast<pki_x509*>(pki);
 	cert->setParent(NULL);
 	pki_base *root = cert->getSigner();
-	if (!treeview || root == cert || root == NULL)
-		root = rootItem;
 
-	insertChild(root, cert);
+	insertChild(cert, root);
 
 	QList<pki_x509 *> childs;
 	QList<pki_x509 *> items;
@@ -197,7 +165,7 @@ void db_x509::inToCont(pki_base *pki)
 
 	/* Search for another certificate (name and key)
 	 * and use its childs if we are newer */
-	items = sqlSELECTpki<pki_x509>(
+	items = Store.sqlSELECTpki<pki_x509>(
 		"SELECT x509super.item FROM x509super "
 		"JOIN certs ON certs.item = x509super.item "
 		"WHERE certs.ca=1 AND x509super.subj_hash=? "
@@ -210,17 +178,19 @@ void db_x509::inToCont(pki_base *pki)
 			continue;
 		if (cert->getNotAfter() < other->getNotAfter())
 			continue;
-		foreach(pki_base *b, other->childItems) {
-			pki_x509 *child = static_cast<pki_x509*>(b);
+		foreach(pki_base *b, other->getChildItems()) {
+			pki_x509 *child = dynamic_cast<pki_x509*>(b);
+			if (!child)
+				continue;
 			child->delSigner(other);
 			childs << child;
 		}
 		revList.merge(other->getRevList());
 	}
 	/* Search rootItem childs, whether they are ours */
-	foreach(pki_base *b, rootItem->childItems) {
-		pki_x509 *child = static_cast<pki_x509*>(b);
-		if (child == cert || child->getSigner() == child)
+	foreach(pki_base *b, rootItem->getChildItems()) {
+		pki_x509 *child = dynamic_cast<pki_x509*>(b);
+		if (!child || child == cert || child->getSigner() == child)
 			continue;
 		if (child->verify_only(cert))
 			childs << child;
@@ -231,22 +201,15 @@ void db_x509::inToCont(pki_base *pki)
 	SQL_PREPARE(q, "UPDATE certs SET issuer=? WHERE item=?");
 	q.bindValue(0, cert->getSqlItemId());
 	foreach(pki_x509 *child, childs) {
-		int row;
 		if (recursiveSigning(cert, child))
 			continue;
 		if (!child->verify(cert))
 			continue;
-		row = child->row();
-		if (treeview) {
-			beginRemoveRows(index(child->getParent()), row, row);
-			child->getParent()->takeChild(child);
-			endRemoveRows();
-			insertChild(cert, child);
-		}
+		insertChild(child, cert);
 		q.bindValue(1, child->getSqlItemId());
 		AffectedItems(child->getSqlItemId());
 		q.exec();
-		mainwin->dbSqlError(q.lastError());
+		XCA_SQLERROR(q.lastError());
 		if (child->isRevoked())
 			revokedChilds << child->getRevocation();
 	}
@@ -255,7 +218,7 @@ void db_x509::inToCont(pki_base *pki)
 	cert->setRevocations(revList);
 
 	/* Update CRLs */
-	QList<pki_crl *> crls = sqlSELECTpki<pki_crl>(
+	QList<pki_crl *> crls = Store.sqlSELECTpki<pki_crl>(
 			"SELECT item FROM crls WHERE iss_hash=?",
 			QList<QVariant>() << namehash);
 	SQL_PREPARE(q, "UPDATE crls SET issuer=? WHERE item=?");
@@ -267,7 +230,7 @@ void db_x509::inToCont(pki_base *pki)
 		q.bindValue(1, crl->getSqlItemId());
 		AffectedItems(crl->getSqlItemId());
 		q.exec();
-		mainwin->dbSqlError(q.lastError());
+		XCA_SQLERROR(q.lastError());
 	}
 }
 
@@ -275,7 +238,7 @@ QList<pki_x509*> db_x509::getCerts(bool unrevoked)
 {
 	QList<pki_x509*> c;
 	c.clear();
-	FOR_ALL_pki(pki, pki_x509) {
+	foreach(pki_x509 *pki, Store.getAll<pki_x509>()) {
 		if (unrevoked && pki->isRevoked())
 			continue;
 		c.append(pki);
@@ -287,12 +250,16 @@ void db_x509::writeIndex(const QString &fname, bool hierarchy) const
 {
 	if (hierarchy) {
 		QString dir = fname + "/";
-		QList<pki_x509*> issuers = sqlSELECTpki<pki_x509>(
+		if (!QDir().mkpath(fname)) {
+			throw errorEx(tr("Failed to create directory '%1'")
+				.arg(fname));
+		}
+		QList<pki_x509*> issuers = Store.sqlSELECTpki<pki_x509>(
 			"SELECT DISTINCT issuer FROM certs WHERE issuer != item");
 		foreach(pki_x509 *ca, issuers) {
 			XFile file(dir + ca->getUnderlinedName() + ".txt");
 			file.open_write();
-			writeIndex(file, sqlSELECTpki<pki_x509>(
+			writeIndex(file, Store.sqlSELECTpki<pki_x509>(
 				"SELECT item FROM certs WHERE issuer=?",
 				QList<QVariant>()<<QVariant(ca->getSqlItemId()))
 			);
@@ -300,7 +267,8 @@ void db_x509::writeIndex(const QString &fname, bool hierarchy) const
 	} else {
 		XFile file(fname);
 		file.open_write();
-		writeIndex(file,sqlSELECTpki<pki_x509>("SELECT item FROM certs"));
+		writeIndex(file, Store.sqlSELECTpki<pki_x509>(
+					"SELECT item FROM certs"));
 	}
 }
 
@@ -358,8 +326,7 @@ pki_base *db_x509::insert(pki_base *item)
 		delete cert;
 		return NULL;
 	}
-	insertPKI(cert);
-	return cert;
+	return insertPKI(cert);
 }
 
 void db_x509::load(void)
@@ -374,9 +341,7 @@ pki_x509 *db_x509::get1SelectedCert()
 	QModelIndex index;
 	if (indexes.count())
 		index = indexes[0];
-	if (index == QModelIndex())
-		return NULL;
-	return static_cast<pki_x509*>(index.internalPointer());
+	return fromIndex<pki_x509>(index);
 }
 
 void db_x509::markRequestSigned(pki_x509req *req, pki_x509 *cert)
@@ -410,8 +375,7 @@ void db_x509::markRequestSigned(pki_x509req *req, pki_x509 *cert)
 
 void db_x509::newItem()
 {
-	NewX509 *dlg = new NewX509(mainwin);
-	emit connNewX509(dlg);
+	NewX509 *dlg = new NewX509();
 	dlg->setCert();
 	pki_x509 *sigcert = get1SelectedCert();
 	dlg->defineSigner((pki_x509*)sigcert, true);
@@ -423,8 +387,7 @@ void db_x509::newItem()
 
 void db_x509::newCert(pki_x509req *req)
 {
-	NewX509 *dlg = new NewX509(mainwin);
-	emit connNewX509(dlg);
+	NewX509 *dlg = new NewX509();
 	pki_x509 *sigcert = get1SelectedCert();
 	dlg->setCert();
 	dlg->defineRequest(req);
@@ -437,8 +400,7 @@ void db_x509::newCert(pki_x509req *req)
 
 void db_x509::newCert(pki_temp *temp)
 {
-	NewX509 *dlg = new NewX509(mainwin);
-	emit connNewX509(dlg);
+	NewX509 *dlg = new NewX509();
 	dlg->setCert();
 	dlg->defineTemplate(temp);
 	if (dlg->exec()) {
@@ -449,8 +411,7 @@ void db_x509::newCert(pki_temp *temp)
 
 void db_x509::newCert(pki_x509 *cert)
 {
-	NewX509 *dlg = new NewX509(mainwin);
-	emit connNewX509(dlg);
+	NewX509 *dlg = new NewX509();
 	dlg->setCert();
 	dlg->fromX509super(cert, false);
 	if (dlg->exec()) {
@@ -555,7 +516,7 @@ pki_x509 *db_x509::newCert(NewX509 *dlg)
 	// set the comment field
 	cert->setComment(dlg->comment->toPlainText());
 	cert->pkiSource = dlg->getPkiSource();
-	cert = (pki_x509*)insert(cert);
+	cert = dynamic_cast<pki_x509*>(insert(cert));
 	createSuccess(cert);
 	if (cert && clientkey->isToken()) {
 		pki_scard *card = (pki_scard*)clientkey;
@@ -565,18 +526,17 @@ pki_x509 *db_x509::newCert(NewX509 *dlg)
 			try {
 				cert->store_token(false);
 			} catch (errorEx &err) {
-				mainwin->Error(err);
+				XCA_ERROR(err);
 			}
 		}
 	}
-	if (tempkey != NULL)
-		delete(tempkey);
+	delete tempkey;
 	markRequestSigned(req, cert);
 	TransCommit();
     }
 
     catch (errorEx &err) {
-		mainwin->Error(err);
+		XCA_ERROR(err);
 		delete cert;
 		if (tempkey != NULL)
 			delete(tempkey);
@@ -601,8 +561,7 @@ void db_x509::store(QModelIndexList list)
 	if (list.size() == 0)
 		return;
 
-	pki_x509 *crt = static_cast<pki_x509*>(list[0].internalPointer());
-	pki_x509 *oldcrt = NULL;
+	pki_x509 *oldcrt, *crt = fromIndex<pki_x509>(list[0]);
 	if (!crt)
 		return;
 
@@ -633,12 +592,12 @@ void db_x509::store(QModelIndexList list)
 
 	if (privkey && privkey->isPrivKey() && !privkey->isToken()) {
 		if (chain) {
-			usual << exportType(exportType::PKCS12_chain, "p12",
+			usual << exportType(exportType::PKCS12_chain, "pfx",
 				tr("PKCS#12 chain"));
-			types << exportType(exportType::PKCS12, "p12",
+			types << exportType(exportType::PKCS12, "pfx",
 				"PKCS #12");
 		} else {
-			usual << exportType(exportType::PKCS12, "p12",
+			usual << exportType(exportType::PKCS12, "pfx",
 				"PKCS #12");
 		}
 		types <<
@@ -667,9 +626,9 @@ void db_x509::store(QModelIndexList list)
 			tr("CA vCalendar"));
 
 	types = usual << exportType() << types;
-	ExportDialog *dlg = new ExportDialog(mainwin, tr("Certificate export"),
-		tr("X509 Certificates ( *.pem *.cer *.crt *.p12 *.p7b )"), crt,
-		QPixmap(":certImg"), types);
+	ExportDialog *dlg = new ExportDialog(NULL, tr("Certificate export"),
+		tr("X509 Certificates ( *.pem *.cer *.crt *.p12 *.pfx *.p7b )"), crt,
+		QPixmap(":certImg"), types, "certexport");
 	if (!dlg->exec()) {
 		delete dlg;
 		return;
@@ -696,19 +655,19 @@ void db_x509::store(QModelIndexList list)
 			break;
 		case exportType::PEM_selected:
 			foreach(QModelIndex idx, list) {
-				crt = static_cast<pki_x509*>
-							(idx.internalPointer());
-				crt->writeCert(file, true);
+				crt = fromIndex<pki_x509>(idx);
+				if (crt)
+					crt->writeCert(file, true);
 			}
 			break;
 		case exportType::PEM_unrevoked:
-			FOR_ALL_pki(pki, pki_x509) {
+			foreach(pki_x509 *pki, Store.getAll<pki_x509>()) {
 				if (!pki->isRevoked())
 					pki->writeCert(file, true);
 			}
 			break;
 		case exportType::PEM_all:
-			FOR_ALL_pki(pki, pki_x509) {
+			foreach(pki_x509 *pki, Store.getAll<pki_x509>()) {
 				pki->writeCert(file, true);
 			}
 			break;
@@ -751,22 +710,26 @@ void db_x509::store(QModelIndexList list)
 			crt->writeCert(file, true);
 			break;
 		case exportType::Index:
-			foreach(QModelIndex idx, list)
-				certs << static_cast<pki_x509*>
-					(idx.internalPointer());
+			foreach(QModelIndex idx, list) {
+				crt = fromIndex<pki_x509>(idx);
+				if (crt)
+					certs << crt;
+			}
 			writeIndex(file, certs);
 			break;
 		case exportType::vcalendar:
 			foreach(QModelIndex idx, list) {
-				crt = static_cast<pki_x509*>(idx.internalPointer());
-				vcal += crt->icsVEVENT();
+				crt = fromIndex<pki_x509>(idx);
+				if (crt)
+					vcal += crt->icsVEVENT();
 			}
 			writeVcalendar(file, vcal);
 			break;
 		case exportType::vcalendar_ca:
 			foreach(QModelIndex idx, list) {
-				crt = static_cast<pki_x509*>(idx.internalPointer());
-				vcal += crt->icsVEVENT_ca();
+				crt = fromIndex<pki_x509>(idx);
+				if (crt)
+					vcal += crt->icsVEVENT_ca();
 			}
 			writeVcalendar(file, vcal);
 			break;
@@ -775,7 +738,7 @@ void db_x509::store(QModelIndexList list)
 		}
 	}
 	catch (errorEx &err) {
-		MainWindow::Error(err);
+		XCA_ERROR(err);
 	}
 	pki_base::pem_comment = false;
 	delete dlg;
@@ -808,17 +771,16 @@ void db_x509::writePKCS12(pki_x509 *cert, XFile &file, bool chain) const
 		p12 = new pki_pkcs12(cert->getIntName(), cert, privkey);
 		pki_x509 *signer = cert->getSigner();
 		while ((signer != NULL ) && (signer != cert) && chain) {
-			p12->addCaCert(signer);
-			cert=signer;
-			signer=signer->getSigner();
+			p12->append_item(signer);
+			cert = signer;
+			signer = signer->getSigner();
 		}
 		p12->writePKCS12(file);
 	}
 	catch (errorEx &err) {
-		MainWindow::Error(err);
+		XCA_ERROR(err);
 	}
-	if (p12)
-		delete p12;
+	delete p12;
 }
 
 void db_x509::writePKCS7(pki_x509 *cert, XFile &file, exportType::etype type,
@@ -827,11 +789,11 @@ void db_x509::writePKCS7(pki_x509 *cert, XFile &file, exportType::etype type,
 	pki_pkcs7 *p7 = NULL;
 
 	try {
-		p7 = new pki_pkcs7("");
+		p7 = new pki_pkcs7(QString());
 		switch (type) {
 		case exportType::PKCS7_chain:
 			while (cert != NULL) {
-				p7->addCert(cert);
+				p7->append_item(cert);
 				if (cert->getSigner() == cert)
 					cert = NULL;
 				else
@@ -839,20 +801,21 @@ void db_x509::writePKCS7(pki_x509 *cert, XFile &file, exportType::etype type,
 			}
 			break;
 		case exportType::PKCS7:
-			p7->addCert(cert);
+			p7->append_item(cert);
 			break;
 		case exportType::PKCS7_selected:
 			foreach(QModelIndex idx, list) {
-				cert = static_cast<pki_x509*>(idx.internalPointer());
-				p7->addCert(cert);
+				cert = fromIndex<pki_x509>(idx);
+				if (cert)
+					p7->append_item(cert);
 			}
 			break;
 		case exportType::PKCS7_unrevoked:
 		case exportType::PKCS7_all:
-			FOR_ALL_pki(cer, pki_x509) {
+			foreach(pki_x509 *cer, Store.getAll<pki_x509>()) {
 				if ((type == exportType::PKCS7_all) ||
 				    (!cer->isRevoked()))
-					p7->addCert(cer);
+					p7->append_item(cer);
 			}
 			break;
 		default:
@@ -861,26 +824,9 @@ void db_x509::writePKCS7(pki_x509 *cert, XFile &file, exportType::etype type,
 		p7->writeP7(file, false);
 	}
 	catch (errorEx &err) {
-		MainWindow::Error(err);
-	}
-	if (p7 != NULL )
-		delete p7;
-
-}
-
-void db_x509::manageRevocations(QModelIndex idx)
-{
-	pki_x509 *cert = static_cast<pki_x509*>(idx.internalPointer());
-	if (!cert)
-		return;
-	RevocationList *dlg = new RevocationList(mainwin);
-	dlg->setRevList(cert->getRevList(), cert);
-	connect(dlg, SIGNAL(genCRL(pki_x509*)),
-		mainwin->crls, SLOT(newItem(pki_x509*)));
-	if (dlg->exec()) {
-		cert->setRevocations(dlg->getRevList());
-		emit columnsContentChanged();
+		XCA_ERROR(err);
 	}
+	delete p7;
 }
 
 void db_x509::certRenewal(QModelIndexList indexes)
@@ -898,13 +844,13 @@ void db_x509::certRenewal(QModelIndexList indexes)
 	QModelIndex idx = indexes[0];
 
 	try {
-		oldcert = static_cast<pki_x509*>(idx.internalPointer());
+		oldcert = fromIndex<pki_x509>(idx);
 		if (!oldcert || !(signer = oldcert->getSigner()) ||
 				!(signkey = signer->getRefKey()) ||
 				signkey->isPubKey())
 			return;
 		bool renew_myself = signer == oldcert;
-		CertExtend *dlg = new CertExtend(mainwin,
+		CertExtend *dlg = new CertExtend(NULL,
 					renew_myself ? NULL : signer);
 		dlg->revoke->setEnabled(!renew_myself);
 		if (!dlg->exec()) {
@@ -912,14 +858,15 @@ void db_x509::certRenewal(QModelIndexList indexes)
 			return;
 		}
 		if (dlg->revoke->isChecked() && !renew_myself) {
-			Revocation *revoke = new Revocation(mainwin, indexes);
+			Revocation *revoke = new Revocation(indexes);
 			doRevoke = revoke->exec();
 			r = revoke->getRevocation();
 			delete revoke;
 		}
 		foreach(idx, indexes) {
-			oldcert = static_cast<pki_x509*>
-					(idx.internalPointer());
+			oldcert = fromIndex<pki_x509>(idx);
+			if (!oldcert)
+				continue;
 			newcert = new pki_x509(oldcert);
 			newcert->pkiSource = renewed;
 			serial = dlg->keepSerial->isChecked() ?
@@ -939,19 +886,17 @@ void db_x509::certRenewal(QModelIndexList indexes)
 
 			// and finally sign the cert
 			newcert->sign(signkey, oldcert->getDigest());
-			newcert = (pki_x509 *)insert(newcert);
+			newcert = dynamic_cast<pki_x509 *>(insert(newcert));
 			createSuccess(newcert);
 		}
 		if (doRevoke)
 			do_revoke(indexes, r);
 	}
 	catch (errorEx &err) {
-		MainWindow::Error(err);
-		if (newcert)
-			delete newcert;
+		XCA_ERROR(err);
+		delete newcert;
 	}
-	if (dlg)
-		delete dlg;
+	delete dlg;
 	emit columnsContentChanged();
 }
 
@@ -960,7 +905,7 @@ void db_x509::revoke(QModelIndexList indexes)
 {
 	if (indexes.size() == 0)
 		return;
-	Revocation *revoke = new Revocation(mainwin, indexes);
+	Revocation *revoke = new Revocation(indexes);
 	if (revoke->exec()) {
 		do_revoke(indexes, revoke->getRevocation());
 	}
@@ -973,7 +918,9 @@ void db_x509::do_revoke(QModelIndexList indexes, const x509rev &r)
 	x509revList revlist;
 
 	foreach(QModelIndex idx, indexes) {
-		cert = static_cast<pki_x509*>(idx.internalPointer());
+		cert = fromIndex<pki_x509>(idx);
+		if (!cert)
+			continue;
 		iss = cert->getSigner();
 		if (parent == NULL) {
 			parent = iss;
@@ -987,7 +934,9 @@ void db_x509::do_revoke(QModelIndexList indexes, const x509rev &r)
 			 __func__, __LINE__);
 	}
 	foreach(QModelIndex idx, indexes) {
-		pki_x509 *cert = static_cast<pki_x509*>(idx.internalPointer());
+		cert = fromIndex<pki_x509>(idx);
+		if (!cert)
+			continue;
 		x509rev rev(r);
 		rev.setSerial(cert->getSerial());
 		cert->setRevoked(rev);
@@ -1002,9 +951,9 @@ void db_x509::unRevoke(QModelIndexList indexes)
 	x509revList revList;
 
 	foreach(QModelIndex idx, indexes) {
-		pki_x509 *cert = static_cast<pki_x509*>(idx.internalPointer());
+		pki_x509 *cert = fromIndex<pki_x509>(idx);
 		if (!cert)
-			return;
+			continue;
 		pki_x509 *iss = cert->getSigner();
 		if (parent == NULL) {
 			parent = iss;
@@ -1021,9 +970,12 @@ void db_x509::unRevoke(QModelIndexList indexes)
 	revList = parent->getRevList();
 
 	foreach(QModelIndex idx, indexes) {
-		pki_x509 *cert = static_cast<pki_x509*>(idx.internalPointer());
 		int i;
 		x509rev rev;
+		pki_x509 *cert = fromIndex<pki_x509>(idx);
+
+		if (!cert)
+			continue;
 
 		cert->setRevoked(x509rev());
 		rev.setSerial(cert->getSerial());
@@ -1037,7 +989,7 @@ void db_x509::unRevoke(QModelIndexList indexes)
 
 void db_x509::toCertificate(QModelIndex index)
 {
-	pki_x509 *cert = static_cast<pki_x509*>(index.internalPointer());
+	pki_x509 *cert = fromIndex<pki_x509>(index);
 	if (!cert)
 		return;
 	if (!cert->getRefKey() && cert->getSigner() != cert)
@@ -1048,7 +1000,8 @@ void db_x509::toCertificate(QModelIndex index)
 
 void db_x509::toRequest(QModelIndex idx)
 {
-	pki_x509 *cert = static_cast<pki_x509*>(idx.internalPointer());
+	db_x509req *reqs = Database.model<db_x509req>();
+	pki_x509 *cert = fromIndex<pki_x509>(idx);
 	if (!cert)
 		return;
 
@@ -1059,22 +1012,22 @@ void db_x509::toRequest(QModelIndex idx)
 		req->setIntName(cert->getIntName());
 		req->createReq(cert->getRefKey(), cert->getSubject(),
 			cert->getDigest(), cert->getV3ext());
-		createSuccess(mainwin->reqs->insert(req));
+		createSuccess(reqs->insert(req));
 	}
 	catch (errorEx &err) {
-		mainwin->Error(err);
+		XCA_ERROR(err);
 	}
 }
 
 void db_x509::toToken(QModelIndex idx, bool alwaysSelect)
 {
-	pki_x509 *cert = static_cast<pki_x509*>(idx.internalPointer());
+	pki_x509 *cert = fromIndex<pki_x509>(idx);
 	if (!cert)
 		return;
 	try {
 		cert->store_token(alwaysSelect);
 	} catch (errorEx &err) {
-		mainwin->Error(err);
+		XCA_ERROR(err);
         }
 }
 
@@ -1083,11 +1036,11 @@ void db_x509::caProperties(QModelIndex idx)
 	QStringList actions;
 	Ui::CaProperties ui;
 
-	pki_x509 *cert = static_cast<pki_x509*>(idx.internalPointer());
+	pki_x509 *cert = fromIndex<pki_x509>(idx);
 	if (!cert)
 		return;
 
-	QDialog *dlg = new QDialog(mainwin);
+	QDialog *dlg = new QDialog(NULL);
 	ui.setupUi(dlg);
 	ui.days->setSuffix(QString(" ") + tr("days"));
 	ui.days->setMaximum(1000000);
@@ -1095,15 +1048,16 @@ void db_x509::caProperties(QModelIndex idx)
 	ui.image->setPixmap(QPixmap(":certImg"));
 
 	QVariant tmplId = cert->getTemplateSqlId();
-	pki_temp *templ = mainwin->temps->lookupPki<pki_temp>(tmplId);
+	pki_temp *templ = Store.lookupPki<pki_temp>(tmplId);
 
-	ui.temp->insertPkiItems(mainwin->temps->getAll<pki_temp>());
+	ui.temp->insertPkiItems(Store.getAll<pki_temp>());
         ui.temp->setNullItem(tr("No template"));
 	ui.temp->setCurrentIndex(0);
 	if (templ)
 		ui.temp->setCurrentPkiItem(templ);
 
 	ui.certName->setTitle(cert->getIntName());
+	mainwin->helpdlg->register_ctxhelp_button(dlg, "ca_properties");
 
 	if (dlg->exec()) {
 		XSqlQuery q;
@@ -1126,7 +1080,7 @@ void db_x509::caProperties(QModelIndex idx)
 		AffectedItems(cert->getSqlItemId());
 		q.exec();
 	        TransDone(q.lastError());
-		mainwin->dbSqlError(q.lastError());
+		XCA_SQLERROR(q.lastError());
 	}
 	delete dlg;
 }
