@@ -12,7 +12,6 @@
 #include "db_key.h"
 #include "pki_x509super.h"
 #include <QList>
-#include <QMap>
 
 class db_x509name: public db_base
 {
@@ -21,7 +20,7 @@ class db_x509name: public db_base
 	protected:
 		dbheaderList getHeaders();
 	public:
-		db_x509name(MainWindow *mw);
+		db_x509name(const char *classname);
 };
 
 class db_x509super: public db_x509name
@@ -32,13 +31,12 @@ class db_x509super: public db_x509name
 		dbheaderList getHeaders();
 		void loadContainer();
 	public:
-		db_x509super(MainWindow *mw);
+		db_x509super(const char *classname);
 		pki_key *findKey(pki_x509super *ref);
 		QList<pki_x509super *> findByPubKey(pki_key *refkey);
 		void extractPubkey(QModelIndex index);
 		void toTemplate(QModelIndex index);
 		void toOpenssl(QModelIndex index) const;
-		void showPki(pki_base *pki);
 };
 
 #endif
