@@ -24,6 +24,9 @@
  *   So we reduce the validation on windows to the RegEx
  */
 #if !defined(Q_OS_WIN32)
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
 #include <arpa/inet.h>
 unsigned char buf[sizeof(struct in6_addr)];
 #else
