@@ -7,7 +7,7 @@
 
 #include "load_obj.h"
 #include "pki_x509.h"
-#include "pki_key.h"
+#include "pki_evp.h"
 #include "pki_x509req.h"
 #include "pki_pkcs7.h"
 #include "pki_pkcs12.h"
@@ -36,6 +36,7 @@ pki_base *load_base::loadItem(const QString &s)
 	}
 	pki->pkiSource = imported;
 	pki->autoIntName(s);
+	pki->setFilename(s);
 	return pki;
 }
 
