@@ -18,15 +18,29 @@
 #include <openssl/rand.h>
 #include <openssl/engine.h>
 #include <openssl/evp.h>
+#include <QPushButton>
 #include <QMessageBox>
 #include <QThread>
 
 #include <ltdl.h>
 #include "ui_SelectToken.h"
 #include "widgets/PwDialog.h"
+#include "widgets/XcaWarning.h"
 
 #include "openssl_compat.h"
 
+void waitcursor(int start, int line)
+{
+	qDebug() << "Waitcursor" << (start ? "start" : "end") << line;
+	ign_openssl_error();
+	if (!IS_GUI_APP)
+		return;
+	if (start)
+		QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+	else
+		QApplication::restoreOverrideCursor();
+}
+
 pkcs11_lib_list pkcs11::libraries;
 
 pkcs11::pkcs11()
@@ -250,14 +264,15 @@ bool pkcs11::selectToken(slotid *slot, QWidget *w)
 
 	for (int i = 0; i < p11_slots.count(); i++) {
 		try {
-			tkInfo info = tokenInfo(p11_slots[i]);
+			tkInfo info;
+			CK_RV rv = tokenInfo(p11_slots[i], &info);
+			if (rv == CKR_TOKEN_NOT_PRESENT)
+				continue;
 			slotsWithToken.append(i);
 			slotnames << QString("%1 (#%2)").
 				arg(info.label()).arg(info.serial());
 		} catch (errorEx &e) {
-			if (e.info != CKR_TOKEN_NOT_PRESENT) {
-				XCA_WARN(QString("Error: %1").arg(e.getString()));
-			}
+			XCA_WARN(QString("Error: %1").arg(e.getString()));
 		}
 	}
 	switch (slotnames.count()) {
@@ -376,15 +391,25 @@ void pkcs11::initToken(const slotid &slot, unsigned char *pin, int pinlen,
 }
 
 tkInfo pkcs11::tokenInfo(const slotid &slot)
+{
+	tkInfo ti;
+	CK_RV rv = tokenInfo(slot, &ti);
+
+	if (rv != CKR_OK) {
+		pk11error(slot, "C_GetTokenInfo", rv);
+	}
+	return ti;
+}
+
+CK_RV pkcs11::tokenInfo(const slotid &slot, tkInfo *tkinfo)
 {
 	CK_TOKEN_INFO token_info;
 	CK_RV rv;
 
 	CALL_P11_C(slot.lib, C_GetTokenInfo, slot.id, &token_info);
-	if (rv != CKR_OK) {
-		pk11error(slot, "C_GetTokenInfo", rv);
-	}
-	return tkInfo(&token_info);
+	if (rv == CKR_OK)
+		tkinfo->set(&token_info);
+	return rv;
 }
 
 void pkcs11::loadAttribute(pk11_attribute &attribute, CK_OBJECT_HANDLE object)
