@@ -75,27 +75,12 @@ class slotid
     public:
 	CK_ULONG id;
 	pkcs11_lib *lib;
-	slotid()
-	{
-		lib = NULL;
-		id = 0;
-	}
+	slotid() = default;
 	slotid(pkcs11_lib *l, CK_ULONG i)
 	{
 		lib = l;
 		id = i;
 	}
-	slotid(const slotid &other)
-	{
-		lib = other.lib;
-		id = other.id;
-	}
-	slotid &operator = (const slotid &other)
-	{
-		lib = other.lib;
-		id = other.id;
-		return *this;
-	}
 	void isValid() const
 	{
 		if (!lib)
