@@ -10,28 +10,47 @@
 #include "xfile.h"
 #include "pki_base.h"
 #include "exception.h"
+#include "widgets/XcaWarning.h"
 #include <QString>
 #include <openssl/evp.h>
 #include <openssl/sha.h>
 #include <openssl/md5.h>
 #include <typeinfo>
 
+pki_lookup Store;
+
 QRegExp pki_base::limitPattern;
 bool pki_base::pem_comment;
+QList<pki_base*> pki_base::allitems;
 
-pki_base::pki_base(const QString name, pki_base *p)
+pki_base::pki_base(const QString &name, pki_base *p)
 {
 	desc = name;
 	parent = p;
 	childItems.clear();
 	pkiType=none;
 	pkiSource=unknown;
+	allitems << this;
+	qDebug() << "NEW pki_base::count" << allitems.count();
+}
+
+pki_base::pki_base(const pki_base *p)
+{
+	desc = p->desc;
+	parent = p->parent;
+	childItems.clear();
+	pkiType = p->pkiType;
+	pkiSource = p->pkiSource;
+	allitems << this;
+	qDebug() << "COPY pki_base::count" << allitems.count();
+	p->inheritFilename(this);
 }
 
 pki_base::~pki_base(void)
 {
-	while (childItems.size() > 0)
-		delete takeFirst();
+	if (!allitems.removeOne(this))
+		qDebug() << "DEL" << getIntName() << "NOT FOUND";
+	qDebug() << "DEL pki_base::count" << allitems.count();
 }
 
 QString pki_base::comboText() const
@@ -81,6 +100,11 @@ void pki_base::PEM_file_comment(XFile &file) const
 				.toUtf8());
 }
 
+void pki_base::clear()
+{
+	childItems.clear();
+}
+
 bool pki_base::childVisible() const
 {
 	foreach(pki_base *child, childItems)
@@ -107,10 +131,9 @@ QByteArray pki_base::i2d() const
 	return QByteArray();
 }
 
-BIO *pki_base::pem(BIO *, int format)
+bool pki_base::pem(BioByteArray &, int)
 {
-	(void)format;
-	return NULL;
+	return false;
 }
 
 const char *pki_base::getClassName() const
@@ -129,10 +152,9 @@ void pki_base::my_error(const QString &error) const
 
 void pki_base::fromPEMbyteArray(const QByteArray &ba, const QString &name)
 {
-	BIO *bio = BIO_from_QByteArray(ba);
-	fromPEM_BIO(bio, name);
-	BIO_free(bio);
+	fromPEM_BIO(BioByteArray(ba).ro(), name);
 	autoIntName(name);
+	setFilename(name);
 }
 
 QString pki_base::rmslashdot(const QString &s)
@@ -212,7 +234,7 @@ QSqlError pki_base::sqlItemNotFound(QVariant sqlId) const
 			QSqlError::UnknownError);
 }
 
-pki_base *pki_base::getParent()
+pki_base *pki_base::getParent() const
 {
 	return parent;
 }
@@ -227,16 +249,10 @@ pki_base *pki_base::child(int row)
 	return childItems.value(row);
 }
 
-void pki_base::append(pki_base *item)
+void pki_base::insert(pki_base *item)
 {
-	childItems.append(item);
-	item->setParent(this);
-}
-
-void pki_base::insert(int row, pki_base *item)
-{
-	childItems.insert(row, item);
-	item->setParent(this);
+	if (!childItems.contains(item))
+		childItems.prepend(item);
 }
 
 int pki_base::childCount() const
@@ -244,32 +260,21 @@ int pki_base::childCount() const
 	return childItems.size();
 }
 
-int pki_base::row(void) const
+int pki_base::indexOf(const pki_base *child) const
 {
-	if (parent)
-		return parent->childItems.indexOf(const_cast<pki_base*>(this));
-	return 0;
-}
-
-pki_base *pki_base::iterate(pki_base *pki)
-{
-	if (pki == NULL)
-		pki = (childItems.isEmpty()) ? NULL : childItems.first();
-	else
-		pki = childItems.value(pki->row()+1);
-
-	if (pki) {
-		return pki;
-	}
-	if (!parent) {
-		return NULL;
-	}
-	return parent->iterate(this);
+	int ret = childItems.indexOf(const_cast<pki_base *>(child));
+	return ret >= 0 ? ret : 0;
 }
 
 void pki_base::takeChild(pki_base *pki)
 {
-	childItems.takeAt(pki->row());
+	childItems.removeOne(pki);
+}
+
+QList<pki_base*> pki_base::getChildItems() const
+{
+	//#warning need to collect all children below folders (later)
+	return childItems;
 }
 
 pki_base *pki_base::takeFirst()
@@ -350,7 +355,7 @@ bool pki_base::compare(const pki_base *ref) const
 }
 
 /* Unsigned 32 bit integer */
-unsigned pki_base::hash(QByteArray ba)
+unsigned pki_base::hash(const QByteArray &ba)
 {
 	unsigned char md[EVP_MAX_MD_SIZE];
 
@@ -384,6 +389,57 @@ void pki_base::selfComment(QString msg)
 	setComment(appendXcaComment(getComment(), msg));
 }
 
+void pki_base::collect_properties(QMap<QString, QString> &prp) const
+{
+	QString t;
+	prp["Descriptor"] = getIntName();
+	if (getComment().size() > 0)
+		prp["Comment"] = "\n" + getComment().replace('\n', "\n    ");
+	switch (pkiType) {
+	case asym_key:   t = "Asymetric Key"; break;
+	case x509_req:   t = "PKCS#10 Certificate request"; break;
+	case x509:       t = "x.509 Certificate"; break;
+	case revocation: t = "Certificate revocation list"; break;
+	case tmpl:       t = "XCA Template"; break;
+	default:         t = "Unknown"; break;
+	}
+	prp["Type"] = t;
+}
+
+void pki_base::print(BioByteArray &bba, enum print_opt opt) const
+{
+	static const QStringList order = {
+		"Type", "Descriptor", "Subject", "Issuer", "Serial",
+		"Not Before", "Not After", "Verify Ok",
+		"Unstructured Name", "Challange Password",
+		"Last Update", "Next Update", "CA", "Self signed",
+		"Key", "Signature", "Extensions", "Comment",
+	};
+	if (opt == print_coloured) {
+		QMap<QString, QString> prp;
+		QStringList keys;
+		int w = 0;
+
+		collect_properties(prp);
+		keys = prp.keys();
+
+		foreach (const QString &key, keys) {
+			if (key.size() > w)
+				w = key.size();
+			if (!order.contains(key))
+				XCA_WARN(tr("Property '%1' not listed in 'pki_base::print'").arg(key));
+		}
+		w = (w + 1) * -1;
+		foreach (const QString &key, order) {
+			if (!prp.contains(key))
+				continue;
+
+			bba += QString(COL_YELL "%1" COL_RESET " %2\n")
+				.arg(key + ":", w).arg(prp[key]).toUtf8();
+		}
+	}
+}
+
 static QString icsValue(QString s)
 {
 	int n = 60;
