@@ -8,24 +8,22 @@
 #ifndef __PKI_BASE_H
 #define __PKI_BASE_H
 
-#include <openssl/err.h>
 #include <QString>
-#include <QListView>
+#include <QRegExp>
+#include <QVariant>
+#include <QByteArray>
+#include "BioByteArray.h"
 #include "asn1time.h"
 #include "pkcs11_lib.h"
-#include "db.h"
 #include "base.h"
+#include "db.h"
+#include "pki_lookup.h"
 #include "headerlist.h"
-#include "settings.h"
 #include "sql.h"
 #include "xfile.h"
 
-#define __ME QString("(%1[%2]:%3)") \
-		.arg(getClassName()) \
-		.arg(getSqlItemId().toString()) \
-		.arg(getIntName())
-#define pki_openssl_error() _openssl_error(__ME, C_FILE, __LINE__)
-#define pki_ign_openssl_error() _ign_openssl_error(__ME, C_FILE, __LINE__)
+#define pki_openssl_error() _openssl_error(*this, C_FILE, __LINE__)
+#define pki_ign_openssl_error() _ign_openssl_error(*this, C_FILE, __LINE__)
 
 enum pki_source {
 	unknown,
@@ -44,6 +42,8 @@ enum pki_source {
 #define VIEW_item_source 4
 #define VIEW_item_comment 5
 
+extern pki_lookup Store;
+
 class pki_base : public QObject
 {
 		Q_OBJECT
@@ -51,8 +51,10 @@ class pki_base : public QObject
 	public: /* static */
 		static QRegExp limitPattern;
 		static QString rmslashdot(const QString &fname);
-		static unsigned hash(QByteArray ba);
+		static unsigned hash(const QByteArray &ba);
 		static bool pem_comment;
+		static int count;
+		static QList<pki_base*> allitems;
 
 	protected:
 		QVariant sqlItemId;
@@ -62,7 +64,10 @@ class pki_base : public QObject
 		/* model data */
 		pki_base *parent;
 		void my_error(const QString &error) const;
+		QString filename;
 		virtual void PEM_file_comment(XFile &file) const;
+		virtual void collect_properties(QMap<QString, QString> &) const;
+		QList<pki_base*> childItems;
 
 	public:
 		enum msg_type {
@@ -71,17 +76,37 @@ class pki_base : public QObject
 			msg_delete_multi,
 			msg_create,
 		};
+		enum print_opt {
+			print_openssl_txt,
+			print_pem,
+			print_coloured,
+		};
 		enum pki_source pkiSource;
-		QList<pki_base*> childItems;
 
-		pki_base(const QString d = "", pki_base *p = NULL);
+		pki_base(const QString &d = QString(), pki_base *p = NULL);
+		pki_base(const pki_base *p);
 		virtual ~pki_base();
 
+		QList<pki_base*> getChildItems() const;
+		void clear();
 		QString getIntName() const
 		{
 			return desc;
 		}
+		void setFilename(const QString &s)
+		{
+			filename = s;
+		}
+		QString getFilename() const
+		{
+			return filename;
+		}
+		void inheritFilename(pki_base *pki) const
+		{
+			pki->setFilename(getFilename());
+		}
 		virtual QString comboText() const;
+		virtual void print(BioByteArray &b, enum print_opt opt) const;
 		QString getUnderlinedName() const;
 		void setIntName(const QString &d)
 		{
@@ -92,7 +117,7 @@ class pki_base : public QObject
 		{
 			return comment;
 		}
-		void setComment(QString c)
+		void setComment(const QString &c)
 		{
 			comment = c;
 		}
@@ -119,14 +144,13 @@ class pki_base : public QObject
 
 		/* Tree View management */
 		void setParent(pki_base *p);
-		virtual pki_base *getParent();
+		pki_base *getParent() const;
 		pki_base *child(int row);
-		void append(pki_base *item);
-		void insert(int row, pki_base *item);
+		void insert(pki_base *item);
 		int childCount() const;
-		pki_base *iterate(pki_base *pki = NULL);
 		void takeChild(pki_base *pki);
 		pki_base *takeFirst();
+		int indexOf(const pki_base *child) const;
 
 		/* Token handling */
 		virtual void deleteFromToken();
@@ -134,7 +158,7 @@ class pki_base : public QObject
 		virtual int renameOnToken(const slotid &, const QString &);
 
 		/* Import / Export management */
-		virtual BIO *pem(BIO *, int format=0);
+		virtual bool pem(BioByteArray &b, int format=0);
 		virtual void fromPEM_BIO(BIO *, const QString &);
 		virtual void fromPEMbyteArray(const QByteArray &, const QString &);
 		virtual void fload(const QString &);
@@ -148,7 +172,6 @@ class pki_base : public QObject
 			(void)hd;
 			return QVariant();
 		}
-		int row() const;
 		virtual QVariant column_data(const dbheader *hd) const;
 		virtual QVariant getIcon(const dbheader *hd) const;
 		virtual QVariant column_tooltip(const dbheader *hd) const;
@@ -177,6 +200,11 @@ class pki_base : public QObject
 		void selfComment(QString msg);
 		QStringList icsVEVENT(const a1time &expires,
 		    const QString &summary, const QString &description) const;
+		operator QString() const
+		{
+			return QString("(%1[%2]:%3)").arg(getClassName())
+				.arg(getSqlItemId().toString()).arg(getIntName());
+		}
 };
 
 Q_DECLARE_METATYPE(pki_base *);
