@@ -9,7 +9,7 @@
 #include "pki_crl.h"
 #include "func.h"
 #include "exception.h"
-#include "db_base.h"
+#include "database_model.h"
 #include <QDir>
 
 #include "openssl_compat.h"
@@ -24,7 +24,7 @@ pki_crl::pki_crl(const QString name )
 
 pki_x509 *pki_crl::getIssuer() const
 {
-	return db_base::lookupPki<pki_x509>(issuerSqlId);
+	return Store.lookupPki<pki_x509>(issuerSqlId);
 }
 
 QString pki_crl::getIssuerName() const
@@ -78,7 +78,7 @@ QSqlError pki_crl::lookupIssuer()
 	if (q.lastError().isValid())
 		return q.lastError();
 	while (q.next()) {
-		pki_x509 *x = db_base::lookupPki<pki_x509>(q.value(0));
+		pki_x509 *x = Store.lookupPki<pki_x509>(q.value(0));
 		if (!x) {
 			qDebug("CA certificate with id %d not found",
 				q.value(0).toInt());
@@ -115,7 +115,7 @@ void pki_crl::restoreSql(const QSqlRecord &rec)
 	QByteArray ba = QByteArray::fromBase64(
 				rec.value(VIEW_crls_crl).toByteArray());
 	d2i(ba);
-	setIssuer(db_base::lookupPki<pki_x509>(rec.value(VIEW_crls_issuer)));
+	setIssuer(Store.lookupPki<pki_x509>(rec.value(VIEW_crls_issuer)));
 }
 
 QSqlError pki_crl::deleteSqlData()
@@ -281,13 +281,9 @@ void pki_crl::writeCrl(XFile &file, bool pem) const
 	pki_openssl_error();
 }
 
-BIO *pki_crl::pem(BIO *b, int format)
+bool pki_crl::pem(BioByteArray &b, int)
 {
-	(void)format;
-	if (!b)
-		b = BIO_new(BIO_s_mem());
-	PEM_write_bio_X509_CRL(b, crl);
-	return b;
+	return PEM_write_bio_X509_CRL(b, crl);
 }
 
 a1time pki_crl::getLastUpdate() const
@@ -440,7 +436,7 @@ QStringList pki_crl::icsVEVENT() const
 		  "It is stored in the XCA database '%3'")
 			.arg(ca->getIntName())
 			.arg(getNextUpdate().toPretty())
-			.arg(currentDB)
+			.arg(Database.name())
 		);
 	}
 	return pki_base::icsVEVENT(getNextUpdate(),
@@ -450,6 +446,28 @@ QStringList pki_crl::icsVEVENT() const
 			.arg(getIntName())
 			.arg(getLastUpdate().toPretty())
 			.arg(getNextUpdate().toPretty())
-			.arg(currentDB)
+			.arg(Database.name())
 	);
 }
+
+void pki_crl::collect_properties(QMap<QString, QString> &prp) const
+{
+	prp["Last Update"] = getLastUpdate().toPretty();
+	prp["Next Update"] = getNextUpdate().toPretty();
+	pki_x509name::collect_properties(prp);
+}
+
+void pki_crl::print(BioByteArray &bba, enum print_opt opt) const
+{
+	pki_x509name::print(bba, opt);
+	switch (opt) {
+	case print_openssl_txt:
+		X509_CRL_print(bba, crl);
+		break;
+	case print_pem:
+		PEM_write_bio_X509_CRL(bba, crl);
+		break;
+	case print_coloured:
+		break;
+	}
+}
