@@ -9,17 +9,43 @@
 #ifndef __PKI_CRL_H
 #define __PKI_CRL_H
 
-#include <iostream>
-#include <openssl/pem.h>
+#include <openssl/bio.h>
 #include "pki_x509.h"
 #include "x509name.h"
-#include "asn1time.h"
-#include "asn1int.h"
 
 #define VIEW_crls_num 6
 #define VIEW_crls_issuer 7
 #define VIEW_crls_crl 8
 
+#include "widgets/hashBox.h"
+class crljob
+{
+    public:
+	pki_x509 *issuer;
+	bool withReason;
+	bool authKeyId;
+	bool subAltName;
+	bool setCrlNumber;
+	a1int crlNumber;
+	int crlDays;
+	const EVP_MD *hashAlgo;
+	a1time lastUpdate;
+	a1time nextUpdate;
+
+	crljob(pki_x509 *x) : issuer(x)
+	{
+		withReason = true;
+		authKeyId = true;
+		subAltName = true;
+		setCrlNumber = issuer->getCrlNumber().getLong() > 0;
+		crlNumber = issuer->getCrlNumber();
+		crlNumber++;
+		crlDays = issuer->getCrlDays();
+		hashAlgo = hashBox::getDefaultMD();
+		nextUpdate = lastUpdate.addDays(crlDays);
+	}
+};
+
 class pki_crl: public pki_x509name
 {
 		Q_OBJECT
@@ -28,6 +54,7 @@ class pki_crl: public pki_x509name
 		QVariant issuerSqlId;
 		X509_CRL *crl;
 		extList extensions() const;
+		void collect_properties(QMap<QString, QString> &prp) const;
 	public:
 		pki_crl(const QString name = "");
 		~pki_crl();
@@ -65,13 +92,14 @@ class pki_crl: public pki_x509name
 		void setCrlNumber(a1int num);
 		bool getCrlNumber(a1int *num) const;
 		a1int getCrlNumber() const;
-		BIO *pem(BIO *, int);
+		bool pem(BioByteArray &, int);
 		bool visible() const;
 		QSqlError lookupIssuer();
 		QSqlError insertSqlData();
 		QSqlError deleteSqlData();
 		void restoreSql(const QSqlRecord &rec);
 		QStringList icsVEVENT() const;
+		void print(BioByteArray &b, enum print_opt opt) const;
 };
 
 #endif
