@@ -11,17 +11,16 @@
 #include "func.h"
 #include "db.h"
 #include "entropy.h"
+#include "BioByteArray.h"
 #include "widgets/PwDialog.h"
+#include "widgets/XcaWarning.h"
+#include "widgets/XcaProgress.h"
 
 #include <openssl/rand.h>
 #include <openssl/evp.h>
 #include <openssl/pem.h>
 #include <openssl/pkcs12.h>
-
-#include <QProgressDialog>
-#include <QApplication>
-#include <QPixmap>
-#include <QDir>
+#include <openssl/err.h>
 
 #include "openssl_compat.h"
 
@@ -86,30 +85,24 @@ bool pki_evp::sqlUpdatePrivateKey()
 	return false;
 }
 
-void pki_evp::generate(int bits, int type, QProgressBar *progress, int curve_nid)
+void pki_evp::generate(const keyjob &task)
 {
 	Entropy::seed_rng();
-
-#ifdef OPENSSL_NO_EC
-	(void)curve_nid;
-#endif
-	progress->setMinimum(0);
-	progress->setMaximum(100);
-	progress->setValue(50);
+	XcaProgress progress;
 
 #if OPENSSL_VERSION_NUMBER < 0x10100000L
 	BN_GENCB _bar, *bar = &_bar;
 #else
 	BN_GENCB *bar = BN_GENCB_new();
 #endif
-	BN_GENCB_set_old(bar, inc_progress_bar, progress);
+	BN_GENCB_set_old(bar, XcaProgress::inc, &progress);
 
-	switch (type) {
+	switch (task.ktype.type) {
 	case EVP_PKEY_RSA: {
 		RSA *rsakey = RSA_new();
 		BIGNUM *e = BN_new();
 		BN_set_word(e, 0x10001);
-		if (RSA_generate_key_ex(rsakey, bits, e, bar))
+		if (RSA_generate_key_ex(rsakey, task.size, e, bar))
 			EVP_PKEY_assign_RSA(key, rsakey);
 		else
 			RSA_free(rsakey);
@@ -118,18 +111,17 @@ void pki_evp::generate(int bits, int type, QProgressBar *progress, int curve_nid
 	}
 	case EVP_PKEY_DSA: {
 		DSA *dsakey = DSA_new();
-		progress->setMaximum(500);
-		if (DSA_generate_parameters_ex(dsakey, bits, NULL, 0, NULL,
-		    NULL, bar) && DSA_generate_key(dsakey))
+		if (DSA_generate_parameters_ex(dsakey, task.size, NULL, 0,
+			 NULL, NULL, bar) && DSA_generate_key(dsakey))
 				EVP_PKEY_assign_DSA(key, dsakey);
 		else
 			DSA_free(dsakey);
 		break;
 	}
 #ifndef OPENSSL_NO_EC
-	case EVP_PKEY_EC:
+	case EVP_PKEY_EC: {
 		EC_KEY *eckey;
-		EC_GROUP *group = EC_GROUP_new_by_curve_name(curve_nid);
+		EC_GROUP *group = EC_GROUP_new_by_curve_name(task.ec_nid);
 		if (!group)
 			break;
 		eckey = EC_KEY_new();
@@ -148,6 +140,18 @@ void pki_evp::generate(int bits, int type, QProgressBar *progress, int curve_nid
 		EC_KEY_free(eckey);
 		EC_GROUP_free(group);
 		break;
+	}
+#ifdef EVP_PKEY_ED25519
+	case EVP_PKEY_ED25519: {
+		EVP_PKEY *pkey = NULL;
+		EVP_PKEY_CTX *pctx = EVP_PKEY_CTX_new_id(EVP_PKEY_ED25519, NULL);
+		EVP_PKEY_keygen_init(pctx);
+		EVP_PKEY_keygen(pctx, &pkey);
+		EVP_PKEY_CTX_free(pctx);
+		EVP_PKEY_free(key);
+		key = pkey;
+	}
+#endif
 #endif
 	}
 #if OPENSSL_VERSION_NUMBER >= 0x10100000L
@@ -169,8 +173,8 @@ pki_evp::pki_evp(const pki_evp *pk)
 	encKey = pk->getEncKey();
 }
 
-pki_evp::pki_evp(const QString name, int type)
-	:pki_key(name)
+pki_evp::pki_evp(const QString &n, int type)
+	:pki_key(n)
 {
 	init();
 	EVP_PKEY_set_type(key, type);
@@ -194,6 +198,15 @@ static bool EVP_PKEY_isPrivKey(EVP_PKEY *key)
 		case EVP_PKEY_EC:
 			return EC_KEY_get0_private_key(
 				EVP_PKEY_get0_EC_KEY(key)) ? true: false;
+#ifdef EVP_PKEY_ED25519
+		case EVP_PKEY_ED25519: {
+			unsigned char buf[ED25519_KEYLEN];
+			size_t len = sizeof buf;
+			int ret = EVP_PKEY_get_raw_private_key(key, buf, &len);
+			ign_openssl_error();
+			return ret && len == ED25519_KEYLEN;
+		}
+#endif
 #endif
 	}
 	return false;
@@ -228,34 +241,40 @@ pki_evp::pki_evp(EVP_PKEY *pkey)
 	set_EVP_PKEY(pkey);
 }
 
-void pki_evp::openssl_pw_error(QString fname)
+bool pki_evp::openssl_pw_error() const
 {
 	switch (ERR_peek_error() & 0xff000fff) {
 	case ERR_PACK(ERR_LIB_PEM, 0, PEM_R_BAD_DECRYPT):
 	case ERR_PACK(ERR_LIB_PEM, 0, PEM_R_BAD_PASSWORD_READ):
 	case ERR_PACK(ERR_LIB_EVP, 0, EVP_R_BAD_DECRYPT):
 		pki_ign_openssl_error();
-		throw errorEx(tr("Failed to decrypt the key (bad password) %1")
-				.arg(fname), getClassName(), E_PASSWD);
+		return true;
 	}
+	return false;
 }
 
 void pki_evp::fromPEMbyteArray(const QByteArray &ba, const QString &name)
 {
-	BIO *bio = BIO_from_QByteArray(ba);
 	EVP_PKEY *pkey;
 	pass_info p(XCA_TITLE,
-		tr("Please enter the password to decrypt the private key.") +
-		" " + name);
-	pkey = PEM_read_bio_PrivateKey(bio, NULL, PwDialog::pwCallback, &p);
-	openssl_pw_error(name);
+		tr("Please enter the password to decrypt the private key %1.")
+			.arg(name));
+	pkey = load_ssh_ed25519_privatekey(ba, p);
+
+	while (!pkey) {
+		pkey = PEM_read_bio_PrivateKey(BioByteArray(ba).ro(), NULL,
+						PwDialog::pwCallback, &p);
+		if (openssl_pw_error())
+			XCA_PASSWD_ERROR();
+		if (p.getResult() != pw_ok)
+			throw p.getResult();
+		if (pki_ign_openssl_error())
+			break;
+	}
 	if (!pkey) {
 		pki_ign_openssl_error();
-		BIO_free(bio);
-		bio = BIO_from_QByteArray(ba);
-		pkey = PEM_read_bio_PUBKEY(bio, NULL, PwDialog::pwCallback, &p);
+		pkey = PEM_read_bio_PUBKEY(BioByteArray(ba).ro(), NULL, NULL,0);
 	}
-	BIO_free(bio);
 	pki_openssl_error();
 	set_EVP_PKEY(pkey, name);
 }
@@ -264,23 +283,26 @@ static void search_ec_oid(EVP_PKEY *pkey)
 {
 #ifndef OPENSSL_NO_EC
 	EC_KEY *ec;
+	EC_GROUP *builtin;
+	const EC_GROUP *ec_group;
+
 	int keytype = EVP_PKEY_id(pkey);
 
 	if (keytype != EVP_PKEY_EC)
 		return;
 
 	ec = EVP_PKEY_get0_EC_KEY(pkey);
+	if (!ec)
+		return;
 
-	const EC_GROUP *ec_group = EC_KEY_get0_group(ec);
-	EC_GROUP *builtin;
-
+	ec_group = EC_KEY_get0_group(ec);
 	if (!ec_group)
 		return;
 	if (EC_GROUP_get_curve_name(ec_group))
 		return;
 	/* There is an EC_GROUP with a missing OID
 	 * because of explicit parameters */
-	foreach(builtin_curve curve, pki_key::builtinCurves) {
+	foreach(builtin_curve curve, builtinCurves) {
 		builtin = EC_GROUP_new_by_curve_name(curve.nid);
 		if (EC_GROUP_cmp(builtin, ec_group, NULL) == 0) {
 			EC_GROUP_set_curve_name((EC_GROUP *)ec_group, curve.nid);
@@ -313,9 +335,82 @@ void pki_evp::set_EVP_PKEY(EVP_PKEY *pkey, QString name)
 	search_ec_oid(pkey);
 
 	autoIntName(name);
+	setFilename(name);
 	pki_openssl_error();
 }
 
+EVP_PKEY *pki_evp::load_ssh_ed25519_privatekey(const QByteArray &ba,
+						const pass_info &p)
+{
+	EVP_PKEY *pkey = NULL;
+	unsigned char *pdata;
+	long plen;
+	QByteArray chunk, enc_algo, kdfname, kdf, pub, priv;
+
+	(void)p; // Will be used later for decryption
+	if (!PEM_bytes_read_bio(&pdata, &plen, NULL, PEM_STRING_OPENSSH_KEY,
+				BioByteArray(ba).ro(), NULL, NULL))
+		return NULL;
+
+	QByteArray content((const char*)pdata, plen);
+	OPENSSL_free(pdata);
+
+	if (!content.startsWith("openssh-key-v1") ||
+	     // also check trailing \0
+	     content.constData()[sizeof "openssh-key-v1" -1])
+		return NULL;
+
+	content.remove(0, sizeof "openssh-key-v1");
+	// encryption: "none", "aes256-ctr"
+	enc_algo = ssh_key_next_chunk(&content);
+	// KDFName "bcrypt"
+	kdfname = ssh_key_next_chunk(&content);
+	kdf = ssh_key_next_chunk(&content);
+
+	if (enc_algo != "none" || kdfname != "none") {
+		qCritical("Encrypted SSH ED25519 keys not supported, yet");
+		return NULL;
+	}
+	// check bytes 00 00 00 01
+	const char *d = content.constData();
+	if (d[0] || d[1] || d[2] || d[3] != 1)
+		return NULL;
+	content.remove(0, 4);
+	// Handle first occurance of the public key
+	pub = ssh_key_next_chunk(&content);
+	ssh_key_check_chunk(&pub, "ssh-ed25519");
+	pub = ssh_key_next_chunk(&pub);
+	if (pub.count() != ED25519_KEYLEN)
+		return NULL;
+
+	// Followed by the private key
+	priv = ssh_key_next_chunk(&content);
+	// Drop 64bit random nonce
+	priv.remove(0, 8);
+
+	ssh_key_check_chunk(&priv, "ssh-ed25519");
+	// The first pubkey must match the second occurance
+	// in front of the private one
+	if (pub != ssh_key_next_chunk(&priv))
+		return NULL;
+	priv = ssh_key_next_chunk(&priv);
+	// The private key is concatenated by the public key in one chunk
+	if (priv.count() != 2 * ED25519_KEYLEN)
+		return NULL;
+	// The last ED25519_KEYLEN bytes must match the public key
+	if (pub != priv.mid(ED25519_KEYLEN))
+		return NULL;
+	// The first ED25519_KEYLEN octets are the private key
+#ifndef OPENSSL_NO_EC
+#ifdef EVP_PKEY_ED25519
+	pkey = EVP_PKEY_new_raw_private_key(EVP_PKEY_ED25519, NULL,
+		(const unsigned char *)priv.constData(), ED25519_KEYLEN);
+#endif
+#endif
+	pki_openssl_error();
+	return pkey;
+}
+
 void pki_evp::fload(const QString &fname)
 {
 	pass_info p(XCA_TITLE, tr("Please enter the password to decrypt the private key from file:\n%1").
@@ -325,8 +420,19 @@ void pki_evp::fload(const QString &fname)
 	pki_ign_openssl_error();
 	XFile file(fname);
 	file.open_read();
-	EVP_PKEY *pkey = PEM_read_PrivateKey(file.fp(), NULL, cb, &p);
-	openssl_pw_error(fname);
+	EVP_PKEY *pkey;
+
+	do {
+		pkey = PEM_read_PrivateKey(file.fp(), NULL, cb, &p);
+		if (openssl_pw_error())
+			XCA_PASSWD_ERROR();
+		if (p.getResult() != pw_ok)
+			throw p.getResult();
+		if (pki_ign_openssl_error())
+			break;
+		file.retry_read();
+	} while (!pkey);
+
 	if (!pkey) {
 		pki_ign_openssl_error();
 		file.retry_read();
@@ -352,8 +458,12 @@ void pki_evp::fload(const QString &fname)
 		pki_ign_openssl_error();
 		file.retry_read();
 		pkey = b2i_PVK_bio(file.bio(), cb, &p);
-		pki_openssl_error();
-        }
+	}
+	if (!pkey) {
+		pki_ign_openssl_error();
+		file.retry_read();
+		pkey = load_ssh_ed25519_privatekey(file.read(10000), p);
+	}
 #endif
 	if (!pkey) {
 		pki_ign_openssl_error();
@@ -369,13 +479,13 @@ void pki_evp::fload(const QString &fname)
 		pki_ign_openssl_error();
 		file.retry_read();
 		pkey = load_ssh2_key(file);
-        }
+	}
 #if OPENSSL_VERSION_NUMBER >= 0x10000000L
 	if (!pkey) {
 		pki_ign_openssl_error();
 		file.retry_read();
 		pkey = b2i_PublicKey_bio(file.bio());
-        }
+	}
 #endif
 	if (pki_ign_openssl_error() || !pkey) {
 		if (pkey)
@@ -453,10 +563,8 @@ EVP_PKEY *pki_evp::decryptKey() const
 	qDebug() << "myencKey.count()"<<myencKey.count();
 	if (myencKey.count() == 0)
 		return NULL;
-	BIO *b = BIO_from_QByteArray(myencKey);
-	check_oom(b);
 	EVP_PKEY *priv = NULL;
-	X509_SIG *p8 = d2i_PKCS8_bio(b, NULL);
+	X509_SIG *p8 = d2i_PKCS8_bio(BioByteArray(myencKey).ro(), NULL);
 	if (p8) {
 		PKCS8_PRIV_KEY_INFO *p8inf = PKCS8_decrypt(p8,
 				ownPassBuf.constData(), ownPassBuf.size());
@@ -466,7 +574,6 @@ EVP_PKEY *pki_evp::decryptKey() const
 		}
 		X509_SIG_free(p8);
 	}
-	BIO_free(b);
 	if (priv)
 		return priv;
 	pki_ign_openssl_error();
@@ -483,9 +590,6 @@ EVP_PKEY *pki_evp::legacyDecryptKey(QByteArray &myencKey,
 	unsigned char ckey[EVP_MAX_KEY_LENGTH];
 
 	EVP_PKEY *tmpkey;
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
-	EVP_CIPHER_CTX ctxbuf;
-#endif
 	EVP_CIPHER_CTX *ctx;
 	const EVP_CIPHER *cipher = EVP_des_ede3_cbc();
 	p = (unsigned char *)OPENSSL_malloc(myencKey.count());
@@ -498,18 +602,14 @@ EVP_PKEY *pki_evp::legacyDecryptKey(QByteArray &myencKey,
 	/* generate the key */
 	EVP_BytesToKey(cipher, EVP_sha1(), iv,
 		ownPassBuf.constUchar(), ownPassBuf.size(), 1, ckey, NULL);
-#if OPENSSL_VERSION_NUMBER >= 0x10100000L
 	ctx = EVP_CIPHER_CTX_new();
-#else
-	ctx = &ctxbuf;
-#endif
-	EVP_CIPHER_CTX_init(ctx);
 	EVP_DecryptInit(ctx, cipher, ckey, iv);
 	EVP_DecryptUpdate(ctx, p , &outl,
 		(const unsigned char*)myencKey.constData() +8,
 		myencKey.count() -8);
 	decsize = outl;
 	EVP_DecryptFinal_ex(ctx, p + decsize , &outl);
+
 	EVP_CIPHER_CTX_cleanup(ctx);
 	decsize += outl;
 	pki_openssl_error();
@@ -517,10 +617,8 @@ EVP_PKEY *pki_evp::legacyDecryptKey(QByteArray &myencKey,
 	pki_openssl_error();
 	OPENSSL_cleanse(p, myencKey.count());
 	OPENSSL_free(p);
-	EVP_CIPHER_CTX_cleanup(ctx);
-#if OPENSSL_VERSION_NUMBER >= 0x10100000L
 	EVP_CIPHER_CTX_free(ctx);
-#endif
+
 	pki_openssl_error();
 	if (EVP_PKEY_type(getKeyType()) == EVP_PKEY_RSA) {
 		RSA *rsa = EVP_PKEY_get0_RSA(tmpkey);
@@ -563,16 +661,16 @@ void pki_evp::encryptKey(const char *password)
 		ret = PwDialog::execute(&p, &ownPassBuf, true);
 		if (ret != 1)
 			throw errorEx("Password input aborted", getClassName());
-	pki_openssl_error();
+		pki_openssl_error();
 	} else if (ownPass == ptBogus) { // BOGUS password
 		ownPassBuf = "Bogus";
-	pki_openssl_error();
+		pki_openssl_error();
 	} else {
 		if (password) {
 			/* use the password parameter
 			 * if this is a common password */
 			ownPassBuf = password;
-	pki_openssl_error();
+			pki_openssl_error();
 		} else {
 			int ret = 0;
 			ownPassBuf = passwd;
@@ -591,14 +689,11 @@ void pki_evp::encryptKey(const char *password)
 	}
 
 	/* Convert private key to DER(PKCS8-aes) */
-	const char *p;
-	BIO *bio = BIO_new(BIO_s_mem());
-	i2d_PKCS8PrivateKey_bio(bio, key, EVP_aes_256_cbc(),
+	BioByteArray bba;
+	i2d_PKCS8PrivateKey_bio(bba, key, EVP_aes_256_cbc(),
 		ownPassBuf.data(), ownPassBuf.size(), NULL, 0);
 	pki_openssl_error();
-	int l = BIO_get_mem_data(bio, &p);
-	encKey = QByteArray(p, l);
-	BIO_free(bio);
+	encKey = bba;
 
 	/* Replace private key by public key and
 	   have the encrypted private in "encKey"
@@ -754,18 +849,10 @@ int PEM_write_bio_PrivateKey_traditional(BIO *bp, EVP_PKEY *x,
                                          unsigned char *kstr, int klen,
                                          pem_password_cb *cb, void *u)
 {
-	const char *t = "";
-	int keytype = EVP_PKEY_id(x);
+	QString pem = keytype::byPKEY(x).traditionalPemName();
 
-	switch (keytype) {
-		case EVP_PKEY_RSA: t = "RSA PRIVATE KEY"; break;
-		case EVP_PKEY_DSA: t = "DSA PRIVATE KEY"; break;
-#ifndef OPENSSL_NO_EC
-		case EVP_PKEY_EC: t = "EC PRIVATE KEY"; break;
-#endif
-	}
 	return PEM_ASN1_write_bio((i2d_of_void *)i2d_PrivateKey,
-				t, bp, (char*)x, enc, kstr, klen, cb, u);
+			pem.toLatin1(), bp, (char*)x, enc, kstr, klen, cb, u);
 }
 #endif
 
@@ -800,37 +887,47 @@ bool pki_evp::verify_priv(EVP_PKEY *pkey) const
 {
 	bool verify = true;
 #if OPENSSL_VERSION_NUMBER >= 0x10000000L
-	unsigned char md[32], sig[1024];
-	size_t mdlen = sizeof md, siglen = sizeof sig;
-	EVP_PKEY_CTX *ctx = NULL;
+	unsigned char data[32], sig[1024];
+	size_t datalen = sizeof data, siglen = sizeof sig;
+	EVP_MD_CTX *ctx = NULL;
+	const EVP_MD *md = EVP_sha256();
+	EVP_PKEY_CTX *pkctx = NULL;
 
 	if (!EVP_PKEY_isPrivKey(pkey))
 		return true;
 	do {
-		ctx = EVP_PKEY_CTX_new(pkey, NULL);
+		ctx = EVP_MD_CTX_new();
 		pki_ign_openssl_error();
-		RAND_bytes(md, mdlen);
+		RAND_bytes(data, datalen);
 		check_oom(ctx);
 		verify = false;
 
-		/* Sign some random data in "md" */
-		if (EVP_PKEY_sign_init(ctx) <= 0)
+		/* Sign some random data in "data" */
+#ifdef EVP_PKEY_ED25519
+		if (EVP_PKEY_id(pkey) == EVP_PKEY_ED25519)
+			md = NULL;
+#endif
+		if (!EVP_DigestSignInit(ctx, &pkctx, md, NULL, pkey))
 			break;
+
 		if (EVP_PKEY_id(pkey) == EVP_PKEY_RSA)
-			EVP_PKEY_CTX_set_rsa_padding(ctx, RSA_PKCS1_PADDING);
-		if (EVP_PKEY_CTX_set_signature_md(ctx, EVP_sha256()) <= 0)
-			break;
-		if (EVP_PKEY_sign(ctx, sig, &siglen, md, mdlen) <= 0)
+			EVP_PKEY_CTX_set_rsa_padding(pkctx, RSA_PKCS1_PADDING);
+
+		if (!EVP_DigestSign(ctx, sig, &siglen, data, datalen))
 			break;
+
 		/* Verify the signature */
-		if (EVP_PKEY_verify_init(ctx) <= 0)
+		if (!EVP_DigestVerifyInit(ctx, NULL, md, NULL, pkey))
 			break;
-		if (EVP_PKEY_verify(ctx, sig, siglen, md, mdlen) <= 0)
+
+		if (EVP_DigestVerify(ctx, sig, siglen, data, datalen) != 1)
 			break;
+
 		verify = true;
 	} while (0);
+
 	if (ctx)
-		EVP_PKEY_CTX_free(ctx);
+		EVP_MD_CTX_free(ctx);
 #endif
 	if (EVP_PKEY_id(pkey) == EVP_PKEY_RSA && EVP_PKEY_isPrivKey(pkey)) {
 		RSA *rsa = EVP_PKEY_get0_RSA(pkey);
