@@ -10,8 +10,6 @@
 
 #include <QString>
 #include <QProgressBar>
-#include <openssl/rsa.h>
-#include <openssl/bn.h>
 #include <openssl/pem.h>
 #include <openssl/evp.h>
 #include "pki_key.h"
@@ -19,6 +17,8 @@
 
 #define VIEW_private_ownpass 9
 
+class pass_info;
+
 class pki_evp: public pki_key
 {
 		Q_OBJECT
@@ -34,7 +34,7 @@ class pki_evp: public pki_key
 		void set_EVP_PKEY(EVP_PKEY *pkey, QString name = QString());
 
 	protected:
-		void openssl_pw_error(QString fname);
+		bool openssl_pw_error() const;
 	public:
 		static QString passHash;
 		static Passwd passwd;
@@ -42,26 +42,26 @@ class pki_evp: public pki_key
 		static QString md5passwd(QByteArray pass);
 		static QString sha512passwd(QByteArray pass, QString salt);
 		static QString sha512passwT(QByteArray pass, QString salt);
-		void generate(int bits, int type, QProgressBar *progress);
-		void generate(int bits, int type, QProgressBar *progress,
-				int curve_nid);
-		void setOwnPass(enum passType);
-		pki_evp(const QString name = "", int type = EVP_PKEY_RSA);
+
+		pki_evp(const QString &n = QString(), int type = EVP_PKEY_RSA);
+		pki_evp(const pki_evp *pkey);
 		pki_evp(EVP_PKEY *pkey);
+		virtual ~pki_evp();
+
+		void generate(const keyjob &task);
+		void setOwnPass(enum passType);
 		void set_evp_key(EVP_PKEY *pkey);
 		void encryptKey(const char *password = NULL);
 		void bogusEncryptKey();
 		EVP_PKEY *decryptKey() const;
 		EVP_PKEY *legacyDecryptKey(QByteArray &myencKey,
 					   Passwd &ownPassBuf) const;
-		pki_evp(const pki_evp *pk);
-		/* destructor */
-		virtual ~pki_evp();
-
 		EVP_PKEY *priv2pub(EVP_PKEY* key);
 		static QString removeTypeFromIntName(QString n);
 		void fromPEMbyteArray(const QByteArray &ba, const QString &name);
 		void fload(const QString &fname);
+		EVP_PKEY *load_ssh_ed25519_privatekey(const QByteArray &ba,
+						const pass_info &p);
 		void writeDefault(const QString &dirname) const;
 		void fromData(const unsigned char *p, db_header_t *head);
 		void writeKey(XFile &file, const EVP_CIPHER *enc,
