@@ -7,20 +7,21 @@
 
 
 #include "pki_key.h"
+#include "pki_x509super.h"
 #include "func.h"
 #include "db.h"
+#include "pkcs11.h"
+#include "widgets/XcaWarning.h"
+#include "widgets/ExportDialog.h"
+
 #include <openssl/rand.h>
 #include <openssl/pem.h>
-#include <QProgressDialog>
-#include <QApplication>
-#include <QDir>
-#include "widgets/PwDialog.h"
 
 #include "openssl_compat.h"
 
-builtin_curves pki_key::builtinCurves;
+builtin_curves builtinCurves;
 
-pki_key::pki_key(const QString name)
+pki_key::pki_key(const QString &name)
         :pki_base(name)
 {
 	key = EVP_PKEY_new();
@@ -30,7 +31,7 @@ pki_key::pki_key(const QString name)
 }
 
 pki_key::pki_key(const pki_key *pk)
-	:pki_base(pk->desc)
+	:pki_base(pk)
 {
 	if (pk->key) {
 		QByteArray ba = i2d_bytearray(I2D_VOID(i2d_PUBKEY), pk->key);
@@ -91,21 +92,48 @@ QByteArray pki_key::i2d() const
         return i2d_bytearray(I2D_VOID(i2d_PUBKEY), key);
 }
 
-BIO *pki_key::pem(BIO *b, int format)
+void pki_key::write_SSH2_ed25519_private(BIO *b,
+			 const EVP_PKEY *pkey, const EVP_CIPHER *enc) const
+{
+#ifndef OPENSSL_NO_EC
+	static const char data0001[] = { 0, 0, 0, 1};
+	char buf_nonce[8];
+	QByteArray data, priv, pubfull;
+	(void)enc;
+
+	pubfull = SSH2publicQByteArray(true);
+	RAND_bytes((unsigned char*)buf_nonce, sizeof buf_nonce);
+	priv.append(buf_nonce, sizeof buf_nonce);
+	priv += pubfull;
+	ssh_key_QBA2data(ed25519PrivKey(pkey) + ed25519PubKey(), &priv);
+
+	data = "openssh-key-v1";
+	data.append('\0');
+	ssh_key_QBA2data("none", &data); // enc-alg
+	ssh_key_QBA2data("none", &data); // KDF name
+	ssh_key_QBA2data("", &data); // KDF data
+	data.append(data0001, sizeof data0001);
+	ssh_key_QBA2data(pubfull, &data);
+	ssh_key_QBA2data(priv, &data);
+
+	PEM_write_bio(b, PEM_STRING_OPENSSH_KEY, (char*)"",
+		(unsigned char*)(data.data()), data.size());
+	pki_openssl_error();
+#endif
+}
+
+bool pki_key::pem(BioByteArray &b, int format)
 {
 	EVP_PKEY *pkey;
 	QByteArray ba;
 	int keytype;
 
-	if (!b)
-		b = BIO_new(BIO_s_mem());
-
 	switch (format) {
 	case exportType::SSH2_public:
-		ba = SSH2publicQByteArray();
-		BIO_write(b, ba.data(), ba.size());
+		b += SSH2publicQByteArray();
 		break;
 	case exportType::PEM_private:
+	case exportType::SSH2_private:
 		pkey = decryptKey();
 		keytype = EVP_PKEY_id(pkey);
 		switch (keytype) {
@@ -125,6 +153,13 @@ BIO *pki_key::pem(BIO *b, int format)
 				EVP_PKEY_get0_EC_KEY(pkey),
 				NULL, NULL, 0, NULL, NULL);
 			break;
+#ifdef EVP_PKEY_ED25519
+		case EVP_PKEY_ED25519:
+			if (format == exportType::PEM_private)
+				return false;
+			write_SSH2_ed25519_private(b, pkey, NULL);
+			break;
+#endif
 #endif
 		}
 		EVP_PKEY_free(pkey);
@@ -137,8 +172,23 @@ BIO *pki_key::pem(BIO *b, int format)
 	case exportType::PEM_key:
 		PEM_write_bio_PUBKEY(b, key);
 		break;
+	default:
+		return false;
 	}
-	return b;
+	return true;
+}
+
+void pki_key::writeSSH2private(XFile &file, pem_password_cb *cb) const
+{
+	(void)cb;
+//	pass_info p(XCA_TITLE, tr("Please enter the password protecting the SSH2 private key '%1'").arg(getIntName()));
+
+	EVP_PKEY *pkey = decryptKey();
+	if (!pkey) {
+		pki_openssl_error();
+		return;
+	}
+	write_SSH2_ed25519_private(file.bio(), pkey, NULL);
 }
 
 QString pki_key::length() const
@@ -164,22 +214,7 @@ QString pki_key::length() const
  */
 QString pki_key::getKeyTypeString() const
 {
-	QString type;
-
-	switch (EVP_PKEY_type(getKeyType())) {
-		case EVP_PKEY_RSA:
-			type = "RSA";
-			break;
-		case EVP_PKEY_DSA:
-			type = "DSA";
-			break;
-		case EVP_PKEY_EC:
-			type = "EC";
-			break;
-		default:
-			type = "---";
-	}
-	return type;
+	return keytype::byPKEY(key).name;
 }
 
 QString pki_key::getTypeString() const
@@ -249,7 +284,7 @@ int pki_key::getUcount() const
 		size = q.value(0).toInt();
 	else
 		qDebug("Failed to get key count for %s", CCHAR(getIntName()));
-	MainWindow::dbSqlError(q.lastError());
+	XCA_SQLERROR(q.lastError());
 	useCount = size;
 	return size;
 }
@@ -337,6 +372,43 @@ QString pki_key::ecPubKey() const
 	}
 	return pub;
 }
+
+#ifdef EVP_PKEY_ED25519
+static QByteArray ed25519Key(int(*EVP_PKEY_get_raw)
+			(const EVP_PKEY*, unsigned char *, size_t *),
+			const EVP_PKEY *pkey)
+{
+	unsigned char k[ED25519_KEYLEN];
+	size_t len = sizeof k;
+
+	if (EVP_PKEY_id(pkey) == EVP_PKEY_ED25519 &&
+	    EVP_PKEY_get_raw(pkey, k, &len))
+		return QByteArray((char*)k, len);
+	return QByteArray();
+}
+
+QByteArray pki_key::ed25519PubKey() const
+{
+	return ed25519Key(EVP_PKEY_get_raw_public_key, key);
+}
+
+QByteArray pki_key::ed25519PrivKey(const EVP_PKEY *pkey) const
+{
+	return ed25519Key(EVP_PKEY_get_raw_private_key, pkey);
+}
+#else
+
+QByteArray pki_key::ed25519PubKey() const
+{
+	return QByteArray();
+}
+
+QByteArray pki_key::ed25519PrivKey(const EVP_PKEY *) const
+{
+	return QByteArray();
+}
+
+#endif
 #endif
 
 QList<int> pki_key::possibleHashNids()
@@ -461,9 +533,10 @@ QSqlError pki_key::insertSqlData()
 	q.exec();
 	if (q.lastError().isValid())
 		return q.lastError();
+
 	while (q.next()) {
 		pki_x509super *x;
-		x = db_base::lookupPki<pki_x509super>(q.value(0));
+		x = Store.lookupPki<pki_x509super>(q.value(0));
 		if (!x) {
 			qDebug("X509 Super class with id %d not found",
 				q.value(0).toInt());
@@ -490,7 +563,7 @@ QSqlError pki_key::insertSqlData()
 	SQL_PREPARE(q, "INSERT INTO public_keys (item, type, hash, len, \"public\") "
 		  "VALUES (?, ?, ?, ?, ?)");
 	q.bindValue(0, sqlItemId);
-	q.bindValue(1, getKeyTypeString());
+	q.bindValue(1, getKeyTypeString().left(4));
 	q.bindValue(2, myhash);
 	q.bindValue(3, EVP_PKEY_bits(key));
 	q.bindValue(4, i2d_b64());
@@ -570,7 +643,11 @@ EVP_PKEY *pki_key::load_ssh2_key(XFile &file)
 	EVP_PKEY *pk = NULL;
 
 	ba = file.read(4096);
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
+	sl = QString(ba).split(" ", Qt::SkipEmptyParts);
+#else
 	sl = QString(ba).split(" ", QString::SkipEmptyParts);
+#endif
 	if (sl.size() < 2)
 		return NULL;
 
@@ -622,6 +699,14 @@ EVP_PKEY *pki_key::load_ssh2_key(XFile &file)
 		pk = EVP_PKEY_new();
 		check_oom(pk);
 		EVP_PKEY_assign_EC_KEY(pk, ec);
+#ifdef EVP_PKEY_ED25519
+	} else if (sl[0].startsWith("ssh-ed25519")) {
+		ssh_key_check_chunk(&ba, "ssh-ed25519");
+		QByteArray pub = ssh_key_next_chunk(&ba);
+		pk = EVP_PKEY_new_raw_public_key(EVP_PKEY_ED25519, NULL,
+			(const unsigned char *)pub.constData(), pub.count());
+		pki_openssl_error();
+#endif
 #endif
 	} else {
 		throw errorEx(tr("Unexpected SSH2 content: '%1'").arg(sl[0]));
@@ -661,9 +746,10 @@ bool pki_key::SSH2_compatible() const
 	switch (getKeyType()) {
 #ifndef OPENSSL_NO_EC
 	case EVP_PKEY_EC:
-		if (ecParamNid() != NID_X9_62_prime256v1)
-			break;
-		/* fall */
+		return ecParamNid() == NID_X9_62_prime256v1;
+#ifdef EVP_PKEY_ED25519
+	case EVP_PKEY_ED25519:
+#endif
 #endif
 	case EVP_PKEY_RSA:
 	case EVP_PKEY_DSA:
@@ -716,6 +802,13 @@ QByteArray pki_key::SSH2publicQByteArray(bool raw) const
 		}
 		pki_openssl_error();
 		break;
+#ifdef EVP_PKEY_ED25519
+	case EVP_PKEY_ED25519:
+		txt = "ssh-ed25519";
+		ssh_key_QBA2data(txt, &data);
+		ssh_key_QBA2data(ed25519PubKey(), &data);
+		break;
+#endif
 #endif
 	default:
 		return QByteArray();
@@ -757,6 +850,14 @@ bool pki_key::verify(EVP_PKEY *pkey) const
 	case EVP_PKEY_EC:
 		verify = EC_KEY_check_key(EVP_PKEY_get0_EC_KEY(pkey)) == 1;
 		break;
+#ifdef EVP_PKEY_ED25519
+	case EVP_PKEY_ED25519: {
+		size_t len;
+		verify = EVP_PKEY_get_raw_private_key(pkey, NULL, &len) == 1 &&
+				len == ED25519_KEYLEN;
+		break;
+	}
+#endif
 #endif
 	default:
 		verify = false;
@@ -828,3 +929,36 @@ void pki_key::PEM_file_comment(XFile &file) const
 	file.write(QString("%1 %2\n").arg(length(), getTypeString())
 			.toUtf8());
 }
+
+void pki_key::collect_properties(QMap<QString, QString> &prp) const
+{
+	QStringList sl;
+	sl << getTypeString() << length();
+	if (isPubKey())
+		sl << tr("Public key");
+#ifndef OPENSSL_NO_EC
+	if (getKeyType() == EVP_PKEY_EC)
+		sl << QString(OBJ_nid2ln(ecParamNid()));
+#endif
+	prp["Key"] = sl.join(" ");
+	pki_base::collect_properties(prp);
+}
+
+void pki_key::print(BioByteArray &bba, enum print_opt opt) const
+{
+	pki_base::print(bba, opt);
+	switch (opt) {
+	case print_openssl_txt:
+#if OPENSSL_VERSION_NUMBER < 0x10000000L
+		bba += "Not supported\n";
+#else
+		EVP_PKEY_print_public(bba, key, 0, NULL);
+#endif
+		break;
+	case print_pem:
+		PEM_write_bio_PUBKEY(bba, key);
+		break;
+	case print_coloured:
+		break;
+	}
+}
