@@ -9,11 +9,8 @@
 #ifndef __PKI_MULTI_H
 #define __PKI_MULTI_H
 
-#include <openssl/pem.h>
-#include "pki_x509.h"
-#include "x509name.h"
-#include "asn1time.h"
-#include "asn1int.h"
+#include <QStringList>
+#include "pki_base.h"
 
 class pki_multi: public pki_base
 {
@@ -21,16 +18,15 @@ class pki_multi: public pki_base
 	protected:
 		QList<pki_base*> multi;
 	public:
-		pki_multi(const QString name = "");
+		pki_multi(const QString &name = "");
 		~pki_multi();
+		QStringList failed_files;
 		void fromPEMbyteArray(const QByteArray &, const QString &);
 		void fload(const QString &fname);
 		void probeAnything(const QString &fname);
-		pki_base *pull();
-		int count() const
-		{
-			return multi.count();
-		}
+		void append_item(pki_base *pki);
+		void print(BioByteArray &b, enum print_opt opt) const;
+		QList<pki_base *> pull();
+		QList<pki_base *> get() const;
 };
-
 #endif
