@@ -86,8 +86,7 @@ class pki_scard: public pki_key
 		void setMech_list(QList<CK_MECHANISM_TYPE> ml) { mech_list = ml; };
 		QList<int> possibleHashNids();
 		EVP_PKEY *load_pubkey(pkcs11 &p11, CK_OBJECT_HANDLE object) const;
-		void generateKey_card(int type, const slotid &slot, int size,
-					int curve_nid, QProgressBar *bar);
+		void generate(const keyjob &task);
 		void deleteFromToken();
 		void deleteFromToken(const slotid &slot);
 		void store_token(const slotid &slot, EVP_PKEY *pkey);
@@ -98,5 +97,4 @@ class pki_scard: public pki_key
 		QSqlError deleteSqlData();
 		void restoreSql(const QSqlRecord &rec);
 };
-
 #endif
