@@ -7,19 +7,17 @@
 
 
 #include "pki_x509.h"
+#include "pki_x509req.h"
 #include "pki_evp.h"
 #include "pki_scard.h"
+#include "pki_crl.h"
 #include "db_base.h"
 #include "func.h"
 #include "base.h"
 #include "exception.h"
 #include "pass_info.h"
-#include "widgets/MainWindow.h"
-#include <QMessageBox>
-#include <QDir>
-#include <openssl/rand.h>
-
 #include "openssl_compat.h"
+#include "widgets/XcaWarning.h"
 
 pki_x509::pki_x509(X509 *c)
 	:pki_x509super()
@@ -30,7 +28,7 @@ pki_x509::pki_x509(X509 *c)
 }
 
 pki_x509::pki_x509(const pki_x509 *crt)
-	:pki_x509super(crt->desc)
+	:pki_x509super(crt)
 {
 	init();
 	cert = X509_dup(crt->cert);
@@ -44,7 +42,7 @@ pki_x509::pki_x509(const pki_x509 *crt)
 	pki_openssl_error();
 }
 
-pki_x509::pki_x509(const QString name)
+pki_x509::pki_x509(const QString &name)
 	:pki_x509super(name)
 {
 	init();
@@ -72,6 +70,17 @@ QString pki_x509::getMsg(msg_type msg) const
 	return pki_base::getMsg(msg);
 }
 
+void pki_x509::resetX509ReqCount() const
+{
+	QList<pki_x509req *> reqs = Store.sqlSELECTpki<pki_x509req>(
+		"SELECT item FROM x509super LEFT JOIN items ON items.id = x509super.item "
+		"WHERE key_hash=? AND items.type=?",
+		QList<QVariant>() << QVariant(pubHash()) << QVariant(x509_req));
+
+	foreach(pki_x509req *req, reqs)
+		req->resetX509count();
+}
+
 QSqlError pki_x509::insertSqlData()
 {
 	XSqlQuery q;
@@ -92,7 +101,9 @@ QSqlError pki_x509::insertSqlData()
 	q.bindValue(5, (int)isCA());
 	q.bindValue(6, i2d_b64());
 	q.exec();
-	MainWindow::reqs->resetX509count();
+
+	resetX509ReqCount();
+
 	if (!isCA())
 		return q.lastError();
 
@@ -146,17 +157,17 @@ QSqlError pki_x509::deleteSqlData()
 			return e;
 	}
 	// Select affected items
-	QList<pki_base*> list = db_base::sqlSELECTpki<pki_base>(
+	q = Store.sqlSELECTpki(
 		"SELECT DISTINCT items.id FROM items, certs, crls "
 		"WHERE (items.id = certs.item OR items.id = crls.item) "
 		"AND crls.issuer = ? AND certs.issuer = ?",
 		QList<QVariant>() << QVariant(sqlItemId)
 				  << QVariant(sqlItemId));
 
-	foreach(pki_base *pki, list)
-		AffectedItems(pki->getSqlItemId());
+	while (q.next())
+		AffectedItems(q.value(0));
 
-	MainWindow::reqs->resetX509count();
+	resetX509ReqCount();
 	return q.lastError();
 }
 
@@ -180,7 +191,7 @@ pki_x509 *pki_x509::findIssuer()
 	q.bindValue(0, hash);
 	q.exec();
 	while (q.next()) {
-		issuer = db_base::lookupPki<pki_x509>(q.value(0));
+		issuer = Store.lookupPki<pki_x509>(q.value(0));
 		if (!issuer) {
 			qDebug("Certificate with id %d not found",
                                 q.value(0).toInt());
@@ -670,13 +681,9 @@ QString pki_x509::getIndexEntry()
 		QString(X509_NAME_oneline(getSubject().get(), NULL, 0)));
 }
 
-BIO *pki_x509::pem(BIO *b, int format)
+bool pki_x509::pem(BioByteArray &b, int)
 {
-	(void)format;
-	if (!b)
-		b = BIO_new(BIO_s_mem());
-	PEM_write_bio_X509(b, cert);
-	return b;
+	return PEM_write_bio_X509(b, cert);
 }
 
 bool pki_x509::cmpIssuerAndSerial(pki_x509 *refcert)
@@ -687,10 +694,10 @@ bool pki_x509::cmpIssuerAndSerial(pki_x509 *refcert)
 
 }
 
-bool pki_x509::verify_only(pki_x509 *signer)
+bool pki_x509::verify_only(const pki_x509 *signer) const
 {
-	X509_NAME *subject = X509_get_subject_name(signer->cert);
-	X509_NAME *issuer = X509_get_issuer_name(cert);
+	const X509_NAME *subject = X509_get_subject_name(signer->cert);
+	const X509_NAME *issuer = X509_get_issuer_name(cert);
 	pki_openssl_error();
 	if (X509_NAME_cmp(subject, issuer)) {
 		return false;
@@ -701,6 +708,7 @@ bool pki_x509::verify_only(pki_x509 *signer)
 		return false;
 	}
 	int i = X509_verify(cert, pub);
+	EVP_PKEY_free(pub);
 	pki_ign_openssl_error();
 	return i>0;
 }
@@ -860,7 +868,7 @@ int pki_x509::sigAlg() const
 
 pki_x509 *pki_x509::getSigner()
 {
-	return db_base::lookupPki<pki_x509>(issuerSqlId);
+	return Store.lookupPki<pki_x509>(issuerSqlId);
 }
 
 bool pki_x509::isRevoked() const
@@ -947,10 +955,36 @@ QStringList pki_x509::icsVEVENT() const
 			.arg(getIntName())
 			.arg(getNotBefore().toPretty())
 			.arg(getNotAfter().toPretty())
-			.arg(currentDB)
+			.arg(Database.name())
 	);
 }
 
+void pki_x509::collect_properties(QMap<QString, QString> &prp) const
+{
+	prp["Issuer"] = getIssuerName().oneLine(XN_FLAG_RFC2253);
+	prp["Serial"] = getSerial().toHex();
+	prp["CA"] = isCA() ? "Yes" : "No";
+	prp["Not Before"] = getNotBefore().toPretty();
+	prp["Not After"] = getNotAfter().toPretty();
+	prp["Self signed"] = verify_only(this) ? "Yes" : "No";
+	pki_x509super::collect_properties(prp);
+}
+
+void pki_x509::print(BioByteArray &bba, enum print_opt opt) const
+{
+	pki_x509super::print(bba, opt);
+	switch (opt) {
+	case print_openssl_txt:
+		X509_print(bba, cert);
+		break;
+	case print_pem:
+		PEM_write_bio_X509(bba, cert);
+		break;
+	case print_coloured:
+		break;
+	}
+}
+
 QStringList pki_x509::icsVEVENT_ca() const
 {
 	QStringList ics;
@@ -963,7 +997,7 @@ QStringList pki_x509::icsVEVENT_ca() const
 			ics << pki->icsVEVENT();
 	}
 
-	QList<pki_crl*> list = db_base::sqlSELECTpki<pki_crl>(
+	QList<pki_crl*> list = Store.sqlSELECTpki<pki_crl>(
 		"SELECT item FROM crls WHERE issuer = ?",
 		QList<QVariant>() << QVariant(sqlItemId));
 
