@@ -8,16 +8,11 @@
 #ifndef __PKI_X509_H
 #define __PKI_X509_H
 
-#include <stdio.h>
 #include <openssl/x509.h>
 #include <openssl/x509v3.h>
 #include <openssl/pem.h>
-#include "pki_temp.h"
-#include "pki_key.h"
-#include "pki_x509req.h"
+#include "database_model.h"
 #include "pki_x509super.h"
-#include "asn1time.h"
-#include "asn1int.h"
 #include "x509rev.h"
 #include "x509v3ext.h"
 #include "pkcs11.h"
@@ -33,6 +28,8 @@
 #define VIEW_x509_auth_dnPolicy_UNUSED 15
 #define VIEW_x509_revocation 16
 
+class pki_key;
+
 class pki_x509 : public pki_x509super
 {
 		Q_OBJECT
@@ -46,14 +43,16 @@ class pki_x509 : public pki_x509super
 		void init();
 		x509rev revocation;
 		x509revList fromDataRevList;
+		void resetX509ReqCount() const;
 
 	protected:
 		int sigAlg() const;
+		void collect_properties(QMap<QString, QString> &prp) const;
 
 	public:
 		pki_x509(X509 *c);
 		pki_x509(const pki_x509 *crt);
-		pki_x509(const QString name = "");
+		pki_x509(const QString &name = QString());
 		~pki_x509();
 
 		void setSigner(pki_x509 *s)
@@ -84,7 +83,7 @@ class pki_x509 : public pki_x509super
 		void writeCert(XFile &file, bool PEM) const;
 		QString getIndexEntry();
 		bool verify(pki_x509 *signer);
-		bool verify_only(pki_x509 *signer);
+		bool verify_only(const pki_x509 *signer) const;
 		pki_key *getPubKey() const;
 		void setPubKey(pki_key *key);
 		pki_x509 *getSigner();
@@ -139,6 +138,7 @@ class pki_x509 : public pki_x509super
 		bool cmpIssuerAndSerial(pki_x509 *refcert);
 		bool visible() const;
 		void updateView();
+		void print(BioByteArray &b, enum print_opt opt) const;
 		x509v3ext getExtByNid(int nid) const;
 		QVariant column_data(const dbheader *hd) const;
 		QVariant getIcon(const dbheader *hd) const;
@@ -149,7 +149,7 @@ class pki_x509 : public pki_x509super
 		void deleteFromToken(const slotid &slot);
 		QString getMsg(msg_type msg) const;
 		int renameOnToken(const slotid &slot, const QString &name);
-		BIO *pem(BIO *, int);
+		bool pem(BioByteArray &, int);
 		QVariant bg_color(const dbheader *hd) const;
 		void mergeRevList(x509revList &l);
 		void setRevocations(const x509revList &rl);
