@@ -9,18 +9,17 @@
 #include <typeinfo>
 
 #include "pki_x509.h"
+#include "pki_x509req.h"
 #include "pki_evp.h"
 #include "func.h"
 #include "db_base.h"
 #include "x509name.h"
 #include "exception.h"
 #include <openssl/bio.h>
-#include <openssl/err.h>
-#include <QDir>
 
 #include "openssl_compat.h"
 
-pki_x509req::pki_x509req(const QString name)
+pki_x509req::pki_x509req(const QString &name)
 	: pki_x509super(name)
 {
 	request = X509_REQ_new();
@@ -30,6 +29,15 @@ pki_x509req::pki_x509req(const QString name)
 	resetX509count();
 }
 
+pki_x509req::pki_x509req(const pki_x509req *req)
+	: pki_x509super(req)
+{
+	request = X509_REQ_dup(request);
+	done = req->done;
+	resetX509count();
+	pki_openssl_error();
+}
+
 pki_x509req::~pki_x509req()
 {
 	if (request)
@@ -257,16 +265,12 @@ void pki_x509req::writeReq(XFile &file, bool pem) const
 	pki_openssl_error();
 }
 
-BIO *pki_x509req::pem(BIO *b, int format)
+bool pki_x509req::pem(BioByteArray &b, int)
 {
-	(void)format;
-	if (!b)
-		b = BIO_new(BIO_s_mem());
-	PEM_write_bio_X509_REQ(b, request);
-	return b;
+	return PEM_write_bio_X509_REQ(b, request);
 }
 
-int pki_x509req::verify() const
+bool pki_x509req::verify() const
 {
 	EVP_PKEY *pkey = X509_REQ_get_pubkey(request);
 	bool x = X509_REQ_verify(request,pkey) > 0;
@@ -323,24 +327,18 @@ int pki_x509req::issuedCerts() const
 	if (x509count != -1)
 		return x509count;
 
-	SQL_PREPARE(q, "SELECT item FROM x509super WHERE key_hash=?");
-	q.bindValue(0, pubHash());
-	q.exec();
-	if (q.lastError().isValid())
-		return 0;
 	pki_key *k = getPubKey();
 	if (!k)
 		return 0;
-	while (q.next()) {
-		pki_x509super *x;
-		x = db_base::lookupPki<pki_x509super>(q.value(0));
-		if (!x) {
-			qDebug("x509 with id %d not found",
-				q.value(0).toInt());
-			continue;
-		}
-		if (typeid(*x) == typeid(pki_x509) && x->compareRefKey(k))
+	QList<pki_x509 *> certs = Store.sqlSELECTpki<pki_x509>(
+		"SELECT item FROM x509super LEFT JOIN items ON items.id = x509super.item "
+		"WHERE key_hash=? AND items.type=?",
+		QList<QVariant>() << QVariant(pubHash()) << QVariant(x509));
+
+	foreach(pki_x509 *x, certs) {
+		if (x->compareRefKey(k))
 			count++;
+
 		qDebug() << "Req:" << getIntName() << "Cert with hash"
 			 << x->getIntName() << count;
 	}
@@ -349,6 +347,35 @@ int pki_x509req::issuedCerts() const
 	return count;
 }
 
+void pki_x509req::collect_properties(QMap<QString, QString> &prp) const
+{
+	QString s = getAttribute(NID_pkcs9_unstructuredName);
+	if (!s.isEmpty())
+		prp["Unstructured Name"] = s;
+
+	s = getAttribute(NID_pkcs9_challengePassword);
+	if (!s.isEmpty())
+		prp["Challange Password"] = s;
+
+	pki_x509super::collect_properties(prp);
+	prp["Verify Ok"] = verify() ? "Yes" : "No";
+}
+
+void pki_x509req::print(BioByteArray &bba, enum print_opt opt) const
+{
+	pki_x509super::print(bba, opt);
+	switch (opt) {
+	case print_openssl_txt:
+		X509_REQ_print(bba, request);
+		break;
+	case print_pem:
+		PEM_write_bio_X509_REQ(bba, request);
+		break;
+	case print_coloured:
+		break;
+	}
+}
+
 QVariant pki_x509req::column_data(const dbheader *hd) const
 {
 	switch (hd->id) {
