@@ -10,11 +10,17 @@
 #include "pki_x509super.h"
 #include "db_base.h"
 
-pki_x509super::pki_x509super(const QString name)
+pki_x509super::pki_x509super(const QString &name)
 	: pki_x509name(name)
 {
 }
 
+pki_x509super::pki_x509super(const pki_x509super *x)
+	: pki_x509name(x)
+{
+	keySqlId = x->keySqlId;
+}
+
 pki_x509super::~pki_x509super()
 {
 }
@@ -30,7 +36,7 @@ QSqlError pki_x509super::lookupKey()
 	if (q.lastError().isValid())
 		return q.lastError();
 	while (q.next()) {
-		pki_key *x = db_base::lookupPki<pki_key>(q.value(0));
+		pki_key *x = Store.lookupPki<pki_key>(q.value(0));
 		if (!x) {
 			qDebug("Public key with id %d not found",
 				q.value(0).toInt());
@@ -65,7 +71,7 @@ QSqlError pki_x509super::insertSqlData()
 void pki_x509super::restoreSql(const QSqlRecord &rec)
 {
 	pki_base::restoreSql(rec);
-	keySqlId = rec.value(VIEW_x509super_keyid);
+	keySqlId = rec.value(VIEW_x509super_keyid).toULongLong();
 }
 
 QSqlError pki_x509super::deleteSqlData()
@@ -82,7 +88,7 @@ QSqlError pki_x509super::deleteSqlData()
 
 pki_key *pki_x509super::getRefKey() const
 {
-	return db_base::lookupPki<pki_key>(keySqlId);
+	return Store.lookupPki<pki_key>(keySqlId);
 }
 
 unsigned pki_x509super::pubHash() const
@@ -117,7 +123,7 @@ bool pki_x509super::compareRefKey(pki_key *ref) const
 
 void pki_x509super::setRefKey(pki_key *ref)
 {
-	keySqlId = ref ? ref->getSqlItemId() : QVariant();
+	keySqlId = ref ? ref->sqlItemId : QVariant();
 }
 
 QString pki_x509super::getSigAlg() const
@@ -173,7 +179,7 @@ QVariant pki_x509super::column_data(const dbheader *hd) const
 		int idx = el.idxByNid(hd->id);
 		if (idx == -1)
 			return QVariant("");
-		return QVariant(el[idx].getValue(false));
+		return QVariant(el[idx].getConsoleValue(""));
 	}
 	return pki_x509name::column_data(hd);
 }
@@ -237,13 +243,29 @@ bool pki_x509super::visible() const
 	return getV3ext().search(limitPattern);
 }
 
-// Start class  pki_x509name
+void pki_x509super::collect_properties(QMap<QString, QString> &prp) const
+{
+	pki_key *key = getPubKey();
+	if (key)
+		key->collect_properties(prp);
+	delete key;
 
-pki_x509name::pki_x509name(const QString name)
+	prp["Signature"] = getSigAlg();
+	prp["Extensions"] = getV3ext().getConsole("    ");
+	pki_x509name::collect_properties(prp);
+}
+
+// Start class  pki_x509name
+pki_x509name::pki_x509name(const QString &name)
 	: pki_base(name)
 {
 }
 
+pki_x509name::pki_x509name(const pki_x509name *n)
+	: pki_base(n)
+{
+}
+
 void pki_x509name::autoIntName(const QString &file)
 {
 	QString name = getSubject().getMostPopular();
@@ -281,3 +303,9 @@ void pki_x509name::PEM_file_comment(XFile &file) const
 	pki_base::PEM_file_comment(file);
 	file.write(getSubject().oneLine(XN_FLAG_RFC2253).toUtf8() + "\n");
 }
+
+void pki_x509name::collect_properties(QMap<QString, QString> &prp) const
+{
+	prp["Subject"] = getSubject().oneLine(XN_FLAG_RFC2253);
+	pki_base::collect_properties(prp);
+}
