@@ -15,6 +15,7 @@
 #include <QStringList>
 #include <QDebug>
 #include "base.h"
+#include "BioByteArray.h"
 
 x509v3ext::x509v3ext()
 {
@@ -141,29 +142,32 @@ ASN1_OCTET_STRING *x509v3ext::getData() const
 	return X509_EXTENSION_get_data(ext);
 }
 
-QString x509v3ext::getValue(bool html) const
+QString x509v3ext::getValue() const
 {
-	QString text = "";
-	int ret;
-	char *p = NULL;
-	BIO *bio = BIO_new(BIO_s_mem());
+	BioByteArray bba;
+	int ret = X509V3_EXT_print(bba, ext, X509V3_EXT_DEFAULT, 0);
+	if (ign_openssl_error() || !ret)
+		ret = ASN1_STRING_print(bba, (ASN1_STRING *)getData());
+	if (ign_openssl_error() || !ret)
+		return QString();
+	return bba.qstring().trimmed();
+}
 
-	ret = X509V3_EXT_print(bio, ext, X509V3_EXT_DEFAULT, 0);
-	if (ign_openssl_error() || !ret) {
-		ret = ASN1_STRING_print(bio, (ASN1_STRING *) getData());
-	}
-	if (!ign_openssl_error() && ret) {
-		long len = BIO_get_mem_data(bio, &p);
-		text = QString::fromLocal8Bit(p, len);
-	}
-	BIO_free(bio);
-	if (html) {
-		text.replace(QRegExp("&"), "&amp;");
-		text.replace(QRegExp("<"), "&lt;");
-		text.replace(QRegExp(">"), "&gt;");
-		text.replace(QRegExp("\n"), "<br>\n");
-	}
-	return text.trimmed();
+QString x509v3ext::getHtmlValue() const
+{
+	QString text = getValue();
+	text.replace(QRegExp("&"), "&amp;");
+	text.replace(QRegExp("<"), "&lt;");
+	text.replace(QRegExp(">"), "&gt;");
+	text.replace(QRegExp("\n"), "<br>\n");
+	return text;
+}
+
+QString x509v3ext::getConsoleValue(const QString &indent) const
+{
+	QString text = getValue();
+	text.replace(QRegExp("\n"), QString("\n") + indent);
+	return text;
 }
 
 static QString vlist2Section(QStringList vlist, QString tag, QString *sect)
@@ -777,7 +781,7 @@ bool x509v3ext::parse_aKeyId(QString *, QString *adv) const
 	if (akeyid->keyid)
 		ret << "keyid";
 	if (akeyid->issuer)
-		ret << "issuer";
+		ret << "issuer:always";
 	if (adv)
 		*adv = QString("%1=%2\n").arg(OBJ_nid2sn(nid())).
 			arg(ret.join(", ")) + *adv;
@@ -975,10 +979,20 @@ QString x509v3ext::getHtml() const
 	html = "<b><u>" + getObject();
 	if (getCritical() != 0)
 		html += " <font color=\"red\">critical</font>";
-	html += ":</u></b><br><tt>" + getValue(true) + "</tt>";
+	html += ":</u></b><br><tt>" + getHtmlValue() + "</tt>";
 	return html;
 }
 
+QString x509v3ext::getConsole(const QString &indent) const
+{
+	QString text, twoind = indent + indent;
+	text = indent + COL_BOLD COL_UNDER + getObject();
+	if (getCritical() != 0)
+		text += " " COL_RED COL_UNDER "[critical]";
+	text += COL_RESET "\n" + twoind + getConsoleValue(twoind);
+	return text;
+}
+
 X509_EXTENSION *x509v3ext::get() const
 {
 	return X509_EXTENSION_dup(ext);
@@ -1028,18 +1042,16 @@ void extList::setStack(const STACK_OF(X509_EXTENSION) *st, int start)
 
 STACK_OF(X509_EXTENSION) *extList::getStack()
 {
-	STACK_OF(X509_EXTENSION) *sk;
-	sk = sk_X509_EXTENSION_new_null();
-	for (int i=0; i< count(); i++) {
-		sk_X509_EXTENSION_push(sk, operator[](i).get());
-	}
+	STACK_OF(X509_EXTENSION) *sk = sk_X509_EXTENSION_new_null();
+	foreach(const x509v3ext &e, *this)
+		sk_X509_EXTENSION_push(sk, e.get());
 	return sk;
 }
 
 bool extList::search(const QRegExp &pattern)
 {
-	for (int i=0; i < size(); i++)
-		if (at(i).getValue(false).contains(pattern))
+	foreach(const x509v3ext &e, *this)
+		if (e.getValue().contains(pattern))
 			return true;
 	return false;
 }
@@ -1047,12 +1059,23 @@ QString extList::getHtml(const QString &sep)
 {
 	x509v3ext e;
 	QStringList s;
-	for (int i=0; i< size(); i++)
-		s << at(i).getHtml();
+	foreach(const x509v3ext &e, *this)
+		s << e.getHtml();
 	QString a = s.join(sep);
 	return a;
 }
 
+QString extList::getConsole(const QString &indent) const
+{
+	x509v3ext e;
+	QStringList s;
+	foreach(const x509v3ext &e, *this)
+		s << e.getConsole(indent);
+	QString a = "\n" + s.join("\n");
+	return a;
+}
+
+
 bool extList::delByNid(int nid)
 {
 	for(int i = 0; i< size(); i++) {
