@@ -12,6 +12,9 @@
 #include <QDebug>
 #include <stdio.h>
 #include <unistd.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+
 #include <errno.h>
 #include "exception.h"
 
@@ -38,9 +41,8 @@ class XFile : public QFile
 		}
 		BIO *bio()
 		{
-			if (b)
-				BIO_free(b);
-			b = BIO_new_fp(fp(), BIO_NOCLOSE);
+			if (!b)
+				b = BIO_new_fp(fp(), BIO_NOCLOSE);
 			return b;
 		}
 		FILE *fp(const char *mode = NULL)
