@@ -10,7 +10,7 @@
 #include "lib/base.h"
 #include "lib/func.h"
 #include "widgets/validity.h"
-#include "widgets/MainWindow.h"
+#include "widgets/XcaWarning.h"
 #include <QLabel>
 #include <QLineEdit>
 #include <QComboBox>
@@ -42,10 +42,10 @@ void CertExtend::accept()
 	if (signer && notBefore->getDate() < signer->getNotBefore()) {
 		QString text = tr("The certificate will be earlier valid than the signer. This is probably not what you want.");
 		xcaWarning msg(this, text);
-		msg.addButton(QMessageBox::Ok)->setText(tr("Edit dates"));
-		msg.addButton(QMessageBox::Close)->setText(tr("Abort rollout"));
-		msg.addButton(QMessageBox::Apply)->setText(tr("Continue rollout"));
-		msg.addButton(QMessageBox::Yes)->setText(tr("Adjust date and continue"));
+		msg.addButton(QMessageBox::Ok, tr("Edit dates"));
+		msg.addButton(QMessageBox::Close, tr("Abort rollout"));
+		msg.addButton(QMessageBox::Apply, tr("Continue rollout"));
+		msg.addButton(QMessageBox::Yes, tr("Adjust date and continue"));
 		switch (msg.exec())
 		{
 			case QMessageBox::Ok:
@@ -64,10 +64,10 @@ void CertExtend::accept()
 				!noWellDefinedExpDate->isChecked()) {
 		QString text = tr("The certificate will be longer valid than the signer. This is probably not what you want.");
 		xcaWarning msg(this, text);
-		msg.addButton(QMessageBox::Ok)->setText(tr("Edit dates"));
-		msg.addButton(QMessageBox::Close)->setText(tr("Abort rollout"));
-		msg.addButton(QMessageBox::Apply)->setText(tr("Continue rollout"));
-		msg.addButton(QMessageBox::Yes)->setText(tr("Adjust date and continue"));
+		msg.addButton(QMessageBox::Ok, tr("Edit dates"));
+		msg.addButton(QMessageBox::Close, tr("Abort rollout"));
+		msg.addButton(QMessageBox::Apply, tr("Continue rollout"));
+		msg.addButton(QMessageBox::Yes, tr("Adjust date and continue"));
 		switch (msg.exec())
 		{
 			case QMessageBox::Ok:
