@@ -11,7 +11,6 @@
 #include "ui_ExportDialog.h"
 #include "lib/pki_base.h"
 
-class MainWindow;
 class QPixmap;
 
 class exportType {
@@ -22,7 +21,7 @@ class exportType {
 		PEM_key, PEM_private, PEM_private_encrypt, DER_private,
 		DER_key, PKCS8, PKCS8_encrypt, SSH2_public,
 		PEM_selected, PKCS7_selected, Index, vcalendar, vcalendar_ca,
-		PVK_private, PVK_encrypt, ETYPE_max };
+		PVK_private, PVK_encrypt, SSH2_private, ETYPE_max };
 	enum etype type;
 	QString desc;
 	QString extension;
@@ -42,6 +41,7 @@ class exportType {
 		case PEM_private:
 		case PEM_private_encrypt:
 		case PEM_selected:
+		case SSH2_private:
 			return true;
 		default:
 			return false;
@@ -56,12 +56,12 @@ class ExportDialog: public QDialog, public Ui::ExportDialog
 
    protected:
 	QString filter;
-	MainWindow *mainwin;
 	QVector<QString> help;
 
    public:
-	ExportDialog(QWidget *mw, const QString &title, const QString &filt,
-		     pki_base *pki, const QPixmap &img, QList<exportType> types);
+	ExportDialog(QWidget *w, const QString &title, const QString &filt,
+		     pki_base *pki, const QPixmap &img, QList<exportType> types,
+		     const QString &help_ctx = QString());
 	static bool mayWriteFile(const QString &fname);
 	enum exportType::etype type();
 
