@@ -6,31 +6,40 @@
  */
 
 
-#include "KeyDetail.h"
-#include "MainWindow.h"
 #include "lib/main.h"
 #include "lib/pki_evp.h"
 #include "lib/pki_scard.h"
-#include "widgets/distname.h"
-#include "widgets/clicklabel.h"
+
+#include "KeyDetail.h"
+#include "MainWindow.h"
+#include "Help.h"
+#include "distname.h"
+#include "clicklabel.h"
+#include "XcaApplication.h"
+#include "OidResolver.h"
+
 #include <QLabel>
 #include <QPushButton>
 #include <QLineEdit>
 
-KeyDetail::KeyDetail(QWidget *parent)
-	:QDialog(parent)
+KeyDetail::KeyDetail(QWidget *w)
+	: QDialog(w ?: mainwin) , keySqlId()
 {
 	setupUi(this);
 	setWindowTitle(XCA_TITLE);
 	image->setPixmap(QPixmap(":keyImg"));
-	keyModulus->setFont(XCA_application::tableFont);
+	mainwin->helpdlg->register_ctxhelp_button(this, "keydetail");
+
+	keyModulus->setFont(XcaApplication::tableFont);
 	tabWidget->setCurrentIndex(0);
+
+	Database.connectToDbChangeEvt(this, SLOT(itemChanged(pki_base*)));
 }
 
 #ifndef OPENSSL_NO_EC
 static QString CurveComment(int nid)
 {
-	foreach(builtin_curve curve, pki_key::builtinCurves) {
+	foreach(builtin_curve curve, builtinCurves) {
 		if (curve.nid == nid)
 			return curve.comment;
 	}
@@ -47,7 +56,7 @@ void KeyDetail::setupFingerprints(pki_key *key)
 	v->addStretch();
 	v->addWidget(widget);
 	v->addStretch();
-	
+
 	QStringList sl; sl <<
 		"ssh MD5" << "ssh SHA256 B64" <<
 		"x509 SHA1" << "DER SHA256";
@@ -70,6 +79,7 @@ void KeyDetail::setupFingerprints(pki_key *key)
 
 void KeyDetail::setKey(pki_key *key)
 {
+	keySqlId = key->getSqlItemId();
 	keyDesc->setText(key->getIntName());
 	keyLength->setText(key->length());
 
@@ -124,8 +134,46 @@ void KeyDetail::setKey(pki_key *key)
 			keyPubEx->setToolTip(CurveComment(nid));
 			keyModulus->setText(key->ecPubKey());
 			break;
+#ifdef EVP_PKEY_ED25519
+		case EVP_PKEY_ED25519:
+			tlModulus->setTitle(tr("Public key"));
+			tlPrivEx->setText(tr("Private key"));
+			tlPubEx->setText(tr("Curve name"));
+			keyPubEx->setText("ed25519");
+			keyModulus->setText(key->ed25519PubKey().toHex());
+			break;
+#endif
 #endif
 		default:
 			tlHeader->setText(tr("Unknown key"));
 	}
 }
+
+void KeyDetail::itemChanged(pki_base *pki)
+{
+	if (pki->getSqlItemId() == keySqlId)
+		keyDesc->setText(pki->getIntName());
+}
+
+void KeyDetail::showKey(QWidget *parent, pki_key *key, bool ro)
+{
+	if (!key)
+		return;
+	KeyDetail *dlg = new KeyDetail(parent);
+	if (!dlg)
+		return;
+	dlg->setKey(key);
+	dlg->keyDesc->setReadOnly(ro);
+	dlg->comment->setReadOnly(ro);
+	if (dlg->exec()) {
+		db_base *db = Database.modelForPki(key);
+		if (!db) {
+			key->setIntName(dlg->keyDesc->text());
+			key->setComment(dlg->comment->toPlainText());
+		} else {
+			db->updateItem(key, dlg->keyDesc->text(),
+					dlg->comment->toPlainText());
+		}
+	}
+        delete dlg;
+}
