@@ -40,7 +40,7 @@ const QList<QStringList> MainWindow::getTranslators() const
 	QStringList{ "pl", tr("Polish"),
 		"Jacek Tyborowski", "jacek", "tyborowski.pl" },
 	QStringList{ "pt_BR", tr("Portuguese in Brazil"),
-		"Vinicius Ocker", "viniciusockerfagundes", "yandex.com" },
+		"Ulisses Guedes", "uli1958", "hotmail.com" },
 	QStringList{ "ru", tr("Russian") },
 	QStringList{ "sk", tr("Slovak"),
 		"Slavko", "linux", "slavino.sk" },
@@ -50,30 +50,6 @@ const QList<QStringList> MainWindow::getTranslators() const
 	};
 };
 
-
-void MainWindow::cmd_version()
-{
-	fprintf(stderr, XCA_TITLE " Version %s\n", version_str(false));
-	exitApp = 1;
-}
-
-void MainWindow::cmd_help(const char* msg)
-{
-	fprintf(stderr, XCA_TITLE " Version %s\n"
-		"\n"
-		" -v show version information and exit\n"
-		" -h shows this help screen and exit\n"
-		" -d expect the following argument to be the database name to use\n"
-		" -i expect the following argument to be the index file to generate\n"
-		" -I expect the following argument to be the base name the index file hierarchy to generate\n"
-		" -x Exit after processing all commandline options\n\n", version_str(false));
-
-	if (msg) {
-		fprintf(stderr, "Cmdline Error: %s\n", msg);
-	}
-	exitApp = 1;
-}
-
 void MainWindow::about()
 {
 	QTextEdit *textbox = new QTextEdit(NULL);
@@ -134,7 +110,7 @@ void MainWindow::about()
 	Entropy::seed_rng();
 	cont = QString(
 	"<p><h3><center><u>XCA%8</u></center></h3>"
-	"<p>Copyright 2001 - 2020 by Christian Hohnstädt\n"
+	"<p>Copyright 2001 - 2021 by Christian Hohnstädt\n"
 	"<p>Version: %4<p>%1<p>%2" /* commithash, Brainpool, OpenSSL & Qt Version */
 	"<p><a href=\"https://hohnstaedt.de/xca\">https://hohnstaedt.de/xca</a>"
 	"<p>Entropy strength: %3"
@@ -153,9 +129,9 @@ void MainWindow::about()
 			.arg(version)
 			.arg(Entropy::strength())
 			.arg(version_str(true))
-			.arg(getPrefix())
-			.arg(getUserSettingsDir())
-			.arg(QString(Settings["workingdir"]))
+			.arg(nativeSeparator(getPrefix()))
+			.arg(nativeSeparator(getUserSettingsDir()))
+			.arg(nativeSeparator(QString(Settings["workingdir"])))
 			.arg(portable_app() ? " (Portable)" : "")
 			.arg(rows.join("</tr><tr>"));
 
@@ -164,27 +140,3 @@ void MainWindow::about()
 	about->exec();
 	delete about;
 }
-
-void MainWindow::help()
-{
-	QDialog *h = new QDialog(this, 0);
-	QString path, uri;
-	Ui::Help ui;
-	ui.setupUi(h);
-
-	path = QString("file://");
-#if defined(Q_OS_WIN32)
-	path += "/";
-#endif
-	path += getDocDir() + "/";
-#if defined(Q_OS_WIN32)
-	path = path.replace("\\","/");
-#endif
-	uri = path + "xca.html";
-
-	ui.textbox->setSource(QUrl(uri));
-	ui.textbox->setSearchPaths(QStringList(path));
-	h->setWindowTitle(XCA_TITLE);
-	h->show();
-}
-
