@@ -7,6 +7,7 @@
 
 
 #include "MainWindow.h"
+#include "XcaApplication.h"
 #include "PwDialog.h"
 #include "Options.h"
 #include "lib/load_obj.h"
@@ -16,14 +17,17 @@
 #include "lib/pki_scard.h"
 #include "lib/func.h"
 #include "lib/db_x509super.h"
+#include "lib/database_model.h"
 #include "ui_Options.h"
 #include "hashBox.h"
 #include "OidResolver.h"
 #include "OpenDb.h"
+#include "Help.h"
 #include <QApplication>
 #include <QClipboard>
 #include <QMenuBar>
 #include <QMessageBox>
+#include <QFileDialog>
 
 static QAction *languageMenuEntry(const QStringList &sl)
 {
@@ -46,10 +50,10 @@ static QAction *languageMenuEntry(const QStringList &sl)
 	QAction *a = new QAction(lang, NULL);
 	a->setToolTip(tooltip);
 	a->setData(QVariant(locale));
-	a->setDisabled(!XCA_application::languageAvailable(locale));
+	a->setDisabled(!XcaApplication::languageAvailable(locale));
 
 	a->setCheckable(true);
-	if (locale == XCA_application::language())
+	if (locale == XcaApplication::language())
 		a->setChecked(true);
 	return a;
 }
@@ -61,14 +65,14 @@ void MainWindow::init_menu()
 	static QActionGroup * langGroup = NULL;
 	QAction *a;
 
-	if (file) delete file;
-	if (help) delete help;
-	if (import) delete import;
-	if (token) delete token;
-	if (extra) delete extra;
-	if (languageMenu) delete languageMenu;
-	if (historyMenu) delete historyMenu;
-	if (langGroup) delete langGroup;
+	delete file;
+	delete help;
+	delete import;
+	delete token;
+	delete extra;
+	delete languageMenu;
+	delete historyMenu;
+	delete langGroup;
 
 	wdMenuList.clear();
 	scardList.clear();
@@ -77,6 +81,8 @@ void MainWindow::init_menu()
 	langGroup = new QActionGroup(this);
 
 	historyMenu = new tipMenu(tr("Recent DataBases") + " ...", this);
+	update_history_menu();
+
 	connect(historyMenu, SIGNAL(triggered(QAction*)),
                 this, SLOT(open_database(QAction*)));
 
@@ -162,7 +168,7 @@ void MainWindow::init_menu()
 	extra->addAction(tr("OID Resolver"), resolver, SLOT(show()));
 
 	help = menuBar()->addMenu(tr("&Help") );
-	help->addAction(tr("Content"), this, SLOT(help()),
+	help->addAction(tr("Content"), helpdlg, SLOT(content()),
 			QKeySequence::HelpContents);
 	a = new QAction(tr("About"), this);
 	connect(a, SIGNAL(triggered()), this, SLOT(about()));
@@ -172,39 +178,32 @@ void MainWindow::init_menu()
 	wdMenuList += import;
 	scardList += token;
 
-	setItemEnabled(!currentDB.isEmpty());
-}
-
-int MainWindow::changeDB(QString fname)
-{
-	if (fname.isEmpty())
-		return 1;
-	close_database();
-	if (!OpenDb::isRemoteDB(fname))
-		homedir = QFileInfo(fname).canonicalPath();
-	return init_database(fname);
+	setItemEnabled(Database.isOpen());
 }
 
 void MainWindow::update_history_menu()
 {
+	QStringList hist = history.get();
+	if (!historyMenu)
+		return;
 	historyMenu->clear();
-	for (int i = 0, j = 0; i < history.size(); i++) {
+	for (int i = 0, j = 0; i < hist.size(); i++) {
 		QAction *a;
-		QString txt = history[i];
-		if (!QFile::exists(txt) && !OpenDb::isRemoteDB(txt))
+		QString txt = hist[i];
+		if (!QFile::exists(txt) && !database_model::isRemoteDB(txt))
 			continue;
 		txt = QFileInfo(txt).fileName();
 		if (txt.size() > 20)
 			txt = QString("...") + txt.mid(txt.size() - 20);
 		a = historyMenu->addAction(QString("%1 %2").arg(j++).arg(txt));
-		a->setData(QVariant(history[i]));
-		a->setToolTip(history[i]);
+		a->setData(QVariant(hist[i]));
+		a->setToolTip(hist[i]);
 	}
 }
 
 void MainWindow::open_database(QAction* a)
 {
-	changeDB(a->data().toString());
+	init_database(a->data().toString());
 }
 
 void MainWindow::new_database()
@@ -217,7 +216,7 @@ void MainWindow::new_database()
 	// in Qt's OS X file open dialog,
 	// the filename actually ends with that extension.
 	// Otherwise usability breaks in jarring ways.
-	changeDB(getFullFilename(fname, selectedFilter));
+	init_database(getFullFilename(fname, selectedFilter));
 }
 
 void MainWindow::load_database()
@@ -225,7 +224,7 @@ void MainWindow::load_database()
 	load_db l;
 	QString fname = QFileDialog::getOpenFileName(this, l.caption, homedir,
 			l.filter);
-	changeDB(fname);
+	init_database(fname);
 }
 
 void MainWindow::setOptions()
@@ -240,5 +239,6 @@ void MainWindow::setOptions()
 	}
 	delete opt;
 
-	load_engine();
+	pkcs11::libraries.load(Settings["pkcs11path"]);
+	enableTokenMenu(pkcs11::libraries.loaded());
 }
