@@ -8,19 +8,14 @@
 
 //#define MDEBUG
 #include "MainWindow.h"
+#include "XcaApplication.h"
 #include "ImportMulti.h"
 #include "dhgen.h"
-#include "lib/Passwd.h"
-#include "lib/entropy.h"
-
-#include <openssl/rand.h>
 
 #include <QApplication>
 #include <QClipboard>
-#include <QMessageBox>
+#include <QFileDialog>
 #include <QLabel>
-#include <QPushButton>
-#include <QListView>
 #include <QLineEdit>
 #include <QTextBrowser>
 #include <QStatusBar>
@@ -30,25 +25,19 @@
 #include <QMimeData>
 #include <QInputDialog>
 
+#include <openssl/err.h>
+
+#include "lib/Passwd.h"
+#include "lib/database_model.h"
 #include "lib/exception.h"
 #include "lib/pki_evp.h"
-#include "lib/pki_scard.h"
-#include "lib/pki_pkcs12.h"
 #include "lib/pki_multi.h"
-#include "lib/load_obj.h"
-#include "lib/pass_info.h"
-#include "lib/func.h"
-#include "lib/pkcs11.h"
-#include "lib/builtin_curves.h"
+#include "lib/pki_scard.h"
 #include "XcaDialog.h"
+#include "XcaWarning.h"
 #include "PwDialog.h"
 #include "OpenDb.h"
-
-db_key *MainWindow::keys = NULL;
-db_x509req *MainWindow::reqs = NULL;
-db_x509	*MainWindow::certs = NULL;
-db_temp	*MainWindow::temps = NULL;
-db_crl	*MainWindow::crls = NULL;
+#include "OidResolver.h"
 
 OidResolver *MainWindow::resolver = NULL;
 
@@ -59,12 +48,6 @@ void MainWindow::enableTokenMenu(bool enable)
 	}
 }
 
-void MainWindow::load_engine()
-{
-	pkcs11::libraries.load(Settings["pkcs11path"]);
-	enableTokenMenu(pkcs11::libraries.loaded());
-}
-
 void MainWindow::initResolver()
 {
 	bool shown = false;
@@ -81,8 +64,7 @@ void MainWindow::initResolver()
 		resolver->searchOid(search);
 }
 
-MainWindow::MainWindow(QWidget *parent)
-	:QMainWindow(parent)
+MainWindow::MainWindow() : QMainWindow()
 {
 	dbindex = new QLabel();
 	dbindex->setFrameStyle(QFrame::Plain | QFrame::NoFrame);
@@ -94,6 +76,7 @@ MainWindow::MainWindow(QWidget *parent)
 	setupUi(this);
 	setWindowTitle(XCA_TITLE);
 
+	OpenDb::checkSqLite();
 	initResolver();
 
 	wdList << keyButtons << reqButtons << certButtons <<
@@ -101,11 +84,12 @@ MainWindow::MainWindow(QWidget *parent)
 
 	QStringList drivers = QSqlDatabase::drivers();
 	foreach(QString driver, drivers) {
-//		QSqlDatabase d = QSqlDatabase::addDatabase(driver, driver +"_C");
+		QSqlDatabase d = QSqlDatabase::addDatabase(driver, driver +"_C");
 		qDebug() << "DB driver:" << driver;
 	}
 
 	historyMenu = NULL;
+	helpdlg = new Help();
 	init_menu();
 	setItemEnabled(false);
 
@@ -133,19 +117,17 @@ MainWindow::MainWindow(QWidget *parent)
 	searchEdit = new QLineEdit();
 	searchEdit->setPlaceholderText(tr("Search"));
 
-	keyView->setMainwin(this, searchEdit);
-	reqView->setMainwin(this, searchEdit);
-	certView->setMainwin(this, searchEdit);
-	tempView->setMainwin(this, searchEdit);
-	crlView->setMainwin(this, searchEdit);
-	keys = NULL; reqs = NULL; certs = NULL; temps = NULL; crls = NULL;
-
 	keyView->setIconSize(QPixmap(":keyIco").size());
 	reqView->setIconSize(QPixmap(":reqIco").size());
 	certView->setIconSize(QPixmap(":validcertIco").size());
 	tempView->setIconSize(QPixmap(":templateIco").size());
 	crlView->setIconSize(QPixmap(":crlIco").size());
 
+	views << keyView << reqView << certView << crlView << tempView;
+
+	foreach(XcaTreeView *v, views)
+		v->setMainwin(this, searchEdit);
+
 	dhgen = NULL;
 	dhgenBar = new QProgressBar();
 	check_oom(dhgenBar);
@@ -180,21 +162,19 @@ void MainWindow::openURLs(QStringList &files)
 
 void MainWindow::openURLs()
 {
-	QStringList failed;
-	QString s;
-	ImportMulti *dlgi = new ImportMulti(this);
-
-	foreach(s, urlsToOpen) {
-		int ret;
-		pki_multi *pki = probeAnything(s, &ret);
-		if (ret)
-			failed << s;
-		else
-			dlgi->addItem(pki);
+	foreach(QString file, urlsToOpen) {
+		if (file.endsWith(".xdb") ||
+		    !database_model::splitRemoteDbName(file).isEmpty())
+		{
+			init_database(file);
+			if (Database.isOpen()) {
+				urlsToOpen.removeAll(file);
+				break;
+			}
+		}
 	}
+	importAnything(urlsToOpen);
 	urlsToOpen.clear();
-	dlgi->execute(1, failed);
-	delete dlgi;
 }
 
 void MainWindow::dragEnterEvent(QDragEnterEvent *event)
@@ -230,89 +210,10 @@ void MainWindow::init_images()
 	setWindowIcon(QPixmap(":appIco"));
 }
 
-void MainWindow::read_cmdline(int argc, char *argv[])
-{
-	int cnt = 1, opt = 0, force_load = 0, export_index = 0;
-	char *arg = NULL;
-	exitApp = 0;
-	QStringList failed;
-	ImportMulti *dlgi = new ImportMulti(this);
-	while (cnt < argc) {
-		arg = argv[cnt];
-		if (arg[0] == '-') { // option
-			opt = 1;
-			switch (arg[1]) {
-				case 'c':
-				case 'r':
-				case 'k':
-				case 'p':
-				case '7':
-				case 'l':
-				case 't':
-				case 'P':
-					break;
-				case 'd':
-					force_load=1;
-					break;
-				case 'i':
-					export_index=1;
-					break;
-				case 'I':
-					export_index=2;
-					break;
-				case 'v':
-					cmd_version();
-					opt=0;
-					break;
-				case 'x':
-					exitApp = 1;
-					opt=0;
-					break;
-				case 'h':
-					cmd_help(NULL);
-					opt=0;
-					break;
-				default:
-					 cmd_help(CCHAR(tr("no such option: %1").arg(arg)));
-			}
-			if (arg[2] != '\0' && opt==1) {
-				 arg+=2;
-			} else {
-				cnt++;
-				continue;
-			}
-		}
-		QString file = filename2QString(arg);
-		if (force_load) {
-			if (changeDB(file) == 2)
-				exitApp = 1;
-			force_load = 0;
-		} else if (export_index) {
-			if (exportIndex(file, (export_index == 2)) == 2)
-				exitApp = 1;
-			export_index = 0;
-		} else {
-			int ret;
-			pki_multi *pki = probeAnything(file, &ret);
-			if (!pki) {
-				if (ret == 2)
-					exitApp = 1;
-				else if (ret == 1)
-					failed << file;
-			}
-			dlgi->addItem(pki);
-		}
-		cnt++;
-	}
-	dlgi->execute(1, failed); /* force showing of import dialog */
-	if (dlgi->result() == QDialog::Rejected)
-		exitApp = 1;
-	delete dlgi;
-}
-
 void MainWindow::loadPem()
 {
 	load_pem l;
+	db_key *keys = Database.model<db_key>();
 	if (keys)
 		keys->load_default(l);
 }
@@ -325,24 +226,17 @@ bool MainWindow::pastePem(QString text, bool silent)
 		return false;
 
 	pki_multi *pem = NULL;
-	ImportMulti *dlgi = NULL;
 	try {
 		pem = new pki_multi();
-		dlgi = new ImportMulti(this);
 		pem->fromPEMbyteArray(pemdata, QString());
-		success = pem->count() != 0;
-		dlgi->addItem(pem);
-		pem = NULL;
-		dlgi->execute(1);
+		success = pem->failed_files.count() == 0;
+		importMulti(pem, 1);
 	}
 	catch (errorEx &err) {
-		if (!silent)
-			Error(err);
-	}
-	if (dlgi)
-		delete dlgi;
-	if (pem)
 		delete pem;
+		if (!silent)
+			XCA_ERROR(err);
+	}
 	return success;
 }
 
@@ -411,7 +305,7 @@ void MainWindow::initToken()
 			return;
 		p11.initToken(slot, pin.constUchar(), pin.size(), label);
 	} catch (errorEx &err) {
-		Error(err);
+		XCA_ERROR(err);
         }
 }
 
@@ -427,7 +321,7 @@ void MainWindow::changePin(bool so)
 			return;
 		p11.changePin(slot, so);
 	} catch (errorEx &err) {
-		Error(err);
+		XCA_ERROR(err);
         }
 }
 
@@ -448,7 +342,7 @@ void MainWindow::initPin()
 			return;
 		p11.initPin(slot);
 	} catch (errorEx &err) {
-		Error(err);
+		XCA_ERROR(err);
         }
 }
 
@@ -490,7 +384,7 @@ void MainWindow::manageToken()
 				card->setMech_list(ml);
 				dlgi->addItem(card);
 			} catch (errorEx &err) {
-				Error(err);
+				XCA_ERROR(err);
 				delete card;
 			}
 			card = NULL;
@@ -506,7 +400,7 @@ void MainWindow::manageToken()
 				cert->load_token(p11, objects[j]);
 				dlgi->addItem(cert);
 			} catch (errorEx &err) {
-				Error(err);
+				XCA_ERROR(err);
 				delete cert;
 			}
 			cert = NULL;
@@ -518,14 +412,11 @@ void MainWindow::manageToken()
 			dlgi->execute(true);
 		}
 	} catch (errorEx &err) {
-		Error(err);
+		XCA_ERROR(err);
         }
-	if (card)
-		delete card;
-	if (cert)
-		delete cert;
-	if (dlgi)
-		delete dlgi;
+	delete card;
+	delete cert;
+	delete dlgi;
 }
 
 MainWindow::~MainWindow()
@@ -534,6 +425,7 @@ MainWindow::~MainWindow()
 	EVP_cleanup();
 	OBJ_cleanup();
 	delete dbindex;
+	delete helpdlg;
 #ifdef MDEBUG
 	fprintf(stderr, "Memdebug:\n");
 	CRYPTO_mem_leaks_fp(stderr);
@@ -549,24 +441,14 @@ void MainWindow::closeEvent(QCloseEvent *e)
 		}
 		dhgen->terminate();
 	}
-	if (resolver) {
-		delete resolver;
-	}
+	delete resolver;
+	resolver = NULL;
+	delete helpdlg;
+	helpdlg = NULL;
 	close_database();
 	QMainWindow::closeEvent(e);
 }
 
-QString makeSalt(void)
-{
-	QString s = "T";
-	unsigned char rand[8];
-
-	Entropy::get(rand, sizeof rand);
-	for (unsigned i=0; i< sizeof rand; i++)
-		s += QString("%1").arg(rand[i]);
-	return s;
-}
-
 int MainWindow::checkOldGetNewPass(Passwd &pass)
 {
 	QString passHash = Settings["pwhash"];
@@ -603,16 +485,16 @@ void MainWindow::changeDbPass()
 	if (!checkOldGetNewPass(pass))
 		return;
 
-	QString salt = makeSalt();
+	QString salt = Entropy::makeSalt();
 	QString passhash = pki_evp::sha512passwT(pass, salt);
-	QList<pki_evp*> key_list = keys->sqlSELECTpki<pki_evp>(
+	QList<pki_evp*> key_list = Store.sqlSELECTpki<pki_evp>(
 		"SELECT item FROM private_keys WHERE ownPass=0");
 
 	try {
 		Transaction;
 		if (!TransBegin()) {
 			errorEx e(tr("Transaction start failed"));
-			Error(e);
+			XCA_ERROR(e);
 			return;
 		}
 		foreach(pki_evp *key, key_list) {
@@ -626,152 +508,147 @@ void MainWindow::changeDbPass()
 		pki_evp::passHash = passhash;
 		pki_evp::passwd = pass;
 	} catch (errorEx &e) {
-		Error(e);
+		XCA_ERROR(e);
 	}
 }
 
-int MainWindow::initPass(QString dbName)
-{
-	QString passhash = Settings["pwhash"];
-	return initPass(dbName, passhash);
-}
-
-static void pwhash_upgrade()
-{
-	/* Start automatic update from sha512 to sha512*8000
-	 * if the password is correct. The old sha512 hash does
-	 * start with 'S', while the new hash starts with T. */
-
-	/* Start automatic update from md5 to salted sha512*8000
-	 * if the password is correct. The md5 hash does not
-	 * start with 'S' or 'T, but with a hex-digit */
-	if (pki_evp::passHash.startsWith("T")) {
-		/* Fine, current hash function used. */
-		return;
-	}
-	if (pki_evp::sha512passwd(pki_evp::passwd,
-				pki_evp::passHash) == pki_evp::passHash ||
-	    pki_evp::md5passwd(pki_evp::passwd) == pki_evp::passHash)
-	{
-		QString salt = makeSalt();
-		pki_evp::passHash = pki_evp::sha512passwT(
-				pki_evp::passwd, salt);
-	}
-}
-
-int MainWindow::initPass(QString dbName, QString passhash)
-{
-	pki_evp::passHash = QString();
-	QString salt, pass;
-	int ret;
-
-	pass_info p(tr("New Password"), tr("Please enter a password, "
-			"that will be used to encrypt your private keys "
-			"in the database:\n%1").
-			arg(compressFilename(dbName)), this);
-
-	pki_evp::passHash = passhash;
-	if (pki_evp::passHash.isEmpty()) {
-		ret = PwDialog::execute(&p, &pki_evp::passwd, true, true);
-		if (ret != 1)
-			return ret;
-		salt = makeSalt();
-		pki_evp::passHash =pki_evp::sha512passwT(pki_evp::passwd,salt);
-		Settings["pwhash"] = pki_evp::passHash;
-	} else {
-		pwhash_upgrade();
-		ret = 0;
-		while (pki_evp::sha512passwT(pki_evp::passwd, pki_evp::passHash)
-				!= pki_evp::passHash)
-		{
-			if (ret)
-				XCA_WARN(
-				tr("Password verify error, please try again"));
-			p.setTitle(tr("Password"));
-			p.setDescription(tr("Please enter the password for unlocking the database:\n%1").arg(compressFilename(dbName)));
-			ret = PwDialog::execute(&p, &pki_evp::passwd,
-						false, true);
-			if (ret != 1) {
-				pki_evp::passwd = QByteArray();
-				return ret;
-			}
-			pwhash_upgrade();
-		}
-	}
-	if (pki_evp::passwd.isNull())
-		pki_evp::passwd = "";
-	return 1;
-}
-
-void MainWindow::Error(const errorEx &err)
-{
-	if (err.isEmpty())
-		 return;
-	QString msg =  tr("The following error occurred:") + "\n" + err.getString();
-	xcaWarning box(NULL, msg);
-	box.addButton(QMessageBox::Apply)->setText(tr("Copy to Clipboard"));
-	box.addButton(QMessageBox::Ok);
-	if (box.exec() == QMessageBox::Apply) {
-		QClipboard *cb = QApplication::clipboard();
-		cb->setText(msg);
-		if (cb->supportsSelection())
-			cb->setText(msg, QClipboard::Selection);
-	}
-}
-
-void MainWindow::connNewX509(NewX509 *nx)
-{
-	connect(nx, SIGNAL(genKey(QString)),
-		keys, SLOT(newItem(QString)));
-	connect(keys, SIGNAL(keyDone(pki_key*)),
-		nx, SLOT(newKeyDone(pki_key*)));
-	connect(nx, SIGNAL(showReq(pki_base*)),
-		reqs, SLOT(showPki(pki_base*)));
-	connect(reqs, SIGNAL(pkiChanged(pki_base*)),
-		nx, SLOT(itemChanged(pki_base*)));
-}
-
 void MainWindow::importAnything(QString file)
 {
-	int ret;
+	importAnything(QStringList(file));
+}
+
+void MainWindow::importAnything(const QStringList &files)
+{
+	pki_multi *multi = new pki_multi();
+
+	foreach(QString s, files)
+		multi->probeAnything(s);
+
+	importMulti(multi, 1);
+}
+
+void MainWindow::importMulti(pki_multi *multi, int force)
+{
+	if (!multi)
+		return;
+
+	QStringList failed_files = multi->failed_files;
 	ImportMulti *dlgi = new ImportMulti(this);
-	QStringList failed;
-	pki_multi *pki = probeAnything(file, &ret);
-	if (ret)
-		failed << file;
-	else
-		dlgi->addItem(pki);
-	dlgi->execute(1, failed);
+
+	// dlgi->addItem() deletes "multi" if appropriate
+	dlgi->addItem(multi);
+	dlgi->execute(force, failed_files);
 	delete dlgi;
 }
 
-pki_multi *MainWindow::probeAnything(QString file, int *ret)
+void MainWindow::openRemoteSqlDB()
 {
-	if (ret)
-		*ret = 0;
-	pki_multi *pki = NULL;
+	OpenDb *opendb = new OpenDb(this, QString());
+	QString descriptor;
+	Passwd pass;
+	DbMap params;
 
+	if (opendb->exec()) {
+		descriptor = opendb->getDescriptor();
+		pass = opendb->dbPassword->text().toLatin1();
+		params = database_model::splitRemoteDbName(descriptor);
+	}
+	delete opendb;
+
+	if (descriptor.isEmpty())
+		return;
+
+	init_database(descriptor, pass);
+}
+
+enum open_result MainWindow::init_database(const QString &name,
+					   const Passwd &pass)
+{
+	close_database();
 	try {
-		if (file.endsWith(".xdb") ||
-		    !OpenDb::splitRemoteDbName(file).isEmpty())
-		{
-			int r = init_database(file);
-			if (ret)
-				*ret = r;
-			return pki;
-		}
-		pki = new pki_multi();
-		pki->probeAnything(file);
+		Database.open(name, pass);
+		return setup_open_database();
 	} catch (errorEx &err) {
-		Error(err);
+		XCA_ERROR(err);
+		return open_abort;
+	} catch (enum open_result r) {
+		return r;
 	}
-	if (pki && !pki->count()) {
-		delete pki;
-		pki = NULL;
+	return pw_ok;
+}
+
+enum open_result MainWindow::setup_open_database()
+{
+	if (!Database.isOpen())
+		return open_abort;
+
+	if (!database_model::isRemoteDB(Database.name()))
+		homedir = QFileInfo(Database.name()).canonicalPath();
+
+	setItemEnabled(true);
+	dbindex->setText(tr("Database") + ": " +
+			 compressFilename(Database.name()));
+	set_geometry(Settings["mw_geometry"]);
+
+	if (pki_evp::passwd.isNull())
+		XCA_INFO(tr("Using or exporting private keys will not be possible without providing the correct password"));
+
+	enableTokenMenu(pkcs11::libraries.loaded());
+
+	hashBox hb(this);
+	if (hb.isInsecure()) {
+		XCA_WARN(tr("The currently used default hash '%1' is insecure. Please select at least 'SHA 224' for security reasons.").arg(hb.currentHashName()));
+		setOptions();
 	}
-	if (!pki && ret)
-		*ret = 1;
-	return pki;
+
+	keyView->setModel(Database.model<db_key>());
+	reqView->setModel(Database.model<db_x509req>());
+	certView->setModel(Database.model<db_x509>());
+	tempView->setModel(Database.model<db_temp>());
+	crlView->setModel(Database.model<db_crl>());
+
+	searchEdit->setText("");
+	searchEdit->show();
+	statusBar()->addWidget(searchEdit, 1);
+
+	connect(tempView, SIGNAL(newCert(pki_temp *)),
+		Database.model<db_x509>(), SLOT(newCert(pki_temp *)));
+	connect(tempView, SIGNAL(newReq(pki_temp *)),
+		Database.model<db_x509req>(), SLOT(newItem(pki_temp *)));
+
+	return pw_ok;
+}
+
+void MainWindow::set_geometry(QString geo)
+{
+	QStringList sl = geo.split(",");
+	if (sl.size() != 3)
+		return;
+	resize(sl[0].toInt(), sl[1].toInt());
+	int i = sl[2].toInt();
+	if (i != -1)
+		tabView->setCurrentIndex(i);
+}
+
+void MainWindow::close_database()
+{
+	if (!Database.isOpen())
+		return;
+
+	Settings["mw_geometry"] = QString("%1,%2,%3")
+			.arg(size().width())
+			.arg(size().height())
+			.arg(tabView->currentIndex());
+
+	history.addEntry(Database.name());
+	foreach(XcaTreeView *v, views)
+		v->setModel(NULL);
+	Database.close();
+
+	setItemEnabled(false);
+	dbindex->clear();
+	update_history_menu();
+	enableTokenMenu(pkcs11::libraries.loaded());
 }
 
 void MainWindow::exportIndex()
@@ -789,18 +666,13 @@ void MainWindow::exportIndexHierarchy()
 		this, XCA_TITLE, Settings["workingdir"]), true);
 }
 
-int MainWindow::exportIndex(QString fname, bool hierarchy)
+void MainWindow::exportIndex(const QString &fname, bool hierarchy) const
 {
 	qDebug() << fname << hierarchy;
-	if (fname.isEmpty())
-		return 1;
-	if (certs == NULL) {
-		open_default_db();
-		if (certs == NULL)
-			return 2;
-	}
+	if (fname.isEmpty() || !Database.isOpen())
+		return;
+	db_x509 *certs = Database.model<db_x509>();
 	certs->writeIndex(fname, hierarchy);
-	return 0;
 }
 
 void MainWindow::generateDHparamDone()
@@ -811,7 +683,7 @@ void MainWindow::generateDHparamDone()
 		XCA_INFO(tr("Diffie-Hellman parameters saved as: %1")
 			.arg(dhgen->filename()));
 	else
-		Error(e);
+		XCA_ERROR(e);
 	dhgen->deleteLater();
 	dhgen = NULL;
 }
@@ -850,7 +722,7 @@ void MainWindow::generateDHparam()
 		connect(dhgen, SIGNAL(finished()),
 			this, SLOT(generateDHparamDone()));
 	} catch (errorEx &err) {
-		Error(err);
+		XCA_ERROR(err);
 	}
 }
 
@@ -860,12 +732,12 @@ void MainWindow::changeEvent(QEvent *event)
 		retranslateUi(this);
 		dn_translations_setup();
 		init_menu();
-		update_history_menu();
-		foreach(db_base *model, models)
+		foreach(db_base *model, Database.getModels())
 			model->updateHeaders();
 
-		if (!currentDB.isEmpty())
-			dbindex->setText(tr("Database") + ": " + currentDB);
+		if (Database.isOpen())
+			dbindex->setText(tr("Database") + ": " +
+					Database.name());
 		searchEdit->setPlaceholderText(tr("Search"));
 	}
 	QMainWindow::changeEvent(event);
@@ -877,16 +749,15 @@ void MainWindow::keyPressEvent(QKeyEvent *e)
 		QMainWindow::keyPressEvent(e);
 		return;
 	}
-	int siz = XCA_application::tableFont.pointSize();
-	QList<XcaTreeView*> views;
+	int siz = XcaApplication::tableFont.pointSize();
 
 	switch (e->key()) {
 	case Qt::Key_Plus:
-		XCA_application::tableFont.setPointSize(siz +1);
+		XcaApplication::tableFont.setPointSize(siz +1);
 		break;
 	case Qt::Key_Minus:
 		if (siz > 4) {
-			XCA_application::tableFont.setPointSize(siz -1);
+			XcaApplication::tableFont.setPointSize(siz -1);
 		}
 		break;
 	case Qt::Key_V:
@@ -894,12 +765,11 @@ void MainWindow::keyPressEvent(QKeyEvent *e)
 			pastePem();
 			break;
 		}
-        /* FALLTHROUGH */
+		/* FALLTHROUGH */
 	default:
 		QMainWindow::keyPressEvent(e);
 		return;
 	}
-	views << keyView << reqView << certView << crlView << tempView;
 	foreach(XcaTreeView *v, views) {
 		if (v) {
 			v->header()->resizeSections(
@@ -909,3 +779,19 @@ void MainWindow::keyPressEvent(QKeyEvent *e)
 	}
 	update();
 }
+
+void MainWindow::dump_database()
+{
+	QString dirname = QFileDialog::getExistingDirectory(
+				NULL, XCA_TITLE, Settings["workingdir"]);
+	try {
+		Database.dump(dirname);
+	} catch (errorEx &err) {
+		XCA_ERROR(err);
+	}
+}
+
+void MainWindow::default_database()
+{
+	Database.as_default();
+}
