@@ -9,7 +9,7 @@
 #define __NEWCRL_H
 
 #include "ui_NewCrl.h"
-#include "lib/pki_x509.h"
+#include "lib/pki_crl.h"
 
 class pki_key;
 
@@ -17,8 +17,12 @@ class NewCrl: public QWidget, public Ui::NewCrl
 {
 	Q_OBJECT
 
+	crljob task;
    public:
-	NewCrl(QWidget *parent, pki_x509 *signer);
+	NewCrl(const crljob &task, QWidget *w = nullptr);
+	~NewCrl();
+	crljob getCrlJob() const;
+	static void newCrl(QWidget *parent, pki_x509 *issuer);
 
    public slots:
 	void on_applyTime_clicked();
