@@ -10,6 +10,7 @@
 
 #include "ui_NewKey.h"
 #include "lib/pkcs11_lib.h"
+#include "lib/pki_key.h"
 #include "lib/builtin_curves.h"
 #include <QStringList>
 
@@ -17,23 +18,16 @@ class NewKey: public QDialog, public Ui::NewKey
 {
 	Q_OBJECT
 	private:
-		static int defaultType;
-		static int defaultEcNid;
-		static int defaultSize;
 		void updateCurves(unsigned min=0, unsigned max=INT_MAX,
 			unsigned long ec_flags=0);
 		void addCurveBoxCurves(const QList<builtin_curve> &curves);
 	public:
-		NewKey(QWidget *parent, QString name);
-		int getKeytype();
-		int getKeysize();
-		int getKeyCurve_nid();
-		slotid getKeyCardSlot();
-		bool isToken();
-		QString getAsString();
-		static int setDefault(QString def);
-	public slots:
-		void on_keyType_currentIndexChanged(int);
+		static keyjob defaultjob;
+		NewKey(QWidget *parent, const QString &name);
+		keyjob getKeyJob() const;
 
+	public slots:
+		void accept();
+		void on_keyType_currentIndexChanged(int);
 };
 #endif
