@@ -19,7 +19,13 @@
 #include <QValidator>
 #include <QMap>
 #include <QPair>
+
+#include "XcaWarning.h"
+#include "NewKey.h"
+#include "CertDetail.h"
+#include "OidResolver.h"
 #include "MainWindow.h"
+#include "Help.h"
 #include "v3ext.h"
 #include "lib/x509name.h"
 #include "lib/db_key.h"
@@ -101,15 +107,17 @@ QList<nameEdit> NewX509::setupExplicitInputs(NIDlist nid_list,
 	return edits;
 }
 
-NewX509::NewX509(QWidget *parent)
-	:QDialog(parent)
+NewX509::NewX509(QWidget *w) : QDialog(w ?: mainwin)
 {
 	int i;
+	QStringList keys;
+	db_key *keymodel = Database.model<db_key>();
+	db_x509req *reqmodel = Database.model<db_x509req>();
 
 	attr_nid << NID_pkcs9_unstructuredName << NID_pkcs9_challengePassword;
-	QStringList keys;
 
 	setupUi(this);
+	mainwin->helpdlg->register_ctxhelp_button(this, "wizard");
 
 	/* temporary storage for creating temporary X509V3_CTX */
 	ctx_cert = NULL;
@@ -117,6 +125,10 @@ NewX509::NewX509(QWidget *parent)
 	foreach(int nid, distname_nid)
 		keys << QString(OBJ_nid2ln(nid));
 
+	tabnames = QStringList({
+		"wizard_src", "wizard_subject", "wizard_extensions",
+		"wizard_keyusage", "wizard_netscape", "wizard_advanced",
+		"comment"});
 	extDNlist->setKeys(keys);
 	extDNlist->setInfoLabel(extDNinfo);
 	connect(extDNlist->itemDelegateForColumn(1),
@@ -130,11 +142,16 @@ NewX509::NewX509(QWidget *parent)
                 this, SLOT(checkCrlDist(const QString &)));
 	connect(authInfAcc, SIGNAL(textChanged(const QString &)),
                 this, SLOT(checkAuthInfAcc(const QString &)));
+	connect(keymodel, SIGNAL(keyDone(pki_key*)),
+		this, SLOT(newKeyDone(pki_key*)));
+	connect(reqmodel, SIGNAL(pkiChanged(pki_base*)),
+		this, SLOT(itemChanged(pki_base*)));
 
 	setWindowTitle(XCA_TITLE);
 
 	for (i=0; i<tabWidget->count(); i++) {
-		tabnames << tabWidget->tabText(i);
+		tabWidget->widget(i)->setObjectName(tabnames[i]);
+		qDebug() << "TAB:" << i << tabWidget->tabText(i);
 	}
 
 	nsImg->setPixmap(QPixmap(":nsImg"));
@@ -143,18 +160,17 @@ NewX509::NewX509(QWidget *parent)
 	newKeyDone(NULL);
 
 	// any PKCS#10 requests to be used ?
-	QList<pki_x509req *> requests = MainWindow::reqs->getAllRequests();
+	QList<pki_x509req *> requests = getAllRequests();
 	if (requests.isEmpty()) {
 		fromReqCB->setDisabled(true);
 		fromReqCB->setChecked(false);
-	}
-	else {
+	} else {
 		reqList->insertPkiItems(requests);
 	}
 	on_fromReqCB_clicked();
 
 	// How about signing certificates ?
-	QList<pki_x509*> issuers = MainWindow::certs->getAllIssuers();
+	QList<pki_x509*> issuers = getAllIssuers();
 	if (issuers.isEmpty()) {
 		foreignSignRB->setDisabled(true);
 	} else {
@@ -167,7 +183,7 @@ NewX509::NewX509(QWidget *parent)
 	on_applyTime_clicked();
 
 	// settings for the templates ....
-	tempList->insertPkiItems(MainWindow::temps->getAllAndPredefs());
+	tempList->insertPkiItems(getAllTempsAndPredefs());
 
 	// setup Extended keyusage
 	foreach(int nid, extkeyuse_nid)
@@ -299,8 +315,7 @@ void NewX509::setRequest()
 
 NewX509::~NewX509()
 {
-	if (ctx_cert)
-		delete(ctx_cert);
+	delete ctx_cert;
 }
 
 void NewX509::setupExtDNwidget(const QString &s, QLineEdit *l)
@@ -432,7 +447,7 @@ pki_temp *NewX509::caTemplate(pki_x509 *ca) const
 	QVariant sqlId = ca->getTemplateSqlId();
 	if (!sqlId.isValid())
 		return NULL;
-	return MainWindow::temps->lookupPki<pki_temp>(sqlId);
+	return Store.lookupPki<pki_temp>(sqlId);
 }
 
 /* Preset the signing certificate */
@@ -455,26 +470,18 @@ void NewX509::defineSigner(pki_x509 *defcert, bool applyTemp)
 static int lb2int(QListWidget *lb)
 {
 	int i, x=0, c=lb->count();
-	QListWidgetItem *item;
 
 	for (i=0; i<c; i++) {
-		item = lb->item(i);
-		if (lb->isItemSelected(item)){
+		if (lb->item(i)->isSelected())
 			x |= 1<<i;
-		}
 	}
 	return x;
 }
 
 static void int2lb(QListWidget *lb, int x)
 {
-	int i, c=lb->count();
-	QListWidgetItem *item;
-
-	for (i=0; i<c; i++) {
-		item = lb->item(i);
-		lb->setItemSelected(item, (1<<i) & x);
-	}
+	for (int i=0; i<lb->count(); i++)
+		lb->item(i)->setSelected((1<<i) & x);
 }
 
 static void QString2lb(QListWidget *lb, QString x)
@@ -486,7 +493,7 @@ static void QString2lb(QListWidget *lb, QString x)
 		QString lname = OBJ_sn2ln(CCHAR(li[i].trimmed()));
 		items = lb->findItems(lname, Qt::MatchExactly);
 		if (items.size() > 0)
-			lb->setItemSelected(items[0], 1);
+			items[0]->setSelected(true);
 	}
 }
 
@@ -496,9 +503,8 @@ static QString lb2QString(QListWidget *lb)
 
 	for (int i=0; i<lb->count(); i++) {
 		QListWidgetItem *item = lb->item(i);
-		if (lb->isItemSelected(item)) {
+		if (item->isSelected())
 			sl << QString(OBJ_ln2sn(CCHAR(item->text())));
-		}
 	}
 	return sl.join(", ");
 }
@@ -640,12 +646,38 @@ void NewX509::switchHashAlgo()
 
 void NewX509::on_showReqBut_clicked()
 {
-	emit showReq(reqList->currentPkiItem());
+	CertDetail::showCert(this, reqList->currentPkiItem());
+}
+
+QList<pki_x509req *> NewX509::getAllRequests() const
+{
+	return Database.model<db_x509req>()->getAllRequests();
+}
+
+QList<pki_x509*> NewX509::getAllIssuers() const
+{
+	return Database.model<db_x509>()->getAllIssuers();
+}
+
+QList<pki_temp*> NewX509::getAllTempsAndPredefs() const
+{
+	return Database.model<db_temp>()->getPredefs() +
+			Store.getAll<pki_temp>();
+}
+
+QList<pki_key*> NewX509::getAllKeys() const
+{
+	return Database.model<db_key>()->getAllKeys();
+}
+
+QList<pki_key*> NewX509::getUnusedKeys() const
+{
+	return Database.model<db_key>()->getUnusedKeys();
 }
 
 void NewX509::itemChanged(pki_base* req)
 {
-	reqList->insertPkiItems(MainWindow::reqs->getAllRequests());
+	reqList->insertPkiItems(getAllRequests());
 	reqList->setCurrentPkiItem(dynamic_cast<pki_x509req*>(req));
 }
 
@@ -654,7 +686,13 @@ void NewX509::on_genKeyBut_clicked()
 	QString name = description->text();
 	if (name.isEmpty())
 		name = getX509name().getMostPopular();
-	emit genKey(name);
+
+	NewKey *dlg = new NewKey(this, name);
+	if (dlg->exec()) {
+		db_key *keys = Database.model<db_key>();
+		keys->newKey(dlg->getKeyJob(), dlg->keyDesc->text());
+	}
+	delete dlg;
 }
 
 void NewX509::on_certList_currentIndexChanged(int)
@@ -741,8 +779,8 @@ void NewX509::on_foreignSignRB_toggled(bool)
 
 void NewX509::newKeyDone(pki_key *nkey)
 {
-	allKeys =   MainWindow::keys->getAllKeys();
-	unusedKeys= MainWindow::keys->getUnusedKeys();
+	allKeys =   getAllKeys();
+	unusedKeys = getUnusedKeys();
 	on_usedKeysToo_toggled(true);
 	if (nkey) {
 		selfComment(tr("New key '%1' created")
@@ -852,8 +890,7 @@ void NewX509::setupTmpCtx()
 	QString errtxt;
 
 	// initially create temporary ctx cert
-	if (ctx_cert)
-		delete ctx_cert;
+	delete ctx_cert;
 	ctx_cert = new pki_x509();
 	ctx_cert->setSubject(getX509name());
 	if (fromReqCB->isChecked()) {
@@ -885,7 +922,7 @@ void NewX509::editV3ext(QLineEdit *le, QString types, int n)
 	setupTmpCtx();
 	dlg->addInfo(le, types.split(','), n, &ext_ctx);
 	dlg->exec();
-	delete(dlg);
+	delete dlg;
 }
 
 void NewX509::on_adv_validate_clicked()
@@ -1080,8 +1117,10 @@ void NewX509::on_editAuthInfAcc_clicked()
 
 void NewX509::on_tabWidget_currentChanged(int tab)
 {
-	if (tabWidget->tabText(tab) == tabnames[5])
+	QString tab_name = tabWidget->widget(tab)->objectName();
+	if (tab_name == tabnames[5])
 		do_validateExtensions();
+	buttonBox->setProperty("help_ctx", QVariant(tab_name));
 }
 
 QString NewX509::mandatoryDnRemain()
@@ -1133,7 +1172,7 @@ void NewX509::accept()
 		gotoTab(1);
 		xcaWarning msg(this, err.getString());
 		msg.addButton(QMessageBox::Ok);
-		msg.addButton(QMessageBox::Close)->setText(tr("Abort rollout"));
+		msg.addButton(QMessageBox::Close, tr("Abort rollout"));
 		if (msg.exec() == QMessageBox::Close) {
 			reject();
 		}
@@ -1145,9 +1184,9 @@ void NewX509::accept()
 		lenErr = tr("The following length restrictions of RFC3280 are violated:") +
 			"\n" + lenErr;
 		xcaWarning msg(this, lenErr);
-		msg.addButton(QMessageBox::Ok)->setText(tr("Edit subject"));
-		msg.addButton(QMessageBox::Close)->setText(tr("Abort rollout"));
-		msg.addButton(QMessageBox::Apply)->setText(tr("Continue rollout"));
+		msg.addButton(QMessageBox::Ok, tr("Edit subject"));
+		msg.addButton(QMessageBox::Close, tr("Abort rollout"));
+		msg.addButton(QMessageBox::Apply, tr("Continue rollout"));
 		switch (msg.exec())
 		{
 			case QMessageBox::Ok:
@@ -1164,8 +1203,8 @@ void NewX509::accept()
 		gotoTab(0);
 		xcaWarning msg(this,
 			tr("The verification of the Certificate request failed.\nThe rollout should be aborted."));
-		msg.addButton(QMessageBox::Ok)->setText(tr("Continue anyway"));
-		msg.addButton(QMessageBox::Close)->setText(tr("Abort rollout"));
+		msg.addButton(QMessageBox::Ok, tr("Continue anyway"));
+		msg.addButton(QMessageBox::Close, tr("Abort rollout"));
 		if (msg.exec() == QMessageBox::Close) {
 			reject();
 		}
@@ -1176,8 +1215,8 @@ void NewX509::accept()
 			gotoTab(1);
 			xcaWarning msg(this,
 				tr("The internal name and the common name are empty.\nPlease set at least the internal name."));
-			msg.addButton(QMessageBox::Ok)->setText(tr("Edit name"));
-			msg.addButton(QMessageBox::Close)->setText(tr("Abort rollout"));
+			msg.addButton(QMessageBox::Ok, tr("Edit name"));
+			msg.addButton(QMessageBox::Close, tr("Abort rollout"));
 			if (msg.exec() == QMessageBox::Close) {
 				reject();
 			}
@@ -1192,8 +1231,8 @@ void NewX509::accept()
 		gotoTab(1);
 		xcaWarning msg(this,
 			tr("There is no Key selected for signing."));
-		msg.addButton(QMessageBox::Ok)->setText(tr("Select key"));
-		msg.addButton(QMessageBox::Close)->setText(tr("Abort rollout"));
+		msg.addButton(QMessageBox::Ok, tr("Select key"));
+		msg.addButton(QMessageBox::Close, tr("Abort rollout"));
 		if (msg.exec() == QMessageBox::Close) {
 			reject();
 		}
@@ -1206,9 +1245,9 @@ void NewX509::accept()
 		gotoTab(1);
 		QString text = tr("The following distinguished name entries are empty:\n%1\nthough you have declared them as mandatory in the options menu.").arg(unsetDN);
 		xcaWarning msg(this, text);
-		msg.addButton(QMessageBox::Ok)->setText(tr("Edit subject"));
-		msg.addButton(QMessageBox::Close)->setText(tr("Abort rollout"));
-		msg.addButton(QMessageBox::Apply)->setText(tr("Continue rollout"));
+		msg.addButton(QMessageBox::Ok, tr("Edit subject"));
+		msg.addButton(QMessageBox::Close, tr("Abort rollout"));
+		msg.addButton(QMessageBox::Apply, tr("Continue rollout"));
 		switch (msg.exec())
 		{
 			case QMessageBox::Ok:
@@ -1240,21 +1279,29 @@ void NewX509::accept()
 		xcaWarning msg(this,
 			tr("The key you selected for signing is not a private one."));
 		txt = signer ? tr("Select other signer"):tr("Select other key");
-		msg.addButton(QMessageBox::Ok)->setText(txt);
-		msg.addButton(QMessageBox::Close)->setText(tr("Abort rollout"));
+		msg.addButton(QMessageBox::Ok, txt);
+		msg.addButton(QMessageBox::Close, tr("Abort rollout"));
 		if (msg.exec() == QMessageBox::Close) {
 			reject();
 		}
 		return;
         }
+	if (hashAlgo->count() > 0 && hashAlgo->isInsecure()) {
+		gotoTab(0);
+		xcaWarning msg(this, tr("The currently selected hash algorithm '%1' is insecure and should not be used.").arg(hashAlgo->currentHashName()));
+		msg.addButton(QMessageBox::Ok, tr("Select other algorithm"));
+		msg.addButton(QMessageBox::Yes, tr("Use algorithm anyway"));
+		if (msg.exec() == QMessageBox::Ok)
+			return;
+	}
 	if (signer && notBefore->getDate() < signer->getNotBefore()) {
 		gotoTab(2);
 		QString text = tr("The certificate will be earlier valid than the signer. This is probably not what you want.");
 		xcaWarning msg(this, text);
-		msg.addButton(QMessageBox::Ok)->setText(tr("Edit dates"));
-		msg.addButton(QMessageBox::Close)->setText(tr("Abort rollout"));
-		msg.addButton(QMessageBox::Apply)->setText(tr("Continue rollout"));
-		msg.addButton(QMessageBox::Yes)->setText(tr("Adjust date and continue"));
+		msg.addButton(QMessageBox::Ok, tr("Edit dates"));
+		msg.addButton(QMessageBox::Close, tr("Abort rollout"));
+		msg.addButton(QMessageBox::Apply, tr("Continue rollout"));
+		msg.addButton(QMessageBox::Yes, tr("Adjust date and continue"));
 		switch (msg.exec())
 		{
 			case QMessageBox::Ok:
@@ -1274,10 +1321,10 @@ void NewX509::accept()
 		gotoTab(2);
 		QString text = tr("The certificate will be longer valid than the signer. This is probably not what you want.");
 		xcaWarning msg(this, text);
-		msg.addButton(QMessageBox::Ok)->setText(tr("Edit dates"));
-		msg.addButton(QMessageBox::Close)->setText(tr("Abort rollout"));
-		msg.addButton(QMessageBox::Apply)->setText(tr("Continue rollout"));
-		msg.addButton(QMessageBox::Yes)->setText(tr("Adjust date and continue"));
+		msg.addButton(QMessageBox::Ok, tr("Edit dates"));
+		msg.addButton(QMessageBox::Close, tr("Abort rollout"));
+		msg.addButton(QMessageBox::Apply, tr("Continue rollout"));
+		msg.addButton(QMessageBox::Yes, tr("Adjust date and continue"));
 		switch (msg.exec())
 		{
 			case QMessageBox::Ok:
@@ -1297,9 +1344,9 @@ void NewX509::accept()
 		gotoTab(2);
 		QString text = tr("The certificate will be out of date before it becomes valid. You most probably mixed up both dates.");
 		xcaWarning msg(this, text);
-		msg.addButton(QMessageBox::Ok)->setText(tr("Edit dates"));
-		msg.addButton(QMessageBox::Close)->setText(tr("Abort rollout"));
-		msg.addButton(QMessageBox::Apply)->setText(tr("Continue rollout"));
+		msg.addButton(QMessageBox::Ok, tr("Edit dates"));
+		msg.addButton(QMessageBox::Close, tr("Abort rollout"));
+		msg.addButton(QMessageBox::Apply, tr("Continue rollout"));
 		switch (msg.exec())
 		{
 			case QMessageBox::Ok:
@@ -1323,9 +1370,9 @@ void NewX509::accept()
 			gotoTab(0);
 		}
 		xcaWarning msg(this, text);
-		msg.addButton(QMessageBox::Ok)->setText(tr("Edit extensions"));
-		msg.addButton(QMessageBox::Close)->setText(tr("Abort rollout"));
-		msg.addButton(QMessageBox::Apply)->setText(tr("Continue rollout"));
+		msg.addButton(QMessageBox::Ok, tr("Edit extensions"));
+		msg.addButton(QMessageBox::Close, tr("Abort rollout"));
+		msg.addButton(QMessageBox::Apply, tr("Continue rollout"));
 		switch (msg.exec())
 		{
 			case QMessageBox::Ok:
@@ -1343,9 +1390,9 @@ void NewX509::accept()
 	if (cn.isEmpty() && san.contains("DNS:copycn") && pt != tmpl) {
 		gotoTab(2);
 		xcaWarning msg(this, tr("The subject alternative name shall contain a copy of the common name. However, the common name is empty."));
-		msg.addButton(QMessageBox::Ok)->setText(tr("Edit extensions"));
-		msg.addButton(QMessageBox::Close)->setText(tr("Abort rollout"));
-		msg.addButton(QMessageBox::Apply)->setText(tr("Continue rollout"));
+		msg.addButton(QMessageBox::Ok, tr("Edit extensions"));
+		msg.addButton(QMessageBox::Close, tr("Abort rollout"));
+		msg.addButton(QMessageBox::Apply, tr("Continue rollout"));
 		switch (msg.exec())
 		{
 			case QMessageBox::Ok:
