@@ -16,6 +16,7 @@
 
 #include "MainWindow.h"
 #include "lib/x509v3ext.h"
+#include "lib/BioByteArray.h"
 #include "lib/func.h"
 
 #include "lib/openssl_compat.h"
@@ -87,9 +88,8 @@ x509v3ext NewX509::getKeyUsage()
 
 	int rows = keyUsage->count();
 	for (int i=0; i<rows; i++) {
-		if (keyUsage->isItemSelected(keyUsage->item(i))) {
+		if (keyUsage->item(i)->isSelected())
 			cont << keyusage[i];
-		}
 	}
 	if (kuCritical->isChecked() && cont.count() > 0)
 		cont.prepend("critical");
@@ -104,9 +104,8 @@ x509v3ext NewX509::getEkeyUsage()
 
 	int rows = ekeyUsage->count();
 	for (int i=0; i<rows; i++) {
-		if (ekeyUsage->isItemSelected(ekeyUsage->item(i))) {
+		if (ekeyUsage->item(i)->isSelected())
 			cont << QString(OBJ_nid2sn(extkeyuse_nid[i]));
-		}
 	}
 	if (ekuCritical->isChecked() && cont.count() > 0)
 		cont.prepend("critical");
@@ -168,7 +167,6 @@ extList NewX509::getAdvanced()
 {
 	QString conf_str;
 	CONF *conf;
-	BIO *bio;
 	extList elist;
 	long err_line=0;
 	STACK_OF(X509_EXTENSION) **sk, *sk_tmp = NULL;
@@ -184,14 +182,10 @@ extList NewX509::getAdvanced()
 	if (conf_str.isEmpty())
 		return elist;
 
-	QByteArray cs = conf_str.toLatin1();
-	bio = BIO_from_QByteArray(cs);
-	if (!bio)
-		return elist;
 	conf = NCONF_new(NULL);
-	ret = NCONF_load_bio(conf, bio, &err_line);
+	ret = NCONF_load_bio(conf, BioByteArray(conf_str.toLatin1()).ro(),
+				&err_line);
 	if (ret != 1) {
-		BIO_free(bio);
 		openssl_error(tr("Configfile error on line %1\n").
 				arg(err_line));
 		return elist;
@@ -224,7 +218,6 @@ extList NewX509::getAdvanced()
 
 	X509V3_set_nconf(&ext_ctx, NULL);
 	NCONF_free(conf);
-	BIO_free(bio);
 	openssl_error();
 	return elist;
 }
@@ -270,9 +263,8 @@ extList NewX509::getNetscapeExt()
 
 	int rows = nsCertType->count();
 	for (int i=0; i<rows; i++) {
-		if (nsCertType->isItemSelected(nsCertType->item(i))) {
-			cont <<  certTypeList[i];
-		}
+		if (nsCertType->item(i)->isSelected())
+			cont << certTypeList[i];
 	}
 
 	el << ext.create(NID_netscape_cert_type, cont.join(", "), &ext_ctx);
