@@ -8,12 +8,12 @@
 #ifndef __OPENDB_H
 #define __OPENDB_H
 
+#include "lib/database_model.h"
+
 #include "ui_OpenDb.h"
 #include <QDialog>
 #include <QSqlDatabase>
 
-typedef QMap<QString, QString> DbMap;
-
 class OpenDb: public QDialog, public Ui::OpenDb
 {
 	Q_OBJECT
@@ -21,20 +21,16 @@ class OpenDb: public QDialog, public Ui::OpenDb
 	static DbMap getDatabases();
 	static QString lastRemote;
 	bool sqlite, show_connection_settings;
-	bool _openDatabase(const QString &connName, const QString &pass) const;
 	void setupDatabaseName(const QString &db);
 	QString getDbType() const;
 	void fillDbDropDown(const QString &current);
 
     public:
 	OpenDb(QWidget *parent, const QString &db);
-	void openDatabase() const;
 	QString getDescriptor() const;
 	static bool hasSqLite();
 	static void checkSqLite();
 	static bool hasRemoteDrivers();
-	static bool isRemoteDB(const QString &db);
-	static DbMap splitRemoteDbName(const QString &db);
 	static void setLastRemote(const QString &db);
 
     public slots:
