@@ -8,10 +8,12 @@
 #include "lib/func.h"
 #include "Options.h"
 #include "SearchPkcs11.h"
+#include "XcaWarning.h"
+#include "Help.h"
 #include "lib/pki_scard.h"
 #include "lib/oid.h"
 #include <openssl/objects.h>
-#include <QMessageBox>
+#include <QFileDialog>
 #include <QToolTip>
 
 Options::Options(QWidget *parent)
@@ -19,6 +21,7 @@ Options::Options(QWidget *parent)
 {
 	setWindowTitle(XCA_TITLE);
 	setupUi(this);
+	mainwin->helpdlg->register_ctxhelp_button(this, "options");
 
 	foreach(int nid, distname_nid) {
 		QString n = OBJ_nid2ln(nid);
@@ -74,8 +77,7 @@ Options::Options(QWidget *parent)
 
 Options::~Options()
 {
-	if (searchP11)
-		delete searchP11;
+	delete searchP11;
 }
 
 void Options::on_extDNadd_clicked()
@@ -186,7 +188,7 @@ void Options::on_removeButton_clicked(void)
 		indexes << i.row();
 
 	/* Delete from highest to lowest index */
-	qSort(indexes.begin(), indexes.end(), qGreater<int>());
+	std::sort(indexes.begin(), indexes.end(), std::greater<int>());
 	foreach(int i, indexes)
 		pkcs11List->model()->removeRow(i);
 }
