@@ -7,6 +7,8 @@
 
 #include "RevocationList.h"
 #include "MainWindow.h"
+#include "NewCrl.h"
+#include "Help.h"
 #include "lib/asn1int.h"
 #include "lib/pki_x509.h"
 
@@ -91,25 +93,24 @@ void RevocationList::setupRevocationView(QTreeWidget *certList,
 	certList->setSelectionMode(QAbstractItemView::ExtendedSelection);
 }
 
-RevocationList::RevocationList(QWidget *w) : QDialog(w)
+RevocationList::RevocationList(QWidget *w) : QDialog(w ?: mainwin)
 {
 	QPushButton *genCrl;
 	setupUi(this);
 	setWindowTitle(XCA_TITLE);
 	image->setPixmap(QPixmap(":revImg"));
+	mainwin->helpdlg->register_ctxhelp_button(this, "crlmanage");
 
 	genCrl = buttonBox->addButton(tr("Generate CRL"),
 				QDialogButtonBox::ActionRole);
-	connect(genCrl, SIGNAL(clicked(void)), this, SLOT(gencrl(void)));
 
-	connect(certList, SIGNAL(doubleClicked(const QModelIndex &)),
-		this, SLOT(on_editRev_clicked(const QModelIndex &)));
+	connect(genCrl, SIGNAL(clicked()), this, SLOT(gencrl()));
 }
 
-void RevocationList::gencrl(void)
+void RevocationList::gencrl()
 {
 	issuer->setRevocations(getRevList());
-	emit genCRL(issuer);
+	NewCrl::newCrl(this, issuer);
 }
 
 void RevocationList::setRevList(const x509revList &rl, pki_x509 *iss)
@@ -124,9 +125,9 @@ const x509revList &RevocationList::getRevList()
 	return revList;
 }
 
-void RevocationList::on_addRev_clicked(void)
+void RevocationList::on_addRev_clicked()
 {
-	Revocation *revoke = new Revocation(this, QModelIndexList());
+	Revocation *revoke = new Revocation(QModelIndexList(), this);
         if (revoke->exec()) {
 		x509rev revit = revoke->getRevocation();
 		revList << revit;
@@ -134,7 +135,7 @@ void RevocationList::on_addRev_clicked(void)
 	}
 }
 
-void RevocationList::on_delRev_clicked(void)
+void RevocationList::on_delRev_clicked()
 {
 	QTreeWidgetItem *current = certList->currentItem();
 	x509rev rev;
@@ -152,7 +153,11 @@ void RevocationList::on_delRev_clicked(void)
 
 void RevocationList::on_editRev_clicked()
 {
-	QTreeWidgetItem *current = certList->currentItem();
+	on_certList_itemDoubleClicked(certList->currentItem());
+}
+
+void RevocationList::on_certList_itemDoubleClicked(QTreeWidgetItem *current)
+{
 	x509rev rev;
 	int idx;
 
@@ -166,7 +171,7 @@ void RevocationList::on_editRev_clicked()
 
 	rev = revList[idx];
 
-	Revocation *revoke = new Revocation(this, QModelIndexList());
+	Revocation *revoke = new Revocation(QModelIndexList(), this);
 	revoke->setRevocation(rev);
         if (revoke->exec()) {
 		a1time a1 = rev.getDate();
@@ -178,10 +183,12 @@ void RevocationList::on_editRev_clicked()
 	delete revoke;
 }
 
-Revocation::Revocation(QWidget *w, QModelIndexList indexes) : QDialog(w)
+Revocation::Revocation(QModelIndexList indexes, QWidget *w) : QDialog(w ?: mainwin)
 {
 	setupUi(this);
 	setWindowTitle(XCA_TITLE);
+	mainwin->helpdlg->register_ctxhelp_button(this, "crlrevocation");
+
 	reason->addItems(x509rev::crlreasons());
 	invalid->setNow();
 
@@ -191,9 +198,9 @@ Revocation::Revocation(QWidget *w, QModelIndexList indexes) : QDialog(w)
 		serial->setText(QString("Batch revocation of %1 Certificates").
 				arg(indexes.size()));
 		foreach(QModelIndex idx, indexes) {
-			pki_x509 *cert = static_cast<pki_x509*>
-				(idx.internalPointer());
-			serials << cert->getSerial();
+			pki_x509 *cert = db_base::fromIndex<pki_x509>(idx);
+			if (cert)
+				serials << cert->getSerial();
 		}
 		std::sort(serials.begin(), serials.end());
 		foreach(a1int a, serials)
@@ -201,8 +208,7 @@ Revocation::Revocation(QWidget *w, QModelIndexList indexes) : QDialog(w)
 		serial->setToolTip(sl.join("\n"));
 		serial->setEnabled(false);
 	} else if (indexes.size() == 1) {
-		pki_x509 *cert = static_cast<pki_x509*>
-				(indexes[0].internalPointer());
+		pki_x509 *cert = db_base::fromIndex<pki_x509>(indexes[0]);
 		serial->setText(cert->getSerial());
 		serial->setEnabled(false);
 	} else {
