@@ -1,6 +1,6 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2006 - 2015 Christian Hohnstaedt.
+ * Copyright (C) 2006 - 2020 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
@@ -14,25 +14,27 @@
 class TempTreeView: public XcaTreeView
 {
 	Q_OBJECT
-	db_temp *temps;
+
+	db_temp *temps() const
+	{
+		return dynamic_cast<db_temp*>(basemodel);
+	}
+
+	bool runTempDlg(pki_temp *temp);
 
     public:
-	TempTreeView(QWidget *parent) : XcaTreeView(parent)
-	{
-		temps = NULL;
-	}
+	TempTreeView(QWidget *parent) : XcaTreeView(parent) { }
 	void fillContextMenu(QMenu *menu, QMenu *subExport,
 			const QModelIndex &index, QModelIndexList indexes);
-	void setModel(QAbstractItemModel *model=NULL)
-	{
-		temps = dynamic_cast <db_temp*> (model);
-		XcaTreeView::setModel(model);
-	}
+	void showPki(pki_base *pki);
+	bool alterTemp(pki_temp *temp);
 
    public slots:
 	void certFromTemp();
 	void reqFromTemp();
 	void duplicateTemp();
+	void newItem();
+
     signals:
 	void newReq(pki_temp *);
 	void newCert(pki_temp *);
