@@ -5,9 +5,12 @@
  * All rights reserved.
  */
 
-#include "lib/pki_x509req.h"
+#include "lib/db_x509super.h"
+#include "lib/database_model.h"
 #include "X509SuperTreeView.h"
+#include "CertDetail.h"
 #include "MainWindow.h"
+
 #include <QAbstractItemModel>
 #include <QAbstractItemView>
 #include <QMenu>
@@ -15,11 +18,11 @@
 void X509SuperTreeView::fillContextMenu(QMenu *menu, QMenu *subExport,
 		const QModelIndex &index, QModelIndexList indexes)
 {
-	pki_x509super *x = static_cast<pki_x509super *>
-				(index.internalPointer());
+	pki_x509super *x = dynamic_cast<pki_x509super*>(
+					db_base::fromIndex(index));
 	transform = NULL;
 
-	if (indexes.size() != 1)
+	if (indexes.size() != 1 || !x)
 		return;
 
 	subExport->addAction(tr("OpenSSL config"), this, SLOT(toOpenssl()));
@@ -33,22 +36,29 @@ void X509SuperTreeView::extractPubkey()
 {
 	QModelIndex idx = currentIndex();
 
-	if (idx.isValid() && x509super)
-		x509super->extractPubkey(idx);
+	if (idx.isValid() && basemodel)
+		x509super()->extractPubkey(idx);
 }
 
 void X509SuperTreeView::toTemplate()
 {
 	QModelIndex idx = currentIndex();
 
-	if (idx.isValid() && x509super)
-		x509super->toTemplate(idx);
+	if (idx.isValid() && basemodel)
+		x509super()->toTemplate(idx);
 }
 
 void X509SuperTreeView::toOpenssl()
 {
 	QModelIndex idx = currentIndex();
 
-	if (idx.isValid() && x509super)
-		x509super->toOpenssl(idx);
+	if (idx.isValid() && basemodel)
+		x509super()->toOpenssl(idx);
+}
+
+
+void X509SuperTreeView::showPki(pki_base *pki)
+{
+	pki_x509super *x = dynamic_cast<pki_x509super *>(pki);
+	CertDetail::showCert(this, x);
 }
