@@ -5,11 +5,6 @@
  * All rights reserved.
  */
 
-
-#include "XcaHeaderView.h"
-#include "XcaTreeView.h"
-#include "XcaProxyModel.h"
-#include "MainWindow.h"
 #include <QAbstractItemModel>
 #include <QAbstractItemView>
 #include <QContextMenuEvent>
@@ -17,6 +12,12 @@
 #include <QVariant>
 #include <QRegExp>
 
+#include "OidResolver.h"
+#include "XcaHeaderView.h"
+#include "XcaTreeView.h"
+#include "XcaProxyModel.h"
+#include "MainWindow.h"
+#include "XcaWarning.h"
 
 XcaTreeView::XcaTreeView(QWidget *parent)
 	:QTreeView(parent)
@@ -93,11 +94,12 @@ void XcaTreeView::setModel(QAbstractItemModel *model)
 {
 	QByteArray ba;
 
-	basemodel = (db_base *)model;
+	basemodel = dynamic_cast<db_base*>(model);
 	proxy->setSourceModel(model);
-	QTreeView::setModel(proxy);
+	QTreeView::setModel(model ? proxy : NULL);
 
 	if (basemodel) {
+		setRootIsDecorated(basemodel->treeViewMode());
 		connect(basemodel, SIGNAL(resetHeader()),
 			header(), SLOT(resetMoves()));
 		connect(basemodel, SIGNAL(resetHeader()),
@@ -202,7 +204,7 @@ void XcaTreeView::setFilter(const QString &pattern)
 	proxy->setFilterFixedString(pattern);
 }
 
-void XcaTreeView::deleteItems(void)
+void XcaTreeView::deleteItems()
 {
 	QModelIndex index;
 	QModelIndexList indexes = getSelectedIndexes();
@@ -214,7 +216,7 @@ void XcaTreeView::deleteItems(void)
 		return;
 
 	foreach(index, indexes) {
-		pki = static_cast<pki_base*>(index.internalPointer());
+		pki = db_base::fromIndex(index);
 		items += "'" + pki->getIntName() + "' ";
 		count++;
 	}
@@ -238,7 +240,7 @@ void XcaTreeView::deleteItems(void)
 	TransCommit();
 }
 
-void XcaTreeView::storeItems(void)
+void XcaTreeView::storeItems()
 {
 	QModelIndexList indexes = getSelectedIndexes();
 	if (basemodel) {
@@ -250,27 +252,25 @@ void XcaTreeView::storeItems(void)
 				return;
 			}
 		} catch (errorEx &err) {
-			MainWindow::Error(err);
+			XCA_ERROR(err);
 		}
 	}
 }
 
-void XcaTreeView::showItems(void)
+void XcaTreeView::showItems()
 {
-	if (basemodel) {
-		QModelIndexList indexes = getSelectedIndexes();
-		foreach(QModelIndex index, indexes)
-			basemodel->showItem(index);
-	}
+	QModelIndexList indexes = getSelectedIndexes();
+	foreach(QModelIndex index, indexes)
+		showItem(index);
 }
 
-void XcaTreeView::newItem(void)
+void XcaTreeView::newItem()
 {
 	if (basemodel)
 		basemodel->newItem();
 }
 
-void XcaTreeView::load(void)
+void XcaTreeView::load()
 {
 	if (basemodel)
 		basemodel->load();
@@ -278,23 +278,22 @@ void XcaTreeView::load(void)
 
 void XcaTreeView::doubleClick(const QModelIndex &m)
 {
-	if (basemodel)
-		basemodel->showItem(getIndex(m));
+	showItem(getIndex(m));
 }
 
-void XcaTreeView::editComment(void)
+void XcaTreeView::editComment()
 {
 	if (basemodel)
 		basemodel->editComment(currentIndex());
 }
 
-void XcaTreeView::pem2clipboard(void)
+void XcaTreeView::pem2clipboard()
 {
 	if (basemodel)
 		basemodel->pem2clipboard(getSelectedIndexes());
 }
 
-void XcaTreeView::headerDetails(void)
+void XcaTreeView::headerDetails()
 {
 	if (curr_hd && curr_hd->id > 0 && mainwin)
 		mainwin->getResolver()->searchOid(QString::number(curr_hd->id));
@@ -306,6 +305,24 @@ void XcaTreeView::columnRemove(void)
 		curr_hd->action->setChecked(false);
 }
 
+void XcaTreeView::showItem(const QModelIndex &index)
+{
+	pki_base *pki = db_base::fromIndex(index);
+	showItem(pki);
+}
+
+void XcaTreeView::showItem(const QString &name)
+{
+	pki_base *pki = Store.lookupPki<pki_base>(name.toULongLong());
+	showItem(pki);
+}
+
+void XcaTreeView::showItem(pki_base *pki)
+{
+	if (pki && pki->isVisible() == 1)
+		showPki(pki);
+}
+
 static void addSubmenu(tipMenu *menu, tipMenu *sub)
 {
 	if (sub->isEmpty())
@@ -389,11 +406,19 @@ void XcaTreeView::contextMenu(QContextMenuEvent *e, QMenu *parent, int col)
 	if (!shown)
 		allHeaders[0]->show = true;
         delete menu;
-	if (parent)
-		delete parent;
+	delete parent;
 	showHideSections();
 }
 
+void XcaTreeView::changeView()
+{
+	if (!basemodel)
+		return;
+	hide();
+	basemodel->changeView();
+	show();
+}
+
 void XcaTreeView::showContextMenu(QContextMenuEvent *e,
 		const QModelIndex &idx)
 {
@@ -426,6 +451,7 @@ void XcaTreeView::showContextMenu(QContextMenuEvent *e,
 
 	contextMenu(e, menu, -1);
 }
+
 void XcaTreeView::keyPressEvent(QKeyEvent *event)
 {
 	switch (event->key()) {
